const version$1 = '1.0.0-b.149';
var version$2 = version$1;

const itkConfig = {
    pipelineWorkerUrl: `https://cdn.jsdelivr.net/npm/itk-wasm@${version$2}/dist/web-workers/min-bundles/pipeline.worker.js`,
    imageIOUrl: `https://cdn.jsdelivr.net/npm/itk-image-io@${version$2}`,
    meshIOUrl: `https://cdn.jsdelivr.net/npm/itk-mesh-io@${version$2}`,
    pipelinesUrl: `https://cdn.jsdelivr.net/npm/itk-wasm@${version$2}/dist/pipelines`
};
var config = itkConfig;

const InterfaceTypes = {
    // Todo: remove Interface prefix after IOTypes has been removed
    TextFile: 'InterfaceTextFile',
    BinaryFile: 'InterfaceBinaryFile',
    TextStream: 'InterfaceTextStream',
    BinaryStream: 'InterfaceBinaryStream',
    Image: 'InterfaceImage',
    Mesh: 'InterfaceMesh',
    PolyData: 'InterfacePolyData',
    JsonCompatible: 'InterfaceJsonCompatible'
};
var InterfaceTypes$1 = InterfaceTypes;

const IntTypes = {
    Int8: 'int8',
    UInt8: 'uint8',
    Int16: 'int16',
    UInt16: 'uint16',
    Int32: 'int32',
    UInt32: 'uint32',
    Int64: 'int64',
    UInt64: 'uint64',
    SizeValueType: 'uint64',
    IdentifierType: 'uint64',
    IndexValueType: 'int64',
    OffsetValueType: 'int64'
};
var IntTypes$1 = IntTypes;

const FloatTypes = {
    Float32: 'float32',
    Float64: 'float64',
    SpacePrecisionType: 'float64'
};
var FloatTypes$1 = FloatTypes;

const IOTypes = {
    Text: 'Text',
    Binary: 'Binary',
    Image: 'Image',
    Mesh: 'Mesh'
};
var IOTypes$1 = IOTypes;

const PixelTypes = {
    Unknown: 'Unknown',
    Scalar: 'Scalar',
    RGB: 'RGB',
    RGBA: 'RGBA',
    Offset: 'Offset',
    Vector: 'Vector',
    Point: 'Point',
    CovariantVector: 'CovariantVector',
    SymmetricSecondRankTensor: 'SymmetricSecondRankTensor',
    DiffusionTensor3D: 'DiffusionTensor3D',
    Complex: 'Complex',
    FixedArray: 'FixedArray',
    Array: 'Array',
    Matrix: 'Matrix',
    VariableLengthVector: 'VariableLengthVector',
    VariableSizeMatrix: 'VariableSizeMatrix'
};
var PixelTypes$1 = PixelTypes;

function setMatrixElement(matrixData, columns, row, column, value) {
    matrixData[column + row * columns] = value;
}

class ImageType {
    constructor(dimension = 2, componentType = IntTypes$1.UInt8, pixelType = PixelTypes$1.Scalar, components = 1) {
        this.dimension = dimension;
        this.componentType = componentType;
        this.pixelType = pixelType;
        this.components = components;
    }
}
var ImageType$1 = ImageType;

class Image {
    constructor(imageType = new ImageType$1()) {
        this.imageType = imageType;
        this.name = 'image';
        const dimension = imageType.dimension;
        this.origin = new Array(dimension);
        this.origin.fill(0.0);
        this.spacing = new Array(dimension);
        this.spacing.fill(1.0);
        this.direction = new Float64Array(dimension * dimension);
        this.direction.fill(0.0);
        for (let ii = 0; ii < dimension; ii++) {
            setMatrixElement(this.direction, dimension, ii, ii, 1.0);
        }
        this.size = new Array(dimension);
        this.size.fill(0);
        this.metadata = new Map();
        this.data = null;
    }
}
var Image$1 = Image;

function bufferToTypedArray(wasmType, buffer) {
    let typedArray = null;
    switch (wasmType) {
        case IntTypes$1.UInt8: {
            typedArray = new Uint8Array(buffer);
            break;
        }
        case IntTypes$1.Int8: {
            typedArray = new Int8Array(buffer);
            break;
        }
        case IntTypes$1.UInt16: {
            typedArray = new Uint16Array(buffer);
            break;
        }
        case IntTypes$1.Int16: {
            typedArray = new Int16Array(buffer);
            break;
        }
        case IntTypes$1.UInt32: {
            typedArray = new Uint32Array(buffer);
            break;
        }
        case IntTypes$1.Int32: {
            typedArray = new Int32Array(buffer);
            break;
        }
        case IntTypes$1.UInt64: {
            if (typeof globalThis.BigUint64Array === 'function') {
                typedArray = new BigUint64Array(buffer);
            }
            else {
                // Sub with reasonable default. Will get cast to Uint8Array when
                // transferred to WebAssembly.
                typedArray = new Uint8Array(buffer);
            }
            break;
        }
        case IntTypes$1.Int64: {
            if (typeof globalThis.BigInt64Array === 'function') {
                typedArray = new BigInt64Array(buffer);
            }
            else {
                // Sub with reasonable default. Will get cast to Uint8Array when
                // transferred to WebAssembly.
                typedArray = new Uint8Array(buffer);
            }
            break;
        }
        case FloatTypes$1.Float32: {
            typedArray = new Float32Array(buffer);
            break;
        }
        case FloatTypes$1.Float64: {
            typedArray = new Float64Array(buffer);
            break;
        }
        case 'null': {
            typedArray = null;
            break;
        }
        case null: {
            typedArray = null;
            break;
        }
        default:
            throw new Error('Type is not supported as a TypedArray');
    }
    return typedArray;
}

function copyImage(image) {
    const copy = new Image$1(image.imageType);
    copy.name = image.name;
    copy.origin = Array.from(image.origin);
    copy.spacing = Array.from(image.spacing);
    copy.direction = image.direction.slice();
    copy.size = Array.from(image.size);
    if (image.data !== null) {
        const CTor = image.data.constructor;
        copy.data = new CTor(image.data.length);
        if (copy.data != null) {
            // @ts-expect-error: error TS2345: Argument of type 'TypedArray' is not assignable to parameter of type 'ArrayLike<number> & ArrayLike<bigint>'
            copy.data.set(image.data, 0);
        }
    }
    return copy;
}

/** Join an array of sequential image slabs into a single image */
function stackImages(images) {
    if (images.length < 1) {
        throw Error('At least one images is required.');
    }
    const firstImage = images[0];
    if (firstImage.data === null) {
        throw Error('Image data is null.');
    }
    const result = new Image$1(firstImage.imageType);
    result.origin = Array.from(firstImage.origin);
    result.spacing = Array.from(firstImage.spacing);
    const dimension = result.imageType.dimension;
    result.direction = firstImage.direction.slice();
    const stackOn = dimension - 1;
    result.size = Array.from(firstImage.size);
    const stackedSize = images.reduce((accumulator, currentValue) => {
        return accumulator + currentValue.size[stackOn];
    }, 0);
    result.size[stackOn] = stackedSize;
    const dataSize = result.size.reduce((accumulator, currentValue) => { return accumulator * currentValue; }, 1) * result.imageType.components;
    const CTor = firstImage.data.constructor;
    result.data = new CTor(dataSize);
    let offsetBase = result.imageType.components;
    for (let subIndex = 0; subIndex < result.size.length - 1; subIndex++) {
        offsetBase *= result.size[subIndex];
    }
    let stackIndex = 0;
    if (result.data != null) {
        for (let index = 0; index < images.length; index++) {
            // @ts-expect-error: error TS2345: Argument of type 'TypedArray' is not assignable to parameter of type 'ArrayLike<number> & ArrayLike<bigint>'.
            result.data.set(images[index].data, offsetBase * stackIndex);
            stackIndex += images[index].size[stackOn];
        }
    }
    else {
        throw Error('Could not create result image data.');
    }
    return result;
}

/**
 * Cast an image to another PixelType and/or ComponentType
 *
 * @param {Image} image - The input image
 * @param {CastImageOptions} options - specify the componentType and/or pixelType of the output
 */
function castImage(inputImage, options) {
    const outputImageType = Object.assign({}, inputImage.imageType);
    if (typeof options !== 'undefined' && typeof options.pixelType !== 'undefined') {
        outputImageType.pixelType = options.pixelType;
        if (options.pixelType === PixelTypes$1.Scalar && outputImageType.components !== 1) {
            throw new Error('Cannot cast multi-component image to a scalar image');
        }
    }
    if (typeof options !== 'undefined' && typeof options.componentType !== 'undefined' && options.componentType !== inputImage.imageType.componentType) {
        outputImageType.componentType = options.componentType;
    }
    const outputImage = new Image$1(outputImageType);
    outputImage.name = inputImage.name;
    outputImage.origin = Array.from(inputImage.origin);
    outputImage.spacing = Array.from(inputImage.spacing);
    outputImage.direction = inputImage.direction.slice();
    outputImage.size = Array.from(inputImage.size);
    // Deep copy the map
    outputImage.metadata = new Map(JSON.parse(JSON.stringify(Array.from(inputImage.metadata))));
    if (inputImage.data !== null) {
        if (typeof options !== 'undefined' && typeof options.componentType !== 'undefined' && options.componentType !== inputImage.imageType.componentType) {
            switch (inputImage.imageType.componentType) {
                case IntTypes$1.UInt8:
                case IntTypes$1.Int8:
                case IntTypes$1.UInt16:
                case IntTypes$1.Int16:
                case IntTypes$1.UInt32:
                case IntTypes$1.Int32:
                case FloatTypes$1.Float32:
                case FloatTypes$1.Float64:
                    switch (outputImage.imageType.componentType) {
                        case IntTypes$1.UInt8:
                            outputImage.data = new Uint8Array(inputImage.data);
                            break;
                        case IntTypes$1.Int8:
                            outputImage.data = new Int8Array(inputImage.data);
                            break;
                        case IntTypes$1.UInt16:
                            outputImage.data = new Uint16Array(inputImage.data);
                            break;
                        case IntTypes$1.Int16:
                            outputImage.data = new Int16Array(inputImage.data);
                            break;
                        case IntTypes$1.UInt32:
                            outputImage.data = new Uint32Array(inputImage.data);
                            break;
                        case IntTypes$1.Int32:
                            outputImage.data = new Int32Array(inputImage.data);
                            break;
                        case FloatTypes$1.Float32:
                            outputImage.data = new Float32Array(inputImage.data);
                            break;
                        case FloatTypes$1.Float64:
                            outputImage.data = new Float64Array(inputImage.data);
                            break;
                        case IntTypes$1.UInt64:
                            outputImage.data = new BigUint64Array(inputImage.data.length);
                            for (let idx = 0; idx < outputImage.data.length; idx++) {
                                outputImage.data[idx] = BigInt.asIntN(64, BigInt(inputImage.data[idx]));
                            }
                            break;
                        case IntTypes$1.Int64:
                            outputImage.data = new BigInt64Array(inputImage.data.length);
                            for (let idx = 0; idx < outputImage.data.length; idx++) {
                                outputImage.data[idx] = BigInt.asUintN(64, BigInt(inputImage.data[idx]));
                            }
                            break;
                    }
                    break;
                case IntTypes$1.UInt64:
                case IntTypes$1.Int64:
                    switch (outputImage.imageType.componentType) {
                        case IntTypes$1.UInt8:
                            outputImage.data = new Uint8Array(inputImage.data.length);
                            for (let idx = 0; idx < outputImage.data.length; idx++) {
                                outputImage.data[idx] = Number(inputImage.data[idx]);
                            }
                            break;
                        case IntTypes$1.Int8:
                            outputImage.data = new Int8Array(inputImage.data.length);
                            for (let idx = 0; idx < outputImage.data.length; idx++) {
                                outputImage.data[idx] = Number(inputImage.data[idx]);
                            }
                            break;
                        case IntTypes$1.UInt16:
                            outputImage.data = new Uint16Array(inputImage.data.length);
                            for (let idx = 0; idx < outputImage.data.length; idx++) {
                                outputImage.data[idx] = Number(inputImage.data[idx]);
                            }
                            break;
                        case IntTypes$1.Int16:
                            outputImage.data = new Int16Array(inputImage.data.length);
                            for (let idx = 0; idx < outputImage.data.length; idx++) {
                                outputImage.data[idx] = Number(inputImage.data[idx]);
                            }
                            break;
                        case IntTypes$1.UInt32:
                            outputImage.data = new Uint32Array(inputImage.data.length);
                            for (let idx = 0; idx < outputImage.data.length; idx++) {
                                outputImage.data[idx] = Number(inputImage.data[idx]);
                            }
                            break;
                        case IntTypes$1.Int32:
                            outputImage.data = new Int32Array(inputImage.data.length);
                            for (let idx = 0; idx < outputImage.data.length; idx++) {
                                outputImage.data[idx] = Number(inputImage.data[idx]);
                            }
                            break;
                        case FloatTypes$1.Float32:
                            outputImage.data = new Float32Array(inputImage.data.length);
                            for (let idx = 0; idx < outputImage.data.length; idx++) {
                                outputImage.data[idx] = Number(inputImage.data[idx]);
                            }
                            break;
                        case FloatTypes$1.Float64:
                            outputImage.data = new Float64Array(inputImage.data.length);
                            for (let idx = 0; idx < outputImage.data.length; idx++) {
                                outputImage.data[idx] = Number(inputImage.data[idx]);
                            }
                            break;
                        case IntTypes$1.UInt64:
                            outputImage.data = new BigUint64Array(inputImage.data);
                            break;
                        case IntTypes$1.Int64:
                            outputImage.data = new BigInt64Array(inputImage.data);
                            break;
                    }
                    break;
            }
        }
        else {
            // copy
            const CTor = inputImage.data.constructor;
            outputImage.data = new CTor(inputImage.data.length);
            if (outputImage.data != null) {
                // @ts-expect-error: error TS2345: Argument of type 'TypedArray' is not assignable to parameter of type 'ArrayLike<number> & ArrayLike<bigint>'
                outputImage.data.set(inputImage.data, 0);
            }
        }
    }
    return outputImage;
}

/* eslint-disable  @typescript-eslint/no-non-null-assertion */
var __rest = (undefined && undefined.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
class WorkerPool {
    /* poolSize is the maximum number of web workers to create in the pool.
     *
     * The function, fcn, should accept null or an existing worker as its first argument.
     * It most also return and object with the used worker on the `webWorker`
     * property.  * Example: runPipeline.
     *
     **/
    constructor(poolSize, fcn) {
        this.fcn = fcn;
        this.workerQueue = new Array(poolSize);
        this.workerQueue.fill(null);
        this.runInfo = [];
    }
    /*
     * Run the tasks specified by the arguments in the taskArgsArray that will
     * be passed to the pool fcn.
     *
     * An optional progressCallback will be called with the number of complete
     * tasks and the total number of tasks as arguments every time a task has
     * completed.
     *
     * Returns an object containing a promise ('promise') to communicate results
     * as well as an id ('runId') which can be used to cancel any remaining pending
     * tasks before they complete.
     */
    runTasks(taskArgsArray, progressCallback = null) {
        const info = {
            taskQueue: [],
            results: [],
            addingTasks: false,
            postponed: false,
            runningWorkers: 0,
            index: 0,
            completedTasks: 0,
            progressCallback: progressCallback,
            canceled: false
        };
        this.runInfo.push(info);
        info.index = this.runInfo.length - 1;
        return {
            promise: new Promise((resolve, reject) => {
                info.resolve = resolve;
                info.reject = reject;
                info.results = new Array(taskArgsArray.length);
                info.completedTasks = 0;
                info.addingTasks = true;
                taskArgsArray.forEach((taskArg, index) => {
                    this.addTask(info.index, index, taskArg);
                });
                info.addingTasks = false;
            }),
            runId: info.index
        };
    }
    terminateWorkers() {
        for (let index = 0; index < this.workerQueue.length; index++) {
            const worker = this.workerQueue[index];
            if (worker != null) {
                worker.terminate();
            }
            this.workerQueue[index] = null;
        }
    }
    cancel(runId) {
        const info = this.runInfo[runId];
        if (info !== null && info !== undefined) {
            info.canceled = true;
        }
    }
    addTask(infoIndex, resultIndex, taskArgs) {
        const info = this.runInfo[infoIndex];
        if ((info === null || info === void 0 ? void 0 : info.canceled) === true) {
            info.reject('Remaining tasks canceled');
            this.clearTask(info.index);
            return;
        }
        if (this.workerQueue.length > 0) {
            const worker = this.workerQueue.pop();
            info.runningWorkers++;
            this.fcn(worker, ...taskArgs).then((_a) => {
                var { webWorker } = _a, result = __rest(_a, ["webWorker"]);
                this.workerQueue.push(webWorker);
                // Check if this task was canceled while it was getting done
                if (this.runInfo[infoIndex] !== null) {
                    info.runningWorkers--;
                    info.results[resultIndex] = result;
                    info.completedTasks++;
                    if (info.progressCallback != null) {
                        info.progressCallback(info.completedTasks, info.results.length);
                    }
                    if (info.taskQueue.length > 0) {
                        const reTask = info.taskQueue.shift();
                        this.addTask(infoIndex, reTask[0], reTask[1]);
                    }
                    else if (!info.addingTasks && info.runningWorkers === 0) {
                        const results = info.results;
                        info.resolve(results);
                        this.clearTask(info.index);
                    }
                }
            }).catch((error) => {
                info.reject(error);
                this.clearTask(info.index);
            });
        }
        else {
            if (info.runningWorkers !== 0 || info.postponed) {
                // At least one worker is working on these tasks, and it will pick up
                // the next item in the taskQueue when done.
                info.taskQueue.push([resultIndex, taskArgs]);
            }
            else {
                // Try again later.
                info.postponed = true;
                setTimeout(() => {
                    info.postponed = false;
                    this.addTask(info.index, resultIndex, taskArgs);
                }, 50);
            }
        }
    }
    clearTask(clearIndex) {
        this.runInfo[clearIndex].results = [];
        this.runInfo[clearIndex].taskQueue = [];
        this.runInfo[clearIndex].progressCallback = null;
        this.runInfo[clearIndex].canceled = null;
        this.runInfo[clearIndex].reject = () => { };
        this.runInfo[clearIndex].resolve = () => { };
    }
}
var WorkerPool$1 = WorkerPool;

const haveSharedArrayBuffer$1 = typeof globalThis.SharedArrayBuffer !== 'undefined'; // eslint-disable-line
function getTransferables(data) {
    if (data === undefined || data === null) {
        return [];
    }
    const transferables = [];
    for (let i = 0; i < data.length; i++) {
        const transferable = getTransferable(data[i]);
        if (transferable !== null) {
            transferables.push(transferable);
        }
    }
    return transferables;
}
function getTransferable(data) {
    if (data === undefined || data === null) {
        return null;
    }
    let result = null;
    if (data.buffer !== undefined) {
        result = data.buffer;
    }
    else if (data.byteLength !== undefined) {
        result = data;
    }
    if (haveSharedArrayBuffer$1 && result instanceof SharedArrayBuffer) { // eslint-disable-line
        return null;
    }
    return result;
}

var global$1 = (typeof global !== "undefined" ? global :
  typeof self !== "undefined" ? self :
  typeof window !== "undefined" ? window : {});

function bind(fn, thisArg) {
  return function wrap() {
    return fn.apply(thisArg, arguments);
  };
}

// utils is a library of generic helper functions non-specific to axios

const {toString: toString$1} = Object.prototype;
const {getPrototypeOf} = Object;

const kindOf = (cache => thing => {
    const str = toString$1.call(thing);
    return cache[str] || (cache[str] = str.slice(8, -1).toLowerCase());
})(Object.create(null));

const kindOfTest = (type) => {
  type = type.toLowerCase();
  return (thing) => kindOf(thing) === type
};

const typeOfTest = type => thing => typeof thing === type;

/**
 * Determine if a value is an Array
 *
 * @param {Object} val The value to test
 *
 * @returns {boolean} True if value is an Array, otherwise false
 */
const {isArray: isArray$1} = Array;

/**
 * Determine if a value is undefined
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if the value is undefined, otherwise false
 */
const isUndefined = typeOfTest('undefined');

/**
 * Determine if a value is a Buffer
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is a Buffer, otherwise false
 */
function isBuffer$1(val) {
  return val !== null && !isUndefined(val) && val.constructor !== null && !isUndefined(val.constructor)
    && isFunction(val.constructor.isBuffer) && val.constructor.isBuffer(val);
}

/**
 * Determine if a value is an ArrayBuffer
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is an ArrayBuffer, otherwise false
 */
const isArrayBuffer = kindOfTest('ArrayBuffer');


/**
 * Determine if a value is a view on an ArrayBuffer
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is a view on an ArrayBuffer, otherwise false
 */
function isArrayBufferView(val) {
  let result;
  if ((typeof ArrayBuffer !== 'undefined') && (ArrayBuffer.isView)) {
    result = ArrayBuffer.isView(val);
  } else {
    result = (val) && (val.buffer) && (isArrayBuffer(val.buffer));
  }
  return result;
}

/**
 * Determine if a value is a String
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is a String, otherwise false
 */
const isString = typeOfTest('string');

/**
 * Determine if a value is a Function
 *
 * @param {*} val The value to test
 * @returns {boolean} True if value is a Function, otherwise false
 */
const isFunction = typeOfTest('function');

/**
 * Determine if a value is a Number
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is a Number, otherwise false
 */
const isNumber = typeOfTest('number');

/**
 * Determine if a value is an Object
 *
 * @param {*} thing The value to test
 *
 * @returns {boolean} True if value is an Object, otherwise false
 */
const isObject = (thing) => thing !== null && typeof thing === 'object';

/**
 * Determine if a value is a Boolean
 *
 * @param {*} thing The value to test
 * @returns {boolean} True if value is a Boolean, otherwise false
 */
const isBoolean = thing => thing === true || thing === false;

/**
 * Determine if a value is a plain Object
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is a plain Object, otherwise false
 */
const isPlainObject = (val) => {
  if (kindOf(val) !== 'object') {
    return false;
  }

  const prototype = getPrototypeOf(val);
  return (prototype === null || prototype === Object.prototype || Object.getPrototypeOf(prototype) === null) && !(Symbol.toStringTag in val) && !(Symbol.iterator in val);
};

/**
 * Determine if a value is a Date
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is a Date, otherwise false
 */
const isDate = kindOfTest('Date');

/**
 * Determine if a value is a File
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is a File, otherwise false
 */
const isFile = kindOfTest('File');

/**
 * Determine if a value is a Blob
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is a Blob, otherwise false
 */
const isBlob = kindOfTest('Blob');

/**
 * Determine if a value is a FileList
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is a File, otherwise false
 */
const isFileList = kindOfTest('FileList');

/**
 * Determine if a value is a Stream
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is a Stream, otherwise false
 */
const isStream = (val) => isObject(val) && isFunction(val.pipe);

/**
 * Determine if a value is a FormData
 *
 * @param {*} thing The value to test
 *
 * @returns {boolean} True if value is an FormData, otherwise false
 */
const isFormData = (thing) => {
  let kind;
  return thing && (
    (typeof FormData === 'function' && thing instanceof FormData) || (
      isFunction(thing.append) && (
        (kind = kindOf(thing)) === 'formdata' ||
        // detect form-data instance
        (kind === 'object' && isFunction(thing.toString) && thing.toString() === '[object FormData]')
      )
    )
  )
};

/**
 * Determine if a value is a URLSearchParams object
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is a URLSearchParams object, otherwise false
 */
const isURLSearchParams = kindOfTest('URLSearchParams');

/**
 * Trim excess whitespace off the beginning and end of a string
 *
 * @param {String} str The String to trim
 *
 * @returns {String} The String freed of excess whitespace
 */
const trim = (str) => str.trim ?
  str.trim() : str.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g, '');

/**
 * Iterate over an Array or an Object invoking a function for each item.
 *
 * If `obj` is an Array callback will be called passing
 * the value, index, and complete array for each item.
 *
 * If 'obj' is an Object callback will be called passing
 * the value, key, and complete object for each property.
 *
 * @param {Object|Array} obj The object to iterate
 * @param {Function} fn The callback to invoke for each item
 *
 * @param {Boolean} [allOwnKeys = false]
 * @returns {any}
 */
function forEach(obj, fn, {allOwnKeys = false} = {}) {
  // Don't bother if no value provided
  if (obj === null || typeof obj === 'undefined') {
    return;
  }

  let i;
  let l;

  // Force an array if not already something iterable
  if (typeof obj !== 'object') {
    /*eslint no-param-reassign:0*/
    obj = [obj];
  }

  if (isArray$1(obj)) {
    // Iterate over array values
    for (i = 0, l = obj.length; i < l; i++) {
      fn.call(null, obj[i], i, obj);
    }
  } else {
    // Iterate over object keys
    const keys = allOwnKeys ? Object.getOwnPropertyNames(obj) : Object.keys(obj);
    const len = keys.length;
    let key;

    for (i = 0; i < len; i++) {
      key = keys[i];
      fn.call(null, obj[key], key, obj);
    }
  }
}

function findKey(obj, key) {
  key = key.toLowerCase();
  const keys = Object.keys(obj);
  let i = keys.length;
  let _key;
  while (i-- > 0) {
    _key = keys[i];
    if (key === _key.toLowerCase()) {
      return _key;
    }
  }
  return null;
}

const _global = (() => {
  /*eslint no-undef:0*/
  if (typeof globalThis !== "undefined") return globalThis;
  return typeof self !== "undefined" ? self : (typeof window !== 'undefined' ? window : global$1)
})();

const isContextDefined = (context) => !isUndefined(context) && context !== _global;

/**
 * Accepts varargs expecting each argument to be an object, then
 * immutably merges the properties of each object and returns result.
 *
 * When multiple objects contain the same key the later object in
 * the arguments list will take precedence.
 *
 * Example:
 *
 * ```js
 * var result = merge({foo: 123}, {foo: 456});
 * console.log(result.foo); // outputs 456
 * ```
 *
 * @param {Object} obj1 Object to merge
 *
 * @returns {Object} Result of all merge properties
 */
function merge(/* obj1, obj2, obj3, ... */) {
  const {caseless} = isContextDefined(this) && this || {};
  const result = {};
  const assignValue = (val, key) => {
    const targetKey = caseless && findKey(result, key) || key;
    if (isPlainObject(result[targetKey]) && isPlainObject(val)) {
      result[targetKey] = merge(result[targetKey], val);
    } else if (isPlainObject(val)) {
      result[targetKey] = merge({}, val);
    } else if (isArray$1(val)) {
      result[targetKey] = val.slice();
    } else {
      result[targetKey] = val;
    }
  };

  for (let i = 0, l = arguments.length; i < l; i++) {
    arguments[i] && forEach(arguments[i], assignValue);
  }
  return result;
}

/**
 * Extends object a by mutably adding to it the properties of object b.
 *
 * @param {Object} a The object to be extended
 * @param {Object} b The object to copy properties from
 * @param {Object} thisArg The object to bind function to
 *
 * @param {Boolean} [allOwnKeys]
 * @returns {Object} The resulting value of object a
 */
const extend = (a, b, thisArg, {allOwnKeys}= {}) => {
  forEach(b, (val, key) => {
    if (thisArg && isFunction(val)) {
      a[key] = bind(val, thisArg);
    } else {
      a[key] = val;
    }
  }, {allOwnKeys});
  return a;
};

/**
 * Remove byte order marker. This catches EF BB BF (the UTF-8 BOM)
 *
 * @param {string} content with BOM
 *
 * @returns {string} content value without BOM
 */
const stripBOM = (content) => {
  if (content.charCodeAt(0) === 0xFEFF) {
    content = content.slice(1);
  }
  return content;
};

/**
 * Inherit the prototype methods from one constructor into another
 * @param {function} constructor
 * @param {function} superConstructor
 * @param {object} [props]
 * @param {object} [descriptors]
 *
 * @returns {void}
 */
const inherits = (constructor, superConstructor, props, descriptors) => {
  constructor.prototype = Object.create(superConstructor.prototype, descriptors);
  constructor.prototype.constructor = constructor;
  Object.defineProperty(constructor, 'super', {
    value: superConstructor.prototype
  });
  props && Object.assign(constructor.prototype, props);
};

/**
 * Resolve object with deep prototype chain to a flat object
 * @param {Object} sourceObj source object
 * @param {Object} [destObj]
 * @param {Function|Boolean} [filter]
 * @param {Function} [propFilter]
 *
 * @returns {Object}
 */
const toFlatObject = (sourceObj, destObj, filter, propFilter) => {
  let props;
  let i;
  let prop;
  const merged = {};

  destObj = destObj || {};
  // eslint-disable-next-line no-eq-null,eqeqeq
  if (sourceObj == null) return destObj;

  do {
    props = Object.getOwnPropertyNames(sourceObj);
    i = props.length;
    while (i-- > 0) {
      prop = props[i];
      if ((!propFilter || propFilter(prop, sourceObj, destObj)) && !merged[prop]) {
        destObj[prop] = sourceObj[prop];
        merged[prop] = true;
      }
    }
    sourceObj = filter !== false && getPrototypeOf(sourceObj);
  } while (sourceObj && (!filter || filter(sourceObj, destObj)) && sourceObj !== Object.prototype);

  return destObj;
};

/**
 * Determines whether a string ends with the characters of a specified string
 *
 * @param {String} str
 * @param {String} searchString
 * @param {Number} [position= 0]
 *
 * @returns {boolean}
 */
const endsWith = (str, searchString, position) => {
  str = String(str);
  if (position === undefined || position > str.length) {
    position = str.length;
  }
  position -= searchString.length;
  const lastIndex = str.indexOf(searchString, position);
  return lastIndex !== -1 && lastIndex === position;
};


/**
 * Returns new array from array like object or null if failed
 *
 * @param {*} [thing]
 *
 * @returns {?Array}
 */
const toArray = (thing) => {
  if (!thing) return null;
  if (isArray$1(thing)) return thing;
  let i = thing.length;
  if (!isNumber(i)) return null;
  const arr = new Array(i);
  while (i-- > 0) {
    arr[i] = thing[i];
  }
  return arr;
};

/**
 * Checking if the Uint8Array exists and if it does, it returns a function that checks if the
 * thing passed in is an instance of Uint8Array
 *
 * @param {TypedArray}
 *
 * @returns {Array}
 */
// eslint-disable-next-line func-names
const isTypedArray = (TypedArray => {
  // eslint-disable-next-line func-names
  return thing => {
    return TypedArray && thing instanceof TypedArray;
  };
})(typeof Uint8Array !== 'undefined' && getPrototypeOf(Uint8Array));

/**
 * For each entry in the object, call the function with the key and value.
 *
 * @param {Object<any, any>} obj - The object to iterate over.
 * @param {Function} fn - The function to call for each entry.
 *
 * @returns {void}
 */
const forEachEntry = (obj, fn) => {
  const generator = obj && obj[Symbol.iterator];

  const iterator = generator.call(obj);

  let result;

  while ((result = iterator.next()) && !result.done) {
    const pair = result.value;
    fn.call(obj, pair[0], pair[1]);
  }
};

/**
 * It takes a regular expression and a string, and returns an array of all the matches
 *
 * @param {string} regExp - The regular expression to match against.
 * @param {string} str - The string to search.
 *
 * @returns {Array<boolean>}
 */
const matchAll = (regExp, str) => {
  let matches;
  const arr = [];

  while ((matches = regExp.exec(str)) !== null) {
    arr.push(matches);
  }

  return arr;
};

/* Checking if the kindOfTest function returns true when passed an HTMLFormElement. */
const isHTMLForm = kindOfTest('HTMLFormElement');

const toCamelCase = str => {
  return str.toLowerCase().replace(/[-_\s]([a-z\d])(\w*)/g,
    function replacer(m, p1, p2) {
      return p1.toUpperCase() + p2;
    }
  );
};

/* Creating a function that will check if an object has a property. */
const hasOwnProperty = (({hasOwnProperty}) => (obj, prop) => hasOwnProperty.call(obj, prop))(Object.prototype);

/**
 * Determine if a value is a RegExp object
 *
 * @param {*} val The value to test
 *
 * @returns {boolean} True if value is a RegExp object, otherwise false
 */
const isRegExp = kindOfTest('RegExp');

const reduceDescriptors = (obj, reducer) => {
  const descriptors = Object.getOwnPropertyDescriptors(obj);
  const reducedDescriptors = {};

  forEach(descriptors, (descriptor, name) => {
    let ret;
    if ((ret = reducer(descriptor, name, obj)) !== false) {
      reducedDescriptors[name] = ret || descriptor;
    }
  });

  Object.defineProperties(obj, reducedDescriptors);
};

/**
 * Makes all methods read-only
 * @param {Object} obj
 */

const freezeMethods = (obj) => {
  reduceDescriptors(obj, (descriptor, name) => {
    // skip restricted props in strict mode
    if (isFunction(obj) && ['arguments', 'caller', 'callee'].indexOf(name) !== -1) {
      return false;
    }

    const value = obj[name];

    if (!isFunction(value)) return;

    descriptor.enumerable = false;

    if ('writable' in descriptor) {
      descriptor.writable = false;
      return;
    }

    if (!descriptor.set) {
      descriptor.set = () => {
        throw Error('Can not rewrite read-only method \'' + name + '\'');
      };
    }
  });
};

const toObjectSet = (arrayOrString, delimiter) => {
  const obj = {};

  const define = (arr) => {
    arr.forEach(value => {
      obj[value] = true;
    });
  };

  isArray$1(arrayOrString) ? define(arrayOrString) : define(String(arrayOrString).split(delimiter));

  return obj;
};

const noop = () => {};

const toFiniteNumber = (value, defaultValue) => {
  value = +value;
  return Number.isFinite(value) ? value : defaultValue;
};

const ALPHA = 'abcdefghijklmnopqrstuvwxyz';

const DIGIT = '0123456789';

const ALPHABET = {
  DIGIT,
  ALPHA,
  ALPHA_DIGIT: ALPHA + ALPHA.toUpperCase() + DIGIT
};

const generateString = (size = 16, alphabet = ALPHABET.ALPHA_DIGIT) => {
  let str = '';
  const {length} = alphabet;
  while (size--) {
    str += alphabet[Math.random() * length|0];
  }

  return str;
};

/**
 * If the thing is a FormData object, return true, otherwise return false.
 *
 * @param {unknown} thing - The thing to check.
 *
 * @returns {boolean}
 */
function isSpecCompliantForm(thing) {
  return !!(thing && isFunction(thing.append) && thing[Symbol.toStringTag] === 'FormData' && thing[Symbol.iterator]);
}

const toJSONObject = (obj) => {
  const stack = new Array(10);

  const visit = (source, i) => {

    if (isObject(source)) {
      if (stack.indexOf(source) >= 0) {
        return;
      }

      if(!('toJSON' in source)) {
        stack[i] = source;
        const target = isArray$1(source) ? [] : {};

        forEach(source, (value, key) => {
          const reducedValue = visit(value, i + 1);
          !isUndefined(reducedValue) && (target[key] = reducedValue);
        });

        stack[i] = undefined;

        return target;
      }
    }

    return source;
  };

  return visit(obj, 0);
};

const isAsyncFn = kindOfTest('AsyncFunction');

const isThenable = (thing) =>
  thing && (isObject(thing) || isFunction(thing)) && isFunction(thing.then) && isFunction(thing.catch);

var utils = {
  isArray: isArray$1,
  isArrayBuffer,
  isBuffer: isBuffer$1,
  isFormData,
  isArrayBufferView,
  isString,
  isNumber,
  isBoolean,
  isObject,
  isPlainObject,
  isUndefined,
  isDate,
  isFile,
  isBlob,
  isRegExp,
  isFunction,
  isStream,
  isURLSearchParams,
  isTypedArray,
  isFileList,
  forEach,
  merge,
  extend,
  trim,
  stripBOM,
  inherits,
  toFlatObject,
  kindOf,
  kindOfTest,
  endsWith,
  toArray,
  forEachEntry,
  matchAll,
  isHTMLForm,
  hasOwnProperty,
  hasOwnProp: hasOwnProperty, // an alias to avoid ESLint no-prototype-builtins detection
  reduceDescriptors,
  freezeMethods,
  toObjectSet,
  toCamelCase,
  noop,
  toFiniteNumber,
  findKey,
  global: _global,
  isContextDefined,
  ALPHABET,
  generateString,
  isSpecCompliantForm,
  toJSONObject,
  isAsyncFn,
  isThenable
};

var lookup = [];
var revLookup = [];
var Arr = typeof Uint8Array !== 'undefined' ? Uint8Array : Array;
var inited = false;
function init () {
  inited = true;
  var code = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
  for (var i = 0, len = code.length; i < len; ++i) {
    lookup[i] = code[i];
    revLookup[code.charCodeAt(i)] = i;
  }

  revLookup['-'.charCodeAt(0)] = 62;
  revLookup['_'.charCodeAt(0)] = 63;
}

function toByteArray (b64) {
  if (!inited) {
    init();
  }
  var i, j, l, tmp, placeHolders, arr;
  var len = b64.length;

  if (len % 4 > 0) {
    throw new Error('Invalid string. Length must be a multiple of 4')
  }

  // the number of equal signs (place holders)
  // if there are two placeholders, than the two characters before it
  // represent one byte
  // if there is only one, then the three characters before it represent 2 bytes
  // this is just a cheap hack to not do indexOf twice
  placeHolders = b64[len - 2] === '=' ? 2 : b64[len - 1] === '=' ? 1 : 0;

  // base64 is 4/3 + up to two characters of the original data
  arr = new Arr(len * 3 / 4 - placeHolders);

  // if there are placeholders, only get up to the last complete 4 chars
  l = placeHolders > 0 ? len - 4 : len;

  var L = 0;

  for (i = 0, j = 0; i < l; i += 4, j += 3) {
    tmp = (revLookup[b64.charCodeAt(i)] << 18) | (revLookup[b64.charCodeAt(i + 1)] << 12) | (revLookup[b64.charCodeAt(i + 2)] << 6) | revLookup[b64.charCodeAt(i + 3)];
    arr[L++] = (tmp >> 16) & 0xFF;
    arr[L++] = (tmp >> 8) & 0xFF;
    arr[L++] = tmp & 0xFF;
  }

  if (placeHolders === 2) {
    tmp = (revLookup[b64.charCodeAt(i)] << 2) | (revLookup[b64.charCodeAt(i + 1)] >> 4);
    arr[L++] = tmp & 0xFF;
  } else if (placeHolders === 1) {
    tmp = (revLookup[b64.charCodeAt(i)] << 10) | (revLookup[b64.charCodeAt(i + 1)] << 4) | (revLookup[b64.charCodeAt(i + 2)] >> 2);
    arr[L++] = (tmp >> 8) & 0xFF;
    arr[L++] = tmp & 0xFF;
  }

  return arr
}

function tripletToBase64 (num) {
  return lookup[num >> 18 & 0x3F] + lookup[num >> 12 & 0x3F] + lookup[num >> 6 & 0x3F] + lookup[num & 0x3F]
}

function encodeChunk (uint8, start, end) {
  var tmp;
  var output = [];
  for (var i = start; i < end; i += 3) {
    tmp = (uint8[i] << 16) + (uint8[i + 1] << 8) + (uint8[i + 2]);
    output.push(tripletToBase64(tmp));
  }
  return output.join('')
}

function fromByteArray (uint8) {
  if (!inited) {
    init();
  }
  var tmp;
  var len = uint8.length;
  var extraBytes = len % 3; // if we have 1 byte left, pad 2 bytes
  var output = '';
  var parts = [];
  var maxChunkLength = 16383; // must be multiple of 3

  // go through the array every three bytes, we'll deal with trailing stuff later
  for (var i = 0, len2 = len - extraBytes; i < len2; i += maxChunkLength) {
    parts.push(encodeChunk(uint8, i, (i + maxChunkLength) > len2 ? len2 : (i + maxChunkLength)));
  }

  // pad the end with zeros, but make sure to not forget the extra bytes
  if (extraBytes === 1) {
    tmp = uint8[len - 1];
    output += lookup[tmp >> 2];
    output += lookup[(tmp << 4) & 0x3F];
    output += '==';
  } else if (extraBytes === 2) {
    tmp = (uint8[len - 2] << 8) + (uint8[len - 1]);
    output += lookup[tmp >> 10];
    output += lookup[(tmp >> 4) & 0x3F];
    output += lookup[(tmp << 2) & 0x3F];
    output += '=';
  }

  parts.push(output);

  return parts.join('')
}

function read (buffer, offset, isLE, mLen, nBytes) {
  var e, m;
  var eLen = nBytes * 8 - mLen - 1;
  var eMax = (1 << eLen) - 1;
  var eBias = eMax >> 1;
  var nBits = -7;
  var i = isLE ? (nBytes - 1) : 0;
  var d = isLE ? -1 : 1;
  var s = buffer[offset + i];

  i += d;

  e = s & ((1 << (-nBits)) - 1);
  s >>= (-nBits);
  nBits += eLen;
  for (; nBits > 0; e = e * 256 + buffer[offset + i], i += d, nBits -= 8) {}

  m = e & ((1 << (-nBits)) - 1);
  e >>= (-nBits);
  nBits += mLen;
  for (; nBits > 0; m = m * 256 + buffer[offset + i], i += d, nBits -= 8) {}

  if (e === 0) {
    e = 1 - eBias;
  } else if (e === eMax) {
    return m ? NaN : ((s ? -1 : 1) * Infinity)
  } else {
    m = m + Math.pow(2, mLen);
    e = e - eBias;
  }
  return (s ? -1 : 1) * m * Math.pow(2, e - mLen)
}

function write (buffer, value, offset, isLE, mLen, nBytes) {
  var e, m, c;
  var eLen = nBytes * 8 - mLen - 1;
  var eMax = (1 << eLen) - 1;
  var eBias = eMax >> 1;
  var rt = (mLen === 23 ? Math.pow(2, -24) - Math.pow(2, -77) : 0);
  var i = isLE ? 0 : (nBytes - 1);
  var d = isLE ? 1 : -1;
  var s = value < 0 || (value === 0 && 1 / value < 0) ? 1 : 0;

  value = Math.abs(value);

  if (isNaN(value) || value === Infinity) {
    m = isNaN(value) ? 1 : 0;
    e = eMax;
  } else {
    e = Math.floor(Math.log(value) / Math.LN2);
    if (value * (c = Math.pow(2, -e)) < 1) {
      e--;
      c *= 2;
    }
    if (e + eBias >= 1) {
      value += rt / c;
    } else {
      value += rt * Math.pow(2, 1 - eBias);
    }
    if (value * c >= 2) {
      e++;
      c /= 2;
    }

    if (e + eBias >= eMax) {
      m = 0;
      e = eMax;
    } else if (e + eBias >= 1) {
      m = (value * c - 1) * Math.pow(2, mLen);
      e = e + eBias;
    } else {
      m = value * Math.pow(2, eBias - 1) * Math.pow(2, mLen);
      e = 0;
    }
  }

  for (; mLen >= 8; buffer[offset + i] = m & 0xff, i += d, m /= 256, mLen -= 8) {}

  e = (e << mLen) | m;
  eLen += mLen;
  for (; eLen > 0; buffer[offset + i] = e & 0xff, i += d, e /= 256, eLen -= 8) {}

  buffer[offset + i - d] |= s * 128;
}

var toString = {}.toString;

var isArray = Array.isArray || function (arr) {
  return toString.call(arr) == '[object Array]';
};

/*!
 * The buffer module from node.js, for the browser.
 *
 * @author   Feross Aboukhadijeh <feross@feross.org> <http://feross.org>
 * @license  MIT
 */

var INSPECT_MAX_BYTES = 50;

/**
 * If `Buffer.TYPED_ARRAY_SUPPORT`:
 *   === true    Use Uint8Array implementation (fastest)
 *   === false   Use Object implementation (most compatible, even IE6)
 *
 * Browsers that support typed arrays are IE 10+, Firefox 4+, Chrome 7+, Safari 5.1+,
 * Opera 11.6+, iOS 4.2+.
 *
 * Due to various browser bugs, sometimes the Object implementation will be used even
 * when the browser supports typed arrays.
 *
 * Note:
 *
 *   - Firefox 4-29 lacks support for adding new properties to `Uint8Array` instances,
 *     See: https://bugzilla.mozilla.org/show_bug.cgi?id=695438.
 *
 *   - Chrome 9-10 is missing the `TypedArray.prototype.subarray` function.
 *
 *   - IE10 has a broken `TypedArray.prototype.subarray` function which returns arrays of
 *     incorrect length in some situations.

 * We detect these buggy browsers and set `Buffer.TYPED_ARRAY_SUPPORT` to `false` so they
 * get the Object implementation, which is slower but behaves correctly.
 */
Buffer.TYPED_ARRAY_SUPPORT = global$1.TYPED_ARRAY_SUPPORT !== undefined
  ? global$1.TYPED_ARRAY_SUPPORT
  : true;

/*
 * Export kMaxLength after typed array support is determined.
 */
kMaxLength();

function kMaxLength () {
  return Buffer.TYPED_ARRAY_SUPPORT
    ? 0x7fffffff
    : 0x3fffffff
}

function createBuffer (that, length) {
  if (kMaxLength() < length) {
    throw new RangeError('Invalid typed array length')
  }
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    // Return an augmented `Uint8Array` instance, for best performance
    that = new Uint8Array(length);
    that.__proto__ = Buffer.prototype;
  } else {
    // Fallback: Return an object instance of the Buffer class
    if (that === null) {
      that = new Buffer(length);
    }
    that.length = length;
  }

  return that
}

/**
 * The Buffer constructor returns instances of `Uint8Array` that have their
 * prototype changed to `Buffer.prototype`. Furthermore, `Buffer` is a subclass of
 * `Uint8Array`, so the returned instances will have all the node `Buffer` methods
 * and the `Uint8Array` methods. Square bracket notation works as expected -- it
 * returns a single octet.
 *
 * The `Uint8Array` prototype remains unmodified.
 */

function Buffer (arg, encodingOrOffset, length) {
  if (!Buffer.TYPED_ARRAY_SUPPORT && !(this instanceof Buffer)) {
    return new Buffer(arg, encodingOrOffset, length)
  }

  // Common case.
  if (typeof arg === 'number') {
    if (typeof encodingOrOffset === 'string') {
      throw new Error(
        'If encoding is specified then the first argument must be a string'
      )
    }
    return allocUnsafe(this, arg)
  }
  return from(this, arg, encodingOrOffset, length)
}

Buffer.poolSize = 8192; // not used by this implementation

// TODO: Legacy, not needed anymore. Remove in next major version.
Buffer._augment = function (arr) {
  arr.__proto__ = Buffer.prototype;
  return arr
};

function from (that, value, encodingOrOffset, length) {
  if (typeof value === 'number') {
    throw new TypeError('"value" argument must not be a number')
  }

  if (typeof ArrayBuffer !== 'undefined' && value instanceof ArrayBuffer) {
    return fromArrayBuffer(that, value, encodingOrOffset, length)
  }

  if (typeof value === 'string') {
    return fromString(that, value, encodingOrOffset)
  }

  return fromObject(that, value)
}

/**
 * Functionally equivalent to Buffer(arg, encoding) but throws a TypeError
 * if value is a number.
 * Buffer.from(str[, encoding])
 * Buffer.from(array)
 * Buffer.from(buffer)
 * Buffer.from(arrayBuffer[, byteOffset[, length]])
 **/
Buffer.from = function (value, encodingOrOffset, length) {
  return from(null, value, encodingOrOffset, length)
};

if (Buffer.TYPED_ARRAY_SUPPORT) {
  Buffer.prototype.__proto__ = Uint8Array.prototype;
  Buffer.__proto__ = Uint8Array;
  if (typeof Symbol !== 'undefined' && Symbol.species &&
      Buffer[Symbol.species] === Buffer) ;
}

function assertSize (size) {
  if (typeof size !== 'number') {
    throw new TypeError('"size" argument must be a number')
  } else if (size < 0) {
    throw new RangeError('"size" argument must not be negative')
  }
}

function alloc (that, size, fill, encoding) {
  assertSize(size);
  if (size <= 0) {
    return createBuffer(that, size)
  }
  if (fill !== undefined) {
    // Only pay attention to encoding if it's a string. This
    // prevents accidentally sending in a number that would
    // be interpretted as a start offset.
    return typeof encoding === 'string'
      ? createBuffer(that, size).fill(fill, encoding)
      : createBuffer(that, size).fill(fill)
  }
  return createBuffer(that, size)
}

/**
 * Creates a new filled Buffer instance.
 * alloc(size[, fill[, encoding]])
 **/
Buffer.alloc = function (size, fill, encoding) {
  return alloc(null, size, fill, encoding)
};

function allocUnsafe (that, size) {
  assertSize(size);
  that = createBuffer(that, size < 0 ? 0 : checked(size) | 0);
  if (!Buffer.TYPED_ARRAY_SUPPORT) {
    for (var i = 0; i < size; ++i) {
      that[i] = 0;
    }
  }
  return that
}

/**
 * Equivalent to Buffer(num), by default creates a non-zero-filled Buffer instance.
 * */
Buffer.allocUnsafe = function (size) {
  return allocUnsafe(null, size)
};
/**
 * Equivalent to SlowBuffer(num), by default creates a non-zero-filled Buffer instance.
 */
Buffer.allocUnsafeSlow = function (size) {
  return allocUnsafe(null, size)
};

function fromString (that, string, encoding) {
  if (typeof encoding !== 'string' || encoding === '') {
    encoding = 'utf8';
  }

  if (!Buffer.isEncoding(encoding)) {
    throw new TypeError('"encoding" must be a valid string encoding')
  }

  var length = byteLength(string, encoding) | 0;
  that = createBuffer(that, length);

  var actual = that.write(string, encoding);

  if (actual !== length) {
    // Writing a hex string, for example, that contains invalid characters will
    // cause everything after the first invalid character to be ignored. (e.g.
    // 'abxxcd' will be treated as 'ab')
    that = that.slice(0, actual);
  }

  return that
}

function fromArrayLike (that, array) {
  var length = array.length < 0 ? 0 : checked(array.length) | 0;
  that = createBuffer(that, length);
  for (var i = 0; i < length; i += 1) {
    that[i] = array[i] & 255;
  }
  return that
}

function fromArrayBuffer (that, array, byteOffset, length) {
  array.byteLength; // this throws if `array` is not a valid ArrayBuffer

  if (byteOffset < 0 || array.byteLength < byteOffset) {
    throw new RangeError('\'offset\' is out of bounds')
  }

  if (array.byteLength < byteOffset + (length || 0)) {
    throw new RangeError('\'length\' is out of bounds')
  }

  if (byteOffset === undefined && length === undefined) {
    array = new Uint8Array(array);
  } else if (length === undefined) {
    array = new Uint8Array(array, byteOffset);
  } else {
    array = new Uint8Array(array, byteOffset, length);
  }

  if (Buffer.TYPED_ARRAY_SUPPORT) {
    // Return an augmented `Uint8Array` instance, for best performance
    that = array;
    that.__proto__ = Buffer.prototype;
  } else {
    // Fallback: Return an object instance of the Buffer class
    that = fromArrayLike(that, array);
  }
  return that
}

function fromObject (that, obj) {
  if (internalIsBuffer(obj)) {
    var len = checked(obj.length) | 0;
    that = createBuffer(that, len);

    if (that.length === 0) {
      return that
    }

    obj.copy(that, 0, 0, len);
    return that
  }

  if (obj) {
    if ((typeof ArrayBuffer !== 'undefined' &&
        obj.buffer instanceof ArrayBuffer) || 'length' in obj) {
      if (typeof obj.length !== 'number' || isnan(obj.length)) {
        return createBuffer(that, 0)
      }
      return fromArrayLike(that, obj)
    }

    if (obj.type === 'Buffer' && isArray(obj.data)) {
      return fromArrayLike(that, obj.data)
    }
  }

  throw new TypeError('First argument must be a string, Buffer, ArrayBuffer, Array, or array-like object.')
}

function checked (length) {
  // Note: cannot use `length < kMaxLength()` here because that fails when
  // length is NaN (which is otherwise coerced to zero.)
  if (length >= kMaxLength()) {
    throw new RangeError('Attempt to allocate Buffer larger than maximum ' +
                         'size: 0x' + kMaxLength().toString(16) + ' bytes')
  }
  return length | 0
}
Buffer.isBuffer = isBuffer;
function internalIsBuffer (b) {
  return !!(b != null && b._isBuffer)
}

Buffer.compare = function compare (a, b) {
  if (!internalIsBuffer(a) || !internalIsBuffer(b)) {
    throw new TypeError('Arguments must be Buffers')
  }

  if (a === b) return 0

  var x = a.length;
  var y = b.length;

  for (var i = 0, len = Math.min(x, y); i < len; ++i) {
    if (a[i] !== b[i]) {
      x = a[i];
      y = b[i];
      break
    }
  }

  if (x < y) return -1
  if (y < x) return 1
  return 0
};

Buffer.isEncoding = function isEncoding (encoding) {
  switch (String(encoding).toLowerCase()) {
    case 'hex':
    case 'utf8':
    case 'utf-8':
    case 'ascii':
    case 'latin1':
    case 'binary':
    case 'base64':
    case 'ucs2':
    case 'ucs-2':
    case 'utf16le':
    case 'utf-16le':
      return true
    default:
      return false
  }
};

Buffer.concat = function concat (list, length) {
  if (!isArray(list)) {
    throw new TypeError('"list" argument must be an Array of Buffers')
  }

  if (list.length === 0) {
    return Buffer.alloc(0)
  }

  var i;
  if (length === undefined) {
    length = 0;
    for (i = 0; i < list.length; ++i) {
      length += list[i].length;
    }
  }

  var buffer = Buffer.allocUnsafe(length);
  var pos = 0;
  for (i = 0; i < list.length; ++i) {
    var buf = list[i];
    if (!internalIsBuffer(buf)) {
      throw new TypeError('"list" argument must be an Array of Buffers')
    }
    buf.copy(buffer, pos);
    pos += buf.length;
  }
  return buffer
};

function byteLength (string, encoding) {
  if (internalIsBuffer(string)) {
    return string.length
  }
  if (typeof ArrayBuffer !== 'undefined' && typeof ArrayBuffer.isView === 'function' &&
      (ArrayBuffer.isView(string) || string instanceof ArrayBuffer)) {
    return string.byteLength
  }
  if (typeof string !== 'string') {
    string = '' + string;
  }

  var len = string.length;
  if (len === 0) return 0

  // Use a for loop to avoid recursion
  var loweredCase = false;
  for (;;) {
    switch (encoding) {
      case 'ascii':
      case 'latin1':
      case 'binary':
        return len
      case 'utf8':
      case 'utf-8':
      case undefined:
        return utf8ToBytes(string).length
      case 'ucs2':
      case 'ucs-2':
      case 'utf16le':
      case 'utf-16le':
        return len * 2
      case 'hex':
        return len >>> 1
      case 'base64':
        return base64ToBytes(string).length
      default:
        if (loweredCase) return utf8ToBytes(string).length // assume utf8
        encoding = ('' + encoding).toLowerCase();
        loweredCase = true;
    }
  }
}
Buffer.byteLength = byteLength;

function slowToString (encoding, start, end) {
  var loweredCase = false;

  // No need to verify that "this.length <= MAX_UINT32" since it's a read-only
  // property of a typed array.

  // This behaves neither like String nor Uint8Array in that we set start/end
  // to their upper/lower bounds if the value passed is out of range.
  // undefined is handled specially as per ECMA-262 6th Edition,
  // Section 13.3.3.7 Runtime Semantics: KeyedBindingInitialization.
  if (start === undefined || start < 0) {
    start = 0;
  }
  // Return early if start > this.length. Done here to prevent potential uint32
  // coercion fail below.
  if (start > this.length) {
    return ''
  }

  if (end === undefined || end > this.length) {
    end = this.length;
  }

  if (end <= 0) {
    return ''
  }

  // Force coersion to uint32. This will also coerce falsey/NaN values to 0.
  end >>>= 0;
  start >>>= 0;

  if (end <= start) {
    return ''
  }

  if (!encoding) encoding = 'utf8';

  while (true) {
    switch (encoding) {
      case 'hex':
        return hexSlice(this, start, end)

      case 'utf8':
      case 'utf-8':
        return utf8Slice(this, start, end)

      case 'ascii':
        return asciiSlice(this, start, end)

      case 'latin1':
      case 'binary':
        return latin1Slice(this, start, end)

      case 'base64':
        return base64Slice(this, start, end)

      case 'ucs2':
      case 'ucs-2':
      case 'utf16le':
      case 'utf-16le':
        return utf16leSlice(this, start, end)

      default:
        if (loweredCase) throw new TypeError('Unknown encoding: ' + encoding)
        encoding = (encoding + '').toLowerCase();
        loweredCase = true;
    }
  }
}

// The property is used by `Buffer.isBuffer` and `is-buffer` (in Safari 5-7) to detect
// Buffer instances.
Buffer.prototype._isBuffer = true;

function swap (b, n, m) {
  var i = b[n];
  b[n] = b[m];
  b[m] = i;
}

Buffer.prototype.swap16 = function swap16 () {
  var len = this.length;
  if (len % 2 !== 0) {
    throw new RangeError('Buffer size must be a multiple of 16-bits')
  }
  for (var i = 0; i < len; i += 2) {
    swap(this, i, i + 1);
  }
  return this
};

Buffer.prototype.swap32 = function swap32 () {
  var len = this.length;
  if (len % 4 !== 0) {
    throw new RangeError('Buffer size must be a multiple of 32-bits')
  }
  for (var i = 0; i < len; i += 4) {
    swap(this, i, i + 3);
    swap(this, i + 1, i + 2);
  }
  return this
};

Buffer.prototype.swap64 = function swap64 () {
  var len = this.length;
  if (len % 8 !== 0) {
    throw new RangeError('Buffer size must be a multiple of 64-bits')
  }
  for (var i = 0; i < len; i += 8) {
    swap(this, i, i + 7);
    swap(this, i + 1, i + 6);
    swap(this, i + 2, i + 5);
    swap(this, i + 3, i + 4);
  }
  return this
};

Buffer.prototype.toString = function toString () {
  var length = this.length | 0;
  if (length === 0) return ''
  if (arguments.length === 0) return utf8Slice(this, 0, length)
  return slowToString.apply(this, arguments)
};

Buffer.prototype.equals = function equals (b) {
  if (!internalIsBuffer(b)) throw new TypeError('Argument must be a Buffer')
  if (this === b) return true
  return Buffer.compare(this, b) === 0
};

Buffer.prototype.inspect = function inspect () {
  var str = '';
  var max = INSPECT_MAX_BYTES;
  if (this.length > 0) {
    str = this.toString('hex', 0, max).match(/.{2}/g).join(' ');
    if (this.length > max) str += ' ... ';
  }
  return '<Buffer ' + str + '>'
};

Buffer.prototype.compare = function compare (target, start, end, thisStart, thisEnd) {
  if (!internalIsBuffer(target)) {
    throw new TypeError('Argument must be a Buffer')
  }

  if (start === undefined) {
    start = 0;
  }
  if (end === undefined) {
    end = target ? target.length : 0;
  }
  if (thisStart === undefined) {
    thisStart = 0;
  }
  if (thisEnd === undefined) {
    thisEnd = this.length;
  }

  if (start < 0 || end > target.length || thisStart < 0 || thisEnd > this.length) {
    throw new RangeError('out of range index')
  }

  if (thisStart >= thisEnd && start >= end) {
    return 0
  }
  if (thisStart >= thisEnd) {
    return -1
  }
  if (start >= end) {
    return 1
  }

  start >>>= 0;
  end >>>= 0;
  thisStart >>>= 0;
  thisEnd >>>= 0;

  if (this === target) return 0

  var x = thisEnd - thisStart;
  var y = end - start;
  var len = Math.min(x, y);

  var thisCopy = this.slice(thisStart, thisEnd);
  var targetCopy = target.slice(start, end);

  for (var i = 0; i < len; ++i) {
    if (thisCopy[i] !== targetCopy[i]) {
      x = thisCopy[i];
      y = targetCopy[i];
      break
    }
  }

  if (x < y) return -1
  if (y < x) return 1
  return 0
};

// Finds either the first index of `val` in `buffer` at offset >= `byteOffset`,
// OR the last index of `val` in `buffer` at offset <= `byteOffset`.
//
// Arguments:
// - buffer - a Buffer to search
// - val - a string, Buffer, or number
// - byteOffset - an index into `buffer`; will be clamped to an int32
// - encoding - an optional encoding, relevant is val is a string
// - dir - true for indexOf, false for lastIndexOf
function bidirectionalIndexOf (buffer, val, byteOffset, encoding, dir) {
  // Empty buffer means no match
  if (buffer.length === 0) return -1

  // Normalize byteOffset
  if (typeof byteOffset === 'string') {
    encoding = byteOffset;
    byteOffset = 0;
  } else if (byteOffset > 0x7fffffff) {
    byteOffset = 0x7fffffff;
  } else if (byteOffset < -0x80000000) {
    byteOffset = -0x80000000;
  }
  byteOffset = +byteOffset;  // Coerce to Number.
  if (isNaN(byteOffset)) {
    // byteOffset: it it's undefined, null, NaN, "foo", etc, search whole buffer
    byteOffset = dir ? 0 : (buffer.length - 1);
  }

  // Normalize byteOffset: negative offsets start from the end of the buffer
  if (byteOffset < 0) byteOffset = buffer.length + byteOffset;
  if (byteOffset >= buffer.length) {
    if (dir) return -1
    else byteOffset = buffer.length - 1;
  } else if (byteOffset < 0) {
    if (dir) byteOffset = 0;
    else return -1
  }

  // Normalize val
  if (typeof val === 'string') {
    val = Buffer.from(val, encoding);
  }

  // Finally, search either indexOf (if dir is true) or lastIndexOf
  if (internalIsBuffer(val)) {
    // Special case: looking for empty string/buffer always fails
    if (val.length === 0) {
      return -1
    }
    return arrayIndexOf(buffer, val, byteOffset, encoding, dir)
  } else if (typeof val === 'number') {
    val = val & 0xFF; // Search for a byte value [0-255]
    if (Buffer.TYPED_ARRAY_SUPPORT &&
        typeof Uint8Array.prototype.indexOf === 'function') {
      if (dir) {
        return Uint8Array.prototype.indexOf.call(buffer, val, byteOffset)
      } else {
        return Uint8Array.prototype.lastIndexOf.call(buffer, val, byteOffset)
      }
    }
    return arrayIndexOf(buffer, [ val ], byteOffset, encoding, dir)
  }

  throw new TypeError('val must be string, number or Buffer')
}

function arrayIndexOf (arr, val, byteOffset, encoding, dir) {
  var indexSize = 1;
  var arrLength = arr.length;
  var valLength = val.length;

  if (encoding !== undefined) {
    encoding = String(encoding).toLowerCase();
    if (encoding === 'ucs2' || encoding === 'ucs-2' ||
        encoding === 'utf16le' || encoding === 'utf-16le') {
      if (arr.length < 2 || val.length < 2) {
        return -1
      }
      indexSize = 2;
      arrLength /= 2;
      valLength /= 2;
      byteOffset /= 2;
    }
  }

  function read (buf, i) {
    if (indexSize === 1) {
      return buf[i]
    } else {
      return buf.readUInt16BE(i * indexSize)
    }
  }

  var i;
  if (dir) {
    var foundIndex = -1;
    for (i = byteOffset; i < arrLength; i++) {
      if (read(arr, i) === read(val, foundIndex === -1 ? 0 : i - foundIndex)) {
        if (foundIndex === -1) foundIndex = i;
        if (i - foundIndex + 1 === valLength) return foundIndex * indexSize
      } else {
        if (foundIndex !== -1) i -= i - foundIndex;
        foundIndex = -1;
      }
    }
  } else {
    if (byteOffset + valLength > arrLength) byteOffset = arrLength - valLength;
    for (i = byteOffset; i >= 0; i--) {
      var found = true;
      for (var j = 0; j < valLength; j++) {
        if (read(arr, i + j) !== read(val, j)) {
          found = false;
          break
        }
      }
      if (found) return i
    }
  }

  return -1
}

Buffer.prototype.includes = function includes (val, byteOffset, encoding) {
  return this.indexOf(val, byteOffset, encoding) !== -1
};

Buffer.prototype.indexOf = function indexOf (val, byteOffset, encoding) {
  return bidirectionalIndexOf(this, val, byteOffset, encoding, true)
};

Buffer.prototype.lastIndexOf = function lastIndexOf (val, byteOffset, encoding) {
  return bidirectionalIndexOf(this, val, byteOffset, encoding, false)
};

function hexWrite (buf, string, offset, length) {
  offset = Number(offset) || 0;
  var remaining = buf.length - offset;
  if (!length) {
    length = remaining;
  } else {
    length = Number(length);
    if (length > remaining) {
      length = remaining;
    }
  }

  // must be an even number of digits
  var strLen = string.length;
  if (strLen % 2 !== 0) throw new TypeError('Invalid hex string')

  if (length > strLen / 2) {
    length = strLen / 2;
  }
  for (var i = 0; i < length; ++i) {
    var parsed = parseInt(string.substr(i * 2, 2), 16);
    if (isNaN(parsed)) return i
    buf[offset + i] = parsed;
  }
  return i
}

function utf8Write (buf, string, offset, length) {
  return blitBuffer(utf8ToBytes(string, buf.length - offset), buf, offset, length)
}

function asciiWrite (buf, string, offset, length) {
  return blitBuffer(asciiToBytes(string), buf, offset, length)
}

function latin1Write (buf, string, offset, length) {
  return asciiWrite(buf, string, offset, length)
}

function base64Write (buf, string, offset, length) {
  return blitBuffer(base64ToBytes(string), buf, offset, length)
}

function ucs2Write (buf, string, offset, length) {
  return blitBuffer(utf16leToBytes(string, buf.length - offset), buf, offset, length)
}

Buffer.prototype.write = function write (string, offset, length, encoding) {
  // Buffer#write(string)
  if (offset === undefined) {
    encoding = 'utf8';
    length = this.length;
    offset = 0;
  // Buffer#write(string, encoding)
  } else if (length === undefined && typeof offset === 'string') {
    encoding = offset;
    length = this.length;
    offset = 0;
  // Buffer#write(string, offset[, length][, encoding])
  } else if (isFinite(offset)) {
    offset = offset | 0;
    if (isFinite(length)) {
      length = length | 0;
      if (encoding === undefined) encoding = 'utf8';
    } else {
      encoding = length;
      length = undefined;
    }
  // legacy write(string, encoding, offset, length) - remove in v0.13
  } else {
    throw new Error(
      'Buffer.write(string, encoding, offset[, length]) is no longer supported'
    )
  }

  var remaining = this.length - offset;
  if (length === undefined || length > remaining) length = remaining;

  if ((string.length > 0 && (length < 0 || offset < 0)) || offset > this.length) {
    throw new RangeError('Attempt to write outside buffer bounds')
  }

  if (!encoding) encoding = 'utf8';

  var loweredCase = false;
  for (;;) {
    switch (encoding) {
      case 'hex':
        return hexWrite(this, string, offset, length)

      case 'utf8':
      case 'utf-8':
        return utf8Write(this, string, offset, length)

      case 'ascii':
        return asciiWrite(this, string, offset, length)

      case 'latin1':
      case 'binary':
        return latin1Write(this, string, offset, length)

      case 'base64':
        // Warning: maxLength not taken into account in base64Write
        return base64Write(this, string, offset, length)

      case 'ucs2':
      case 'ucs-2':
      case 'utf16le':
      case 'utf-16le':
        return ucs2Write(this, string, offset, length)

      default:
        if (loweredCase) throw new TypeError('Unknown encoding: ' + encoding)
        encoding = ('' + encoding).toLowerCase();
        loweredCase = true;
    }
  }
};

Buffer.prototype.toJSON = function toJSON () {
  return {
    type: 'Buffer',
    data: Array.prototype.slice.call(this._arr || this, 0)
  }
};

function base64Slice (buf, start, end) {
  if (start === 0 && end === buf.length) {
    return fromByteArray(buf)
  } else {
    return fromByteArray(buf.slice(start, end))
  }
}

function utf8Slice (buf, start, end) {
  end = Math.min(buf.length, end);
  var res = [];

  var i = start;
  while (i < end) {
    var firstByte = buf[i];
    var codePoint = null;
    var bytesPerSequence = (firstByte > 0xEF) ? 4
      : (firstByte > 0xDF) ? 3
      : (firstByte > 0xBF) ? 2
      : 1;

    if (i + bytesPerSequence <= end) {
      var secondByte, thirdByte, fourthByte, tempCodePoint;

      switch (bytesPerSequence) {
        case 1:
          if (firstByte < 0x80) {
            codePoint = firstByte;
          }
          break
        case 2:
          secondByte = buf[i + 1];
          if ((secondByte & 0xC0) === 0x80) {
            tempCodePoint = (firstByte & 0x1F) << 0x6 | (secondByte & 0x3F);
            if (tempCodePoint > 0x7F) {
              codePoint = tempCodePoint;
            }
          }
          break
        case 3:
          secondByte = buf[i + 1];
          thirdByte = buf[i + 2];
          if ((secondByte & 0xC0) === 0x80 && (thirdByte & 0xC0) === 0x80) {
            tempCodePoint = (firstByte & 0xF) << 0xC | (secondByte & 0x3F) << 0x6 | (thirdByte & 0x3F);
            if (tempCodePoint > 0x7FF && (tempCodePoint < 0xD800 || tempCodePoint > 0xDFFF)) {
              codePoint = tempCodePoint;
            }
          }
          break
        case 4:
          secondByte = buf[i + 1];
          thirdByte = buf[i + 2];
          fourthByte = buf[i + 3];
          if ((secondByte & 0xC0) === 0x80 && (thirdByte & 0xC0) === 0x80 && (fourthByte & 0xC0) === 0x80) {
            tempCodePoint = (firstByte & 0xF) << 0x12 | (secondByte & 0x3F) << 0xC | (thirdByte & 0x3F) << 0x6 | (fourthByte & 0x3F);
            if (tempCodePoint > 0xFFFF && tempCodePoint < 0x110000) {
              codePoint = tempCodePoint;
            }
          }
      }
    }

    if (codePoint === null) {
      // we did not generate a valid codePoint so insert a
      // replacement char (U+FFFD) and advance only 1 byte
      codePoint = 0xFFFD;
      bytesPerSequence = 1;
    } else if (codePoint > 0xFFFF) {
      // encode to utf16 (surrogate pair dance)
      codePoint -= 0x10000;
      res.push(codePoint >>> 10 & 0x3FF | 0xD800);
      codePoint = 0xDC00 | codePoint & 0x3FF;
    }

    res.push(codePoint);
    i += bytesPerSequence;
  }

  return decodeCodePointsArray(res)
}

// Based on http://stackoverflow.com/a/22747272/680742, the browser with
// the lowest limit is Chrome, with 0x10000 args.
// We go 1 magnitude less, for safety
var MAX_ARGUMENTS_LENGTH = 0x1000;

function decodeCodePointsArray (codePoints) {
  var len = codePoints.length;
  if (len <= MAX_ARGUMENTS_LENGTH) {
    return String.fromCharCode.apply(String, codePoints) // avoid extra slice()
  }

  // Decode in chunks to avoid "call stack size exceeded".
  var res = '';
  var i = 0;
  while (i < len) {
    res += String.fromCharCode.apply(
      String,
      codePoints.slice(i, i += MAX_ARGUMENTS_LENGTH)
    );
  }
  return res
}

function asciiSlice (buf, start, end) {
  var ret = '';
  end = Math.min(buf.length, end);

  for (var i = start; i < end; ++i) {
    ret += String.fromCharCode(buf[i] & 0x7F);
  }
  return ret
}

function latin1Slice (buf, start, end) {
  var ret = '';
  end = Math.min(buf.length, end);

  for (var i = start; i < end; ++i) {
    ret += String.fromCharCode(buf[i]);
  }
  return ret
}

function hexSlice (buf, start, end) {
  var len = buf.length;

  if (!start || start < 0) start = 0;
  if (!end || end < 0 || end > len) end = len;

  var out = '';
  for (var i = start; i < end; ++i) {
    out += toHex(buf[i]);
  }
  return out
}

function utf16leSlice (buf, start, end) {
  var bytes = buf.slice(start, end);
  var res = '';
  for (var i = 0; i < bytes.length; i += 2) {
    res += String.fromCharCode(bytes[i] + bytes[i + 1] * 256);
  }
  return res
}

Buffer.prototype.slice = function slice (start, end) {
  var len = this.length;
  start = ~~start;
  end = end === undefined ? len : ~~end;

  if (start < 0) {
    start += len;
    if (start < 0) start = 0;
  } else if (start > len) {
    start = len;
  }

  if (end < 0) {
    end += len;
    if (end < 0) end = 0;
  } else if (end > len) {
    end = len;
  }

  if (end < start) end = start;

  var newBuf;
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    newBuf = this.subarray(start, end);
    newBuf.__proto__ = Buffer.prototype;
  } else {
    var sliceLen = end - start;
    newBuf = new Buffer(sliceLen, undefined);
    for (var i = 0; i < sliceLen; ++i) {
      newBuf[i] = this[i + start];
    }
  }

  return newBuf
};

/*
 * Need to make sure that buffer isn't trying to write out of bounds.
 */
function checkOffset (offset, ext, length) {
  if ((offset % 1) !== 0 || offset < 0) throw new RangeError('offset is not uint')
  if (offset + ext > length) throw new RangeError('Trying to access beyond buffer length')
}

Buffer.prototype.readUIntLE = function readUIntLE (offset, byteLength, noAssert) {
  offset = offset | 0;
  byteLength = byteLength | 0;
  if (!noAssert) checkOffset(offset, byteLength, this.length);

  var val = this[offset];
  var mul = 1;
  var i = 0;
  while (++i < byteLength && (mul *= 0x100)) {
    val += this[offset + i] * mul;
  }

  return val
};

Buffer.prototype.readUIntBE = function readUIntBE (offset, byteLength, noAssert) {
  offset = offset | 0;
  byteLength = byteLength | 0;
  if (!noAssert) {
    checkOffset(offset, byteLength, this.length);
  }

  var val = this[offset + --byteLength];
  var mul = 1;
  while (byteLength > 0 && (mul *= 0x100)) {
    val += this[offset + --byteLength] * mul;
  }

  return val
};

Buffer.prototype.readUInt8 = function readUInt8 (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 1, this.length);
  return this[offset]
};

Buffer.prototype.readUInt16LE = function readUInt16LE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 2, this.length);
  return this[offset] | (this[offset + 1] << 8)
};

Buffer.prototype.readUInt16BE = function readUInt16BE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 2, this.length);
  return (this[offset] << 8) | this[offset + 1]
};

Buffer.prototype.readUInt32LE = function readUInt32LE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 4, this.length);

  return ((this[offset]) |
      (this[offset + 1] << 8) |
      (this[offset + 2] << 16)) +
      (this[offset + 3] * 0x1000000)
};

Buffer.prototype.readUInt32BE = function readUInt32BE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 4, this.length);

  return (this[offset] * 0x1000000) +
    ((this[offset + 1] << 16) |
    (this[offset + 2] << 8) |
    this[offset + 3])
};

Buffer.prototype.readIntLE = function readIntLE (offset, byteLength, noAssert) {
  offset = offset | 0;
  byteLength = byteLength | 0;
  if (!noAssert) checkOffset(offset, byteLength, this.length);

  var val = this[offset];
  var mul = 1;
  var i = 0;
  while (++i < byteLength && (mul *= 0x100)) {
    val += this[offset + i] * mul;
  }
  mul *= 0x80;

  if (val >= mul) val -= Math.pow(2, 8 * byteLength);

  return val
};

Buffer.prototype.readIntBE = function readIntBE (offset, byteLength, noAssert) {
  offset = offset | 0;
  byteLength = byteLength | 0;
  if (!noAssert) checkOffset(offset, byteLength, this.length);

  var i = byteLength;
  var mul = 1;
  var val = this[offset + --i];
  while (i > 0 && (mul *= 0x100)) {
    val += this[offset + --i] * mul;
  }
  mul *= 0x80;

  if (val >= mul) val -= Math.pow(2, 8 * byteLength);

  return val
};

Buffer.prototype.readInt8 = function readInt8 (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 1, this.length);
  if (!(this[offset] & 0x80)) return (this[offset])
  return ((0xff - this[offset] + 1) * -1)
};

Buffer.prototype.readInt16LE = function readInt16LE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 2, this.length);
  var val = this[offset] | (this[offset + 1] << 8);
  return (val & 0x8000) ? val | 0xFFFF0000 : val
};

Buffer.prototype.readInt16BE = function readInt16BE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 2, this.length);
  var val = this[offset + 1] | (this[offset] << 8);
  return (val & 0x8000) ? val | 0xFFFF0000 : val
};

Buffer.prototype.readInt32LE = function readInt32LE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 4, this.length);

  return (this[offset]) |
    (this[offset + 1] << 8) |
    (this[offset + 2] << 16) |
    (this[offset + 3] << 24)
};

Buffer.prototype.readInt32BE = function readInt32BE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 4, this.length);

  return (this[offset] << 24) |
    (this[offset + 1] << 16) |
    (this[offset + 2] << 8) |
    (this[offset + 3])
};

Buffer.prototype.readFloatLE = function readFloatLE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 4, this.length);
  return read(this, offset, true, 23, 4)
};

Buffer.prototype.readFloatBE = function readFloatBE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 4, this.length);
  return read(this, offset, false, 23, 4)
};

Buffer.prototype.readDoubleLE = function readDoubleLE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 8, this.length);
  return read(this, offset, true, 52, 8)
};

Buffer.prototype.readDoubleBE = function readDoubleBE (offset, noAssert) {
  if (!noAssert) checkOffset(offset, 8, this.length);
  return read(this, offset, false, 52, 8)
};

function checkInt (buf, value, offset, ext, max, min) {
  if (!internalIsBuffer(buf)) throw new TypeError('"buffer" argument must be a Buffer instance')
  if (value > max || value < min) throw new RangeError('"value" argument is out of bounds')
  if (offset + ext > buf.length) throw new RangeError('Index out of range')
}

Buffer.prototype.writeUIntLE = function writeUIntLE (value, offset, byteLength, noAssert) {
  value = +value;
  offset = offset | 0;
  byteLength = byteLength | 0;
  if (!noAssert) {
    var maxBytes = Math.pow(2, 8 * byteLength) - 1;
    checkInt(this, value, offset, byteLength, maxBytes, 0);
  }

  var mul = 1;
  var i = 0;
  this[offset] = value & 0xFF;
  while (++i < byteLength && (mul *= 0x100)) {
    this[offset + i] = (value / mul) & 0xFF;
  }

  return offset + byteLength
};

Buffer.prototype.writeUIntBE = function writeUIntBE (value, offset, byteLength, noAssert) {
  value = +value;
  offset = offset | 0;
  byteLength = byteLength | 0;
  if (!noAssert) {
    var maxBytes = Math.pow(2, 8 * byteLength) - 1;
    checkInt(this, value, offset, byteLength, maxBytes, 0);
  }

  var i = byteLength - 1;
  var mul = 1;
  this[offset + i] = value & 0xFF;
  while (--i >= 0 && (mul *= 0x100)) {
    this[offset + i] = (value / mul) & 0xFF;
  }

  return offset + byteLength
};

Buffer.prototype.writeUInt8 = function writeUInt8 (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 1, 0xff, 0);
  if (!Buffer.TYPED_ARRAY_SUPPORT) value = Math.floor(value);
  this[offset] = (value & 0xff);
  return offset + 1
};

function objectWriteUInt16 (buf, value, offset, littleEndian) {
  if (value < 0) value = 0xffff + value + 1;
  for (var i = 0, j = Math.min(buf.length - offset, 2); i < j; ++i) {
    buf[offset + i] = (value & (0xff << (8 * (littleEndian ? i : 1 - i)))) >>>
      (littleEndian ? i : 1 - i) * 8;
  }
}

Buffer.prototype.writeUInt16LE = function writeUInt16LE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 2, 0xffff, 0);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value & 0xff);
    this[offset + 1] = (value >>> 8);
  } else {
    objectWriteUInt16(this, value, offset, true);
  }
  return offset + 2
};

Buffer.prototype.writeUInt16BE = function writeUInt16BE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 2, 0xffff, 0);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value >>> 8);
    this[offset + 1] = (value & 0xff);
  } else {
    objectWriteUInt16(this, value, offset, false);
  }
  return offset + 2
};

function objectWriteUInt32 (buf, value, offset, littleEndian) {
  if (value < 0) value = 0xffffffff + value + 1;
  for (var i = 0, j = Math.min(buf.length - offset, 4); i < j; ++i) {
    buf[offset + i] = (value >>> (littleEndian ? i : 3 - i) * 8) & 0xff;
  }
}

Buffer.prototype.writeUInt32LE = function writeUInt32LE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 4, 0xffffffff, 0);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset + 3] = (value >>> 24);
    this[offset + 2] = (value >>> 16);
    this[offset + 1] = (value >>> 8);
    this[offset] = (value & 0xff);
  } else {
    objectWriteUInt32(this, value, offset, true);
  }
  return offset + 4
};

Buffer.prototype.writeUInt32BE = function writeUInt32BE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 4, 0xffffffff, 0);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value >>> 24);
    this[offset + 1] = (value >>> 16);
    this[offset + 2] = (value >>> 8);
    this[offset + 3] = (value & 0xff);
  } else {
    objectWriteUInt32(this, value, offset, false);
  }
  return offset + 4
};

Buffer.prototype.writeIntLE = function writeIntLE (value, offset, byteLength, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) {
    var limit = Math.pow(2, 8 * byteLength - 1);

    checkInt(this, value, offset, byteLength, limit - 1, -limit);
  }

  var i = 0;
  var mul = 1;
  var sub = 0;
  this[offset] = value & 0xFF;
  while (++i < byteLength && (mul *= 0x100)) {
    if (value < 0 && sub === 0 && this[offset + i - 1] !== 0) {
      sub = 1;
    }
    this[offset + i] = ((value / mul) >> 0) - sub & 0xFF;
  }

  return offset + byteLength
};

Buffer.prototype.writeIntBE = function writeIntBE (value, offset, byteLength, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) {
    var limit = Math.pow(2, 8 * byteLength - 1);

    checkInt(this, value, offset, byteLength, limit - 1, -limit);
  }

  var i = byteLength - 1;
  var mul = 1;
  var sub = 0;
  this[offset + i] = value & 0xFF;
  while (--i >= 0 && (mul *= 0x100)) {
    if (value < 0 && sub === 0 && this[offset + i + 1] !== 0) {
      sub = 1;
    }
    this[offset + i] = ((value / mul) >> 0) - sub & 0xFF;
  }

  return offset + byteLength
};

Buffer.prototype.writeInt8 = function writeInt8 (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 1, 0x7f, -0x80);
  if (!Buffer.TYPED_ARRAY_SUPPORT) value = Math.floor(value);
  if (value < 0) value = 0xff + value + 1;
  this[offset] = (value & 0xff);
  return offset + 1
};

Buffer.prototype.writeInt16LE = function writeInt16LE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 2, 0x7fff, -0x8000);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value & 0xff);
    this[offset + 1] = (value >>> 8);
  } else {
    objectWriteUInt16(this, value, offset, true);
  }
  return offset + 2
};

Buffer.prototype.writeInt16BE = function writeInt16BE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 2, 0x7fff, -0x8000);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value >>> 8);
    this[offset + 1] = (value & 0xff);
  } else {
    objectWriteUInt16(this, value, offset, false);
  }
  return offset + 2
};

Buffer.prototype.writeInt32LE = function writeInt32LE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 4, 0x7fffffff, -0x80000000);
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value & 0xff);
    this[offset + 1] = (value >>> 8);
    this[offset + 2] = (value >>> 16);
    this[offset + 3] = (value >>> 24);
  } else {
    objectWriteUInt32(this, value, offset, true);
  }
  return offset + 4
};

Buffer.prototype.writeInt32BE = function writeInt32BE (value, offset, noAssert) {
  value = +value;
  offset = offset | 0;
  if (!noAssert) checkInt(this, value, offset, 4, 0x7fffffff, -0x80000000);
  if (value < 0) value = 0xffffffff + value + 1;
  if (Buffer.TYPED_ARRAY_SUPPORT) {
    this[offset] = (value >>> 24);
    this[offset + 1] = (value >>> 16);
    this[offset + 2] = (value >>> 8);
    this[offset + 3] = (value & 0xff);
  } else {
    objectWriteUInt32(this, value, offset, false);
  }
  return offset + 4
};

function checkIEEE754 (buf, value, offset, ext, max, min) {
  if (offset + ext > buf.length) throw new RangeError('Index out of range')
  if (offset < 0) throw new RangeError('Index out of range')
}

function writeFloat (buf, value, offset, littleEndian, noAssert) {
  if (!noAssert) {
    checkIEEE754(buf, value, offset, 4);
  }
  write(buf, value, offset, littleEndian, 23, 4);
  return offset + 4
}

Buffer.prototype.writeFloatLE = function writeFloatLE (value, offset, noAssert) {
  return writeFloat(this, value, offset, true, noAssert)
};

Buffer.prototype.writeFloatBE = function writeFloatBE (value, offset, noAssert) {
  return writeFloat(this, value, offset, false, noAssert)
};

function writeDouble (buf, value, offset, littleEndian, noAssert) {
  if (!noAssert) {
    checkIEEE754(buf, value, offset, 8);
  }
  write(buf, value, offset, littleEndian, 52, 8);
  return offset + 8
}

Buffer.prototype.writeDoubleLE = function writeDoubleLE (value, offset, noAssert) {
  return writeDouble(this, value, offset, true, noAssert)
};

Buffer.prototype.writeDoubleBE = function writeDoubleBE (value, offset, noAssert) {
  return writeDouble(this, value, offset, false, noAssert)
};

// copy(targetBuffer, targetStart=0, sourceStart=0, sourceEnd=buffer.length)
Buffer.prototype.copy = function copy (target, targetStart, start, end) {
  if (!start) start = 0;
  if (!end && end !== 0) end = this.length;
  if (targetStart >= target.length) targetStart = target.length;
  if (!targetStart) targetStart = 0;
  if (end > 0 && end < start) end = start;

  // Copy 0 bytes; we're done
  if (end === start) return 0
  if (target.length === 0 || this.length === 0) return 0

  // Fatal error conditions
  if (targetStart < 0) {
    throw new RangeError('targetStart out of bounds')
  }
  if (start < 0 || start >= this.length) throw new RangeError('sourceStart out of bounds')
  if (end < 0) throw new RangeError('sourceEnd out of bounds')

  // Are we oob?
  if (end > this.length) end = this.length;
  if (target.length - targetStart < end - start) {
    end = target.length - targetStart + start;
  }

  var len = end - start;
  var i;

  if (this === target && start < targetStart && targetStart < end) {
    // descending copy from end
    for (i = len - 1; i >= 0; --i) {
      target[i + targetStart] = this[i + start];
    }
  } else if (len < 1000 || !Buffer.TYPED_ARRAY_SUPPORT) {
    // ascending copy from start
    for (i = 0; i < len; ++i) {
      target[i + targetStart] = this[i + start];
    }
  } else {
    Uint8Array.prototype.set.call(
      target,
      this.subarray(start, start + len),
      targetStart
    );
  }

  return len
};

// Usage:
//    buffer.fill(number[, offset[, end]])
//    buffer.fill(buffer[, offset[, end]])
//    buffer.fill(string[, offset[, end]][, encoding])
Buffer.prototype.fill = function fill (val, start, end, encoding) {
  // Handle string cases:
  if (typeof val === 'string') {
    if (typeof start === 'string') {
      encoding = start;
      start = 0;
      end = this.length;
    } else if (typeof end === 'string') {
      encoding = end;
      end = this.length;
    }
    if (val.length === 1) {
      var code = val.charCodeAt(0);
      if (code < 256) {
        val = code;
      }
    }
    if (encoding !== undefined && typeof encoding !== 'string') {
      throw new TypeError('encoding must be a string')
    }
    if (typeof encoding === 'string' && !Buffer.isEncoding(encoding)) {
      throw new TypeError('Unknown encoding: ' + encoding)
    }
  } else if (typeof val === 'number') {
    val = val & 255;
  }

  // Invalid ranges are not set to a default, so can range check early.
  if (start < 0 || this.length < start || this.length < end) {
    throw new RangeError('Out of range index')
  }

  if (end <= start) {
    return this
  }

  start = start >>> 0;
  end = end === undefined ? this.length : end >>> 0;

  if (!val) val = 0;

  var i;
  if (typeof val === 'number') {
    for (i = start; i < end; ++i) {
      this[i] = val;
    }
  } else {
    var bytes = internalIsBuffer(val)
      ? val
      : utf8ToBytes(new Buffer(val, encoding).toString());
    var len = bytes.length;
    for (i = 0; i < end - start; ++i) {
      this[i + start] = bytes[i % len];
    }
  }

  return this
};

// HELPER FUNCTIONS
// ================

var INVALID_BASE64_RE = /[^+\/0-9A-Za-z-_]/g;

function base64clean (str) {
  // Node strips out invalid characters like \n and \t from the string, base64-js does not
  str = stringtrim(str).replace(INVALID_BASE64_RE, '');
  // Node converts strings with length < 2 to ''
  if (str.length < 2) return ''
  // Node allows for non-padded base64 strings (missing trailing ===), base64-js does not
  while (str.length % 4 !== 0) {
    str = str + '=';
  }
  return str
}

function stringtrim (str) {
  if (str.trim) return str.trim()
  return str.replace(/^\s+|\s+$/g, '')
}

function toHex (n) {
  if (n < 16) return '0' + n.toString(16)
  return n.toString(16)
}

function utf8ToBytes (string, units) {
  units = units || Infinity;
  var codePoint;
  var length = string.length;
  var leadSurrogate = null;
  var bytes = [];

  for (var i = 0; i < length; ++i) {
    codePoint = string.charCodeAt(i);

    // is surrogate component
    if (codePoint > 0xD7FF && codePoint < 0xE000) {
      // last char was a lead
      if (!leadSurrogate) {
        // no lead yet
        if (codePoint > 0xDBFF) {
          // unexpected trail
          if ((units -= 3) > -1) bytes.push(0xEF, 0xBF, 0xBD);
          continue
        } else if (i + 1 === length) {
          // unpaired lead
          if ((units -= 3) > -1) bytes.push(0xEF, 0xBF, 0xBD);
          continue
        }

        // valid lead
        leadSurrogate = codePoint;

        continue
      }

      // 2 leads in a row
      if (codePoint < 0xDC00) {
        if ((units -= 3) > -1) bytes.push(0xEF, 0xBF, 0xBD);
        leadSurrogate = codePoint;
        continue
      }

      // valid surrogate pair
      codePoint = (leadSurrogate - 0xD800 << 10 | codePoint - 0xDC00) + 0x10000;
    } else if (leadSurrogate) {
      // valid bmp char, but last char was a lead
      if ((units -= 3) > -1) bytes.push(0xEF, 0xBF, 0xBD);
    }

    leadSurrogate = null;

    // encode utf8
    if (codePoint < 0x80) {
      if ((units -= 1) < 0) break
      bytes.push(codePoint);
    } else if (codePoint < 0x800) {
      if ((units -= 2) < 0) break
      bytes.push(
        codePoint >> 0x6 | 0xC0,
        codePoint & 0x3F | 0x80
      );
    } else if (codePoint < 0x10000) {
      if ((units -= 3) < 0) break
      bytes.push(
        codePoint >> 0xC | 0xE0,
        codePoint >> 0x6 & 0x3F | 0x80,
        codePoint & 0x3F | 0x80
      );
    } else if (codePoint < 0x110000) {
      if ((units -= 4) < 0) break
      bytes.push(
        codePoint >> 0x12 | 0xF0,
        codePoint >> 0xC & 0x3F | 0x80,
        codePoint >> 0x6 & 0x3F | 0x80,
        codePoint & 0x3F | 0x80
      );
    } else {
      throw new Error('Invalid code point')
    }
  }

  return bytes
}

function asciiToBytes (str) {
  var byteArray = [];
  for (var i = 0; i < str.length; ++i) {
    // Node's code seems to be doing this and not & 0x7F..
    byteArray.push(str.charCodeAt(i) & 0xFF);
  }
  return byteArray
}

function utf16leToBytes (str, units) {
  var c, hi, lo;
  var byteArray = [];
  for (var i = 0; i < str.length; ++i) {
    if ((units -= 2) < 0) break

    c = str.charCodeAt(i);
    hi = c >> 8;
    lo = c % 256;
    byteArray.push(lo);
    byteArray.push(hi);
  }

  return byteArray
}


function base64ToBytes (str) {
  return toByteArray(base64clean(str))
}

function blitBuffer (src, dst, offset, length) {
  for (var i = 0; i < length; ++i) {
    if ((i + offset >= dst.length) || (i >= src.length)) break
    dst[i + offset] = src[i];
  }
  return i
}

function isnan (val) {
  return val !== val // eslint-disable-line no-self-compare
}


// the following is from is-buffer, also by Feross Aboukhadijeh and with same lisence
// The _isBuffer check is for Safari 5-7 support, because it's missing
// Object.prototype.constructor. Remove this eventually
function isBuffer(obj) {
  return obj != null && (!!obj._isBuffer || isFastBuffer(obj) || isSlowBuffer(obj))
}

function isFastBuffer (obj) {
  return !!obj.constructor && typeof obj.constructor.isBuffer === 'function' && obj.constructor.isBuffer(obj)
}

// For Node v0.10 support. Remove this eventually.
function isSlowBuffer (obj) {
  return typeof obj.readFloatLE === 'function' && typeof obj.slice === 'function' && isFastBuffer(obj.slice(0, 0))
}

/**
 * Create an Error with the specified message, config, error code, request and response.
 *
 * @param {string} message The error message.
 * @param {string} [code] The error code (for example, 'ECONNABORTED').
 * @param {Object} [config] The config.
 * @param {Object} [request] The request.
 * @param {Object} [response] The response.
 *
 * @returns {Error} The created error.
 */
function AxiosError(message, code, config, request, response) {
  Error.call(this);

  if (Error.captureStackTrace) {
    Error.captureStackTrace(this, this.constructor);
  } else {
    this.stack = (new Error()).stack;
  }

  this.message = message;
  this.name = 'AxiosError';
  code && (this.code = code);
  config && (this.config = config);
  request && (this.request = request);
  response && (this.response = response);
}

utils.inherits(AxiosError, Error, {
  toJSON: function toJSON() {
    return {
      // Standard
      message: this.message,
      name: this.name,
      // Microsoft
      description: this.description,
      number: this.number,
      // Mozilla
      fileName: this.fileName,
      lineNumber: this.lineNumber,
      columnNumber: this.columnNumber,
      stack: this.stack,
      // Axios
      config: utils.toJSONObject(this.config),
      code: this.code,
      status: this.response && this.response.status ? this.response.status : null
    };
  }
});

const prototype$1 = AxiosError.prototype;
const descriptors = {};

[
  'ERR_BAD_OPTION_VALUE',
  'ERR_BAD_OPTION',
  'ECONNABORTED',
  'ETIMEDOUT',
  'ERR_NETWORK',
  'ERR_FR_TOO_MANY_REDIRECTS',
  'ERR_DEPRECATED',
  'ERR_BAD_RESPONSE',
  'ERR_BAD_REQUEST',
  'ERR_CANCELED',
  'ERR_NOT_SUPPORT',
  'ERR_INVALID_URL'
// eslint-disable-next-line func-names
].forEach(code => {
  descriptors[code] = {value: code};
});

Object.defineProperties(AxiosError, descriptors);
Object.defineProperty(prototype$1, 'isAxiosError', {value: true});

// eslint-disable-next-line func-names
AxiosError.from = (error, code, config, request, response, customProps) => {
  const axiosError = Object.create(prototype$1);

  utils.toFlatObject(error, axiosError, function filter(obj) {
    return obj !== Error.prototype;
  }, prop => {
    return prop !== 'isAxiosError';
  });

  AxiosError.call(axiosError, error.message, code, config, request, response);

  axiosError.cause = error;

  axiosError.name = error.name;

  customProps && Object.assign(axiosError, customProps);

  return axiosError;
};

// eslint-disable-next-line strict
var httpAdapter = null;

/**
 * Determines if the given thing is a array or js object.
 *
 * @param {string} thing - The object or array to be visited.
 *
 * @returns {boolean}
 */
function isVisitable(thing) {
  return utils.isPlainObject(thing) || utils.isArray(thing);
}

/**
 * It removes the brackets from the end of a string
 *
 * @param {string} key - The key of the parameter.
 *
 * @returns {string} the key without the brackets.
 */
function removeBrackets(key) {
  return utils.endsWith(key, '[]') ? key.slice(0, -2) : key;
}

/**
 * It takes a path, a key, and a boolean, and returns a string
 *
 * @param {string} path - The path to the current key.
 * @param {string} key - The key of the current object being iterated over.
 * @param {string} dots - If true, the key will be rendered with dots instead of brackets.
 *
 * @returns {string} The path to the current key.
 */
function renderKey(path, key, dots) {
  if (!path) return key;
  return path.concat(key).map(function each(token, i) {
    // eslint-disable-next-line no-param-reassign
    token = removeBrackets(token);
    return !dots && i ? '[' + token + ']' : token;
  }).join(dots ? '.' : '');
}

/**
 * If the array is an array and none of its elements are visitable, then it's a flat array.
 *
 * @param {Array<any>} arr - The array to check
 *
 * @returns {boolean}
 */
function isFlatArray(arr) {
  return utils.isArray(arr) && !arr.some(isVisitable);
}

const predicates = utils.toFlatObject(utils, {}, null, function filter(prop) {
  return /^is[A-Z]/.test(prop);
});

/**
 * Convert a data object to FormData
 *
 * @param {Object} obj
 * @param {?Object} [formData]
 * @param {?Object} [options]
 * @param {Function} [options.visitor]
 * @param {Boolean} [options.metaTokens = true]
 * @param {Boolean} [options.dots = false]
 * @param {?Boolean} [options.indexes = false]
 *
 * @returns {Object}
 **/

/**
 * It converts an object into a FormData object
 *
 * @param {Object<any, any>} obj - The object to convert to form data.
 * @param {string} formData - The FormData object to append to.
 * @param {Object<string, any>} options
 *
 * @returns
 */
function toFormData(obj, formData, options) {
  if (!utils.isObject(obj)) {
    throw new TypeError('target must be an object');
  }

  // eslint-disable-next-line no-param-reassign
  formData = formData || new (FormData)();

  // eslint-disable-next-line no-param-reassign
  options = utils.toFlatObject(options, {
    metaTokens: true,
    dots: false,
    indexes: false
  }, false, function defined(option, source) {
    // eslint-disable-next-line no-eq-null,eqeqeq
    return !utils.isUndefined(source[option]);
  });

  const metaTokens = options.metaTokens;
  // eslint-disable-next-line no-use-before-define
  const visitor = options.visitor || defaultVisitor;
  const dots = options.dots;
  const indexes = options.indexes;
  const _Blob = options.Blob || typeof Blob !== 'undefined' && Blob;
  const useBlob = _Blob && utils.isSpecCompliantForm(formData);

  if (!utils.isFunction(visitor)) {
    throw new TypeError('visitor must be a function');
  }

  function convertValue(value) {
    if (value === null) return '';

    if (utils.isDate(value)) {
      return value.toISOString();
    }

    if (!useBlob && utils.isBlob(value)) {
      throw new AxiosError('Blob is not supported. Use a Buffer instead.');
    }

    if (utils.isArrayBuffer(value) || utils.isTypedArray(value)) {
      return useBlob && typeof Blob === 'function' ? new Blob([value]) : Buffer.from(value);
    }

    return value;
  }

  /**
   * Default visitor.
   *
   * @param {*} value
   * @param {String|Number} key
   * @param {Array<String|Number>} path
   * @this {FormData}
   *
   * @returns {boolean} return true to visit the each prop of the value recursively
   */
  function defaultVisitor(value, key, path) {
    let arr = value;

    if (value && !path && typeof value === 'object') {
      if (utils.endsWith(key, '{}')) {
        // eslint-disable-next-line no-param-reassign
        key = metaTokens ? key : key.slice(0, -2);
        // eslint-disable-next-line no-param-reassign
        value = JSON.stringify(value);
      } else if (
        (utils.isArray(value) && isFlatArray(value)) ||
        ((utils.isFileList(value) || utils.endsWith(key, '[]')) && (arr = utils.toArray(value))
        )) {
        // eslint-disable-next-line no-param-reassign
        key = removeBrackets(key);

        arr.forEach(function each(el, index) {
          !(utils.isUndefined(el) || el === null) && formData.append(
            // eslint-disable-next-line no-nested-ternary
            indexes === true ? renderKey([key], index, dots) : (indexes === null ? key : key + '[]'),
            convertValue(el)
          );
        });
        return false;
      }
    }

    if (isVisitable(value)) {
      return true;
    }

    formData.append(renderKey(path, key, dots), convertValue(value));

    return false;
  }

  const stack = [];

  const exposedHelpers = Object.assign(predicates, {
    defaultVisitor,
    convertValue,
    isVisitable
  });

  function build(value, path) {
    if (utils.isUndefined(value)) return;

    if (stack.indexOf(value) !== -1) {
      throw Error('Circular reference detected in ' + path.join('.'));
    }

    stack.push(value);

    utils.forEach(value, function each(el, key) {
      const result = !(utils.isUndefined(el) || el === null) && visitor.call(
        formData, el, utils.isString(key) ? key.trim() : key, path, exposedHelpers
      );

      if (result === true) {
        build(el, path ? path.concat(key) : [key]);
      }
    });

    stack.pop();
  }

  if (!utils.isObject(obj)) {
    throw new TypeError('data must be an object');
  }

  build(obj);

  return formData;
}

/**
 * It encodes a string by replacing all characters that are not in the unreserved set with
 * their percent-encoded equivalents
 *
 * @param {string} str - The string to encode.
 *
 * @returns {string} The encoded string.
 */
function encode$1(str) {
  const charMap = {
    '!': '%21',
    "'": '%27',
    '(': '%28',
    ')': '%29',
    '~': '%7E',
    '%20': '+',
    '%00': '\x00'
  };
  return encodeURIComponent(str).replace(/[!'()~]|%20|%00/g, function replacer(match) {
    return charMap[match];
  });
}

/**
 * It takes a params object and converts it to a FormData object
 *
 * @param {Object<string, any>} params - The parameters to be converted to a FormData object.
 * @param {Object<string, any>} options - The options object passed to the Axios constructor.
 *
 * @returns {void}
 */
function AxiosURLSearchParams(params, options) {
  this._pairs = [];

  params && toFormData(params, this, options);
}

const prototype = AxiosURLSearchParams.prototype;

prototype.append = function append(name, value) {
  this._pairs.push([name, value]);
};

prototype.toString = function toString(encoder) {
  const _encode = encoder ? function(value) {
    return encoder.call(this, value, encode$1);
  } : encode$1;

  return this._pairs.map(function each(pair) {
    return _encode(pair[0]) + '=' + _encode(pair[1]);
  }, '').join('&');
};

/**
 * It replaces all instances of the characters `:`, `$`, `,`, `+`, `[`, and `]` with their
 * URI encoded counterparts
 *
 * @param {string} val The value to be encoded.
 *
 * @returns {string} The encoded value.
 */
function encode(val) {
  return encodeURIComponent(val).
    replace(/%3A/gi, ':').
    replace(/%24/g, '$').
    replace(/%2C/gi, ',').
    replace(/%20/g, '+').
    replace(/%5B/gi, '[').
    replace(/%5D/gi, ']');
}

/**
 * Build a URL by appending params to the end
 *
 * @param {string} url The base of the url (e.g., http://www.google.com)
 * @param {object} [params] The params to be appended
 * @param {?object} options
 *
 * @returns {string} The formatted url
 */
function buildURL(url, params, options) {
  /*eslint no-param-reassign:0*/
  if (!params) {
    return url;
  }
  
  const _encode = options && options.encode || encode;

  const serializeFn = options && options.serialize;

  let serializedParams;

  if (serializeFn) {
    serializedParams = serializeFn(params, options);
  } else {
    serializedParams = utils.isURLSearchParams(params) ?
      params.toString() :
      new AxiosURLSearchParams(params, options).toString(_encode);
  }

  if (serializedParams) {
    const hashmarkIndex = url.indexOf("#");

    if (hashmarkIndex !== -1) {
      url = url.slice(0, hashmarkIndex);
    }
    url += (url.indexOf('?') === -1 ? '?' : '&') + serializedParams;
  }

  return url;
}

class InterceptorManager {
  constructor() {
    this.handlers = [];
  }

  /**
   * Add a new interceptor to the stack
   *
   * @param {Function} fulfilled The function to handle `then` for a `Promise`
   * @param {Function} rejected The function to handle `reject` for a `Promise`
   *
   * @return {Number} An ID used to remove interceptor later
   */
  use(fulfilled, rejected, options) {
    this.handlers.push({
      fulfilled,
      rejected,
      synchronous: options ? options.synchronous : false,
      runWhen: options ? options.runWhen : null
    });
    return this.handlers.length - 1;
  }

  /**
   * Remove an interceptor from the stack
   *
   * @param {Number} id The ID that was returned by `use`
   *
   * @returns {Boolean} `true` if the interceptor was removed, `false` otherwise
   */
  eject(id) {
    if (this.handlers[id]) {
      this.handlers[id] = null;
    }
  }

  /**
   * Clear all interceptors from the stack
   *
   * @returns {void}
   */
  clear() {
    if (this.handlers) {
      this.handlers = [];
    }
  }

  /**
   * Iterate over all the registered interceptors
   *
   * This method is particularly useful for skipping over any
   * interceptors that may have become `null` calling `eject`.
   *
   * @param {Function} fn The function to call for each interceptor
   *
   * @returns {void}
   */
  forEach(fn) {
    utils.forEach(this.handlers, function forEachHandler(h) {
      if (h !== null) {
        fn(h);
      }
    });
  }
}

var InterceptorManager$1 = InterceptorManager;

var transitionalDefaults = {
  silentJSONParsing: true,
  forcedJSONParsing: true,
  clarifyTimeoutError: false
};

var URLSearchParams$1 = typeof URLSearchParams !== 'undefined' ? URLSearchParams : AxiosURLSearchParams;

var FormData$1 = typeof FormData !== 'undefined' ? FormData : null;

var Blob$1 = typeof Blob !== 'undefined' ? Blob : null;

/**
 * Determine if we're running in a standard browser environment
 *
 * This allows axios to run in a web worker, and react-native.
 * Both environments support XMLHttpRequest, but not fully standard globals.
 *
 * web workers:
 *  typeof window -> undefined
 *  typeof document -> undefined
 *
 * react-native:
 *  navigator.product -> 'ReactNative'
 * nativescript
 *  navigator.product -> 'NativeScript' or 'NS'
 *
 * @returns {boolean}
 */
const isStandardBrowserEnv = (() => {
  let product;
  if (typeof navigator !== 'undefined' && (
    (product = navigator.product) === 'ReactNative' ||
    product === 'NativeScript' ||
    product === 'NS')
  ) {
    return false;
  }

  return typeof window !== 'undefined' && typeof document !== 'undefined';
})();

/**
 * Determine if we're running in a standard browser webWorker environment
 *
 * Although the `isStandardBrowserEnv` method indicates that
 * `allows axios to run in a web worker`, the WebWorker will still be
 * filtered out due to its judgment standard
 * `typeof window !== 'undefined' && typeof document !== 'undefined'`.
 * This leads to a problem when axios post `FormData` in webWorker
 */
 const isStandardBrowserWebWorkerEnv = (() => {
  return (
    typeof WorkerGlobalScope !== 'undefined' &&
    // eslint-disable-next-line no-undef
    self instanceof WorkerGlobalScope &&
    typeof self.importScripts === 'function'
  );
})();


var platform = {
  isBrowser: true,
  classes: {
    URLSearchParams: URLSearchParams$1,
    FormData: FormData$1,
    Blob: Blob$1
  },
  isStandardBrowserEnv,
  isStandardBrowserWebWorkerEnv,
  protocols: ['http', 'https', 'file', 'blob', 'url', 'data']
};

function toURLEncodedForm(data, options) {
  return toFormData(data, new platform.classes.URLSearchParams(), Object.assign({
    visitor: function(value, key, path, helpers) {
      if (platform.isNode && utils.isBuffer(value)) {
        this.append(key, value.toString('base64'));
        return false;
      }

      return helpers.defaultVisitor.apply(this, arguments);
    }
  }, options));
}

/**
 * It takes a string like `foo[x][y][z]` and returns an array like `['foo', 'x', 'y', 'z']
 *
 * @param {string} name - The name of the property to get.
 *
 * @returns An array of strings.
 */
function parsePropPath(name) {
  // foo[x][y][z]
  // foo.x.y.z
  // foo-x-y-z
  // foo x y z
  return utils.matchAll(/\w+|\[(\w*)]/g, name).map(match => {
    return match[0] === '[]' ? '' : match[1] || match[0];
  });
}

/**
 * Convert an array to an object.
 *
 * @param {Array<any>} arr - The array to convert to an object.
 *
 * @returns An object with the same keys and values as the array.
 */
function arrayToObject(arr) {
  const obj = {};
  const keys = Object.keys(arr);
  let i;
  const len = keys.length;
  let key;
  for (i = 0; i < len; i++) {
    key = keys[i];
    obj[key] = arr[key];
  }
  return obj;
}

/**
 * It takes a FormData object and returns a JavaScript object
 *
 * @param {string} formData The FormData object to convert to JSON.
 *
 * @returns {Object<string, any> | null} The converted object.
 */
function formDataToJSON(formData) {
  function buildPath(path, value, target, index) {
    let name = path[index++];
    const isNumericKey = Number.isFinite(+name);
    const isLast = index >= path.length;
    name = !name && utils.isArray(target) ? target.length : name;

    if (isLast) {
      if (utils.hasOwnProp(target, name)) {
        target[name] = [target[name], value];
      } else {
        target[name] = value;
      }

      return !isNumericKey;
    }

    if (!target[name] || !utils.isObject(target[name])) {
      target[name] = [];
    }

    const result = buildPath(path, value, target[name], index);

    if (result && utils.isArray(target[name])) {
      target[name] = arrayToObject(target[name]);
    }

    return !isNumericKey;
  }

  if (utils.isFormData(formData) && utils.isFunction(formData.entries)) {
    const obj = {};

    utils.forEachEntry(formData, (name, value) => {
      buildPath(parsePropPath(name), value, obj, 0);
    });

    return obj;
  }

  return null;
}

/**
 * It takes a string, tries to parse it, and if it fails, it returns the stringified version
 * of the input
 *
 * @param {any} rawValue - The value to be stringified.
 * @param {Function} parser - A function that parses a string into a JavaScript object.
 * @param {Function} encoder - A function that takes a value and returns a string.
 *
 * @returns {string} A stringified version of the rawValue.
 */
function stringifySafely(rawValue, parser, encoder) {
  if (utils.isString(rawValue)) {
    try {
      (parser || JSON.parse)(rawValue);
      return utils.trim(rawValue);
    } catch (e) {
      if (e.name !== 'SyntaxError') {
        throw e;
      }
    }
  }

  return (encoder || JSON.stringify)(rawValue);
}

const defaults = {

  transitional: transitionalDefaults,

  adapter: ['xhr', 'http'],

  transformRequest: [function transformRequest(data, headers) {
    const contentType = headers.getContentType() || '';
    const hasJSONContentType = contentType.indexOf('application/json') > -1;
    const isObjectPayload = utils.isObject(data);

    if (isObjectPayload && utils.isHTMLForm(data)) {
      data = new FormData(data);
    }

    const isFormData = utils.isFormData(data);

    if (isFormData) {
      if (!hasJSONContentType) {
        return data;
      }
      return hasJSONContentType ? JSON.stringify(formDataToJSON(data)) : data;
    }

    if (utils.isArrayBuffer(data) ||
      utils.isBuffer(data) ||
      utils.isStream(data) ||
      utils.isFile(data) ||
      utils.isBlob(data)
    ) {
      return data;
    }
    if (utils.isArrayBufferView(data)) {
      return data.buffer;
    }
    if (utils.isURLSearchParams(data)) {
      headers.setContentType('application/x-www-form-urlencoded;charset=utf-8', false);
      return data.toString();
    }

    let isFileList;

    if (isObjectPayload) {
      if (contentType.indexOf('application/x-www-form-urlencoded') > -1) {
        return toURLEncodedForm(data, this.formSerializer).toString();
      }

      if ((isFileList = utils.isFileList(data)) || contentType.indexOf('multipart/form-data') > -1) {
        const _FormData = this.env && this.env.FormData;

        return toFormData(
          isFileList ? {'files[]': data} : data,
          _FormData && new _FormData(),
          this.formSerializer
        );
      }
    }

    if (isObjectPayload || hasJSONContentType ) {
      headers.setContentType('application/json', false);
      return stringifySafely(data);
    }

    return data;
  }],

  transformResponse: [function transformResponse(data) {
    const transitional = this.transitional || defaults.transitional;
    const forcedJSONParsing = transitional && transitional.forcedJSONParsing;
    const JSONRequested = this.responseType === 'json';

    if (data && utils.isString(data) && ((forcedJSONParsing && !this.responseType) || JSONRequested)) {
      const silentJSONParsing = transitional && transitional.silentJSONParsing;
      const strictJSONParsing = !silentJSONParsing && JSONRequested;

      try {
        return JSON.parse(data);
      } catch (e) {
        if (strictJSONParsing) {
          if (e.name === 'SyntaxError') {
            throw AxiosError.from(e, AxiosError.ERR_BAD_RESPONSE, this, null, this.response);
          }
          throw e;
        }
      }
    }

    return data;
  }],

  /**
   * A timeout in milliseconds to abort a request. If set to 0 (default) a
   * timeout is not created.
   */
  timeout: 0,

  xsrfCookieName: 'XSRF-TOKEN',
  xsrfHeaderName: 'X-XSRF-TOKEN',

  maxContentLength: -1,
  maxBodyLength: -1,

  env: {
    FormData: platform.classes.FormData,
    Blob: platform.classes.Blob
  },

  validateStatus: function validateStatus(status) {
    return status >= 200 && status < 300;
  },

  headers: {
    common: {
      'Accept': 'application/json, text/plain, */*',
      'Content-Type': undefined
    }
  }
};

utils.forEach(['delete', 'get', 'head', 'post', 'put', 'patch'], (method) => {
  defaults.headers[method] = {};
});

var defaults$1 = defaults;

// RawAxiosHeaders whose duplicates are ignored by node
// c.f. https://nodejs.org/api/http.html#http_message_headers
const ignoreDuplicateOf = utils.toObjectSet([
  'age', 'authorization', 'content-length', 'content-type', 'etag',
  'expires', 'from', 'host', 'if-modified-since', 'if-unmodified-since',
  'last-modified', 'location', 'max-forwards', 'proxy-authorization',
  'referer', 'retry-after', 'user-agent'
]);

/**
 * Parse headers into an object
 *
 * ```
 * Date: Wed, 27 Aug 2014 08:58:49 GMT
 * Content-Type: application/json
 * Connection: keep-alive
 * Transfer-Encoding: chunked
 * ```
 *
 * @param {String} rawHeaders Headers needing to be parsed
 *
 * @returns {Object} Headers parsed into an object
 */
var parseHeaders = rawHeaders => {
  const parsed = {};
  let key;
  let val;
  let i;

  rawHeaders && rawHeaders.split('\n').forEach(function parser(line) {
    i = line.indexOf(':');
    key = line.substring(0, i).trim().toLowerCase();
    val = line.substring(i + 1).trim();

    if (!key || (parsed[key] && ignoreDuplicateOf[key])) {
      return;
    }

    if (key === 'set-cookie') {
      if (parsed[key]) {
        parsed[key].push(val);
      } else {
        parsed[key] = [val];
      }
    } else {
      parsed[key] = parsed[key] ? parsed[key] + ', ' + val : val;
    }
  });

  return parsed;
};

const $internals = Symbol('internals');

function normalizeHeader(header) {
  return header && String(header).trim().toLowerCase();
}

function normalizeValue(value) {
  if (value === false || value == null) {
    return value;
  }

  return utils.isArray(value) ? value.map(normalizeValue) : String(value);
}

function parseTokens(str) {
  const tokens = Object.create(null);
  const tokensRE = /([^\s,;=]+)\s*(?:=\s*([^,;]+))?/g;
  let match;

  while ((match = tokensRE.exec(str))) {
    tokens[match[1]] = match[2];
  }

  return tokens;
}

const isValidHeaderName = (str) => /^[-_a-zA-Z0-9^`|~,!#$%&'*+.]+$/.test(str.trim());

function matchHeaderValue(context, value, header, filter, isHeaderNameFilter) {
  if (utils.isFunction(filter)) {
    return filter.call(this, value, header);
  }

  if (isHeaderNameFilter) {
    value = header;
  }

  if (!utils.isString(value)) return;

  if (utils.isString(filter)) {
    return value.indexOf(filter) !== -1;
  }

  if (utils.isRegExp(filter)) {
    return filter.test(value);
  }
}

function formatHeader(header) {
  return header.trim()
    .toLowerCase().replace(/([a-z\d])(\w*)/g, (w, char, str) => {
      return char.toUpperCase() + str;
    });
}

function buildAccessors(obj, header) {
  const accessorName = utils.toCamelCase(' ' + header);

  ['get', 'set', 'has'].forEach(methodName => {
    Object.defineProperty(obj, methodName + accessorName, {
      value: function(arg1, arg2, arg3) {
        return this[methodName].call(this, header, arg1, arg2, arg3);
      },
      configurable: true
    });
  });
}

class AxiosHeaders {
  constructor(headers) {
    headers && this.set(headers);
  }

  set(header, valueOrRewrite, rewrite) {
    const self = this;

    function setHeader(_value, _header, _rewrite) {
      const lHeader = normalizeHeader(_header);

      if (!lHeader) {
        throw new Error('header name must be a non-empty string');
      }

      const key = utils.findKey(self, lHeader);

      if(!key || self[key] === undefined || _rewrite === true || (_rewrite === undefined && self[key] !== false)) {
        self[key || _header] = normalizeValue(_value);
      }
    }

    const setHeaders = (headers, _rewrite) =>
      utils.forEach(headers, (_value, _header) => setHeader(_value, _header, _rewrite));

    if (utils.isPlainObject(header) || header instanceof this.constructor) {
      setHeaders(header, valueOrRewrite);
    } else if(utils.isString(header) && (header = header.trim()) && !isValidHeaderName(header)) {
      setHeaders(parseHeaders(header), valueOrRewrite);
    } else {
      header != null && setHeader(valueOrRewrite, header, rewrite);
    }

    return this;
  }

  get(header, parser) {
    header = normalizeHeader(header);

    if (header) {
      const key = utils.findKey(this, header);

      if (key) {
        const value = this[key];

        if (!parser) {
          return value;
        }

        if (parser === true) {
          return parseTokens(value);
        }

        if (utils.isFunction(parser)) {
          return parser.call(this, value, key);
        }

        if (utils.isRegExp(parser)) {
          return parser.exec(value);
        }

        throw new TypeError('parser must be boolean|regexp|function');
      }
    }
  }

  has(header, matcher) {
    header = normalizeHeader(header);

    if (header) {
      const key = utils.findKey(this, header);

      return !!(key && this[key] !== undefined && (!matcher || matchHeaderValue(this, this[key], key, matcher)));
    }

    return false;
  }

  delete(header, matcher) {
    const self = this;
    let deleted = false;

    function deleteHeader(_header) {
      _header = normalizeHeader(_header);

      if (_header) {
        const key = utils.findKey(self, _header);

        if (key && (!matcher || matchHeaderValue(self, self[key], key, matcher))) {
          delete self[key];

          deleted = true;
        }
      }
    }

    if (utils.isArray(header)) {
      header.forEach(deleteHeader);
    } else {
      deleteHeader(header);
    }

    return deleted;
  }

  clear(matcher) {
    const keys = Object.keys(this);
    let i = keys.length;
    let deleted = false;

    while (i--) {
      const key = keys[i];
      if(!matcher || matchHeaderValue(this, this[key], key, matcher, true)) {
        delete this[key];
        deleted = true;
      }
    }

    return deleted;
  }

  normalize(format) {
    const self = this;
    const headers = {};

    utils.forEach(this, (value, header) => {
      const key = utils.findKey(headers, header);

      if (key) {
        self[key] = normalizeValue(value);
        delete self[header];
        return;
      }

      const normalized = format ? formatHeader(header) : String(header).trim();

      if (normalized !== header) {
        delete self[header];
      }

      self[normalized] = normalizeValue(value);

      headers[normalized] = true;
    });

    return this;
  }

  concat(...targets) {
    return this.constructor.concat(this, ...targets);
  }

  toJSON(asStrings) {
    const obj = Object.create(null);

    utils.forEach(this, (value, header) => {
      value != null && value !== false && (obj[header] = asStrings && utils.isArray(value) ? value.join(', ') : value);
    });

    return obj;
  }

  [Symbol.iterator]() {
    return Object.entries(this.toJSON())[Symbol.iterator]();
  }

  toString() {
    return Object.entries(this.toJSON()).map(([header, value]) => header + ': ' + value).join('\n');
  }

  get [Symbol.toStringTag]() {
    return 'AxiosHeaders';
  }

  static from(thing) {
    return thing instanceof this ? thing : new this(thing);
  }

  static concat(first, ...targets) {
    const computed = new this(first);

    targets.forEach((target) => computed.set(target));

    return computed;
  }

  static accessor(header) {
    const internals = this[$internals] = (this[$internals] = {
      accessors: {}
    });

    const accessors = internals.accessors;
    const prototype = this.prototype;

    function defineAccessor(_header) {
      const lHeader = normalizeHeader(_header);

      if (!accessors[lHeader]) {
        buildAccessors(prototype, _header);
        accessors[lHeader] = true;
      }
    }

    utils.isArray(header) ? header.forEach(defineAccessor) : defineAccessor(header);

    return this;
  }
}

AxiosHeaders.accessor(['Content-Type', 'Content-Length', 'Accept', 'Accept-Encoding', 'User-Agent', 'Authorization']);

// reserved names hotfix
utils.reduceDescriptors(AxiosHeaders.prototype, ({value}, key) => {
  let mapped = key[0].toUpperCase() + key.slice(1); // map `set` => `Set`
  return {
    get: () => value,
    set(headerValue) {
      this[mapped] = headerValue;
    }
  }
});

utils.freezeMethods(AxiosHeaders);

var AxiosHeaders$1 = AxiosHeaders;

/**
 * Transform the data for a request or a response
 *
 * @param {Array|Function} fns A single function or Array of functions
 * @param {?Object} response The response object
 *
 * @returns {*} The resulting transformed data
 */
function transformData(fns, response) {
  const config = this || defaults$1;
  const context = response || config;
  const headers = AxiosHeaders$1.from(context.headers);
  let data = context.data;

  utils.forEach(fns, function transform(fn) {
    data = fn.call(config, data, headers.normalize(), response ? response.status : undefined);
  });

  headers.normalize();

  return data;
}

function isCancel(value) {
  return !!(value && value.__CANCEL__);
}

/**
 * A `CanceledError` is an object that is thrown when an operation is canceled.
 *
 * @param {string=} message The message.
 * @param {Object=} config The config.
 * @param {Object=} request The request.
 *
 * @returns {CanceledError} The created error.
 */
function CanceledError(message, config, request) {
  // eslint-disable-next-line no-eq-null,eqeqeq
  AxiosError.call(this, message == null ? 'canceled' : message, AxiosError.ERR_CANCELED, config, request);
  this.name = 'CanceledError';
}

utils.inherits(CanceledError, AxiosError, {
  __CANCEL__: true
});

/**
 * Resolve or reject a Promise based on response status.
 *
 * @param {Function} resolve A function that resolves the promise.
 * @param {Function} reject A function that rejects the promise.
 * @param {object} response The response.
 *
 * @returns {object} The response.
 */
function settle(resolve, reject, response) {
  const validateStatus = response.config.validateStatus;
  if (!response.status || !validateStatus || validateStatus(response.status)) {
    resolve(response);
  } else {
    reject(new AxiosError(
      'Request failed with status code ' + response.status,
      [AxiosError.ERR_BAD_REQUEST, AxiosError.ERR_BAD_RESPONSE][Math.floor(response.status / 100) - 4],
      response.config,
      response.request,
      response
    ));
  }
}

var cookies = platform.isStandardBrowserEnv ?

// Standard browser envs support document.cookie
  (function standardBrowserEnv() {
    return {
      write: function write(name, value, expires, path, domain, secure) {
        const cookie = [];
        cookie.push(name + '=' + encodeURIComponent(value));

        if (utils.isNumber(expires)) {
          cookie.push('expires=' + new Date(expires).toGMTString());
        }

        if (utils.isString(path)) {
          cookie.push('path=' + path);
        }

        if (utils.isString(domain)) {
          cookie.push('domain=' + domain);
        }

        if (secure === true) {
          cookie.push('secure');
        }

        document.cookie = cookie.join('; ');
      },

      read: function read(name) {
        const match = document.cookie.match(new RegExp('(^|;\\s*)(' + name + ')=([^;]*)'));
        return (match ? decodeURIComponent(match[3]) : null);
      },

      remove: function remove(name) {
        this.write(name, '', Date.now() - 86400000);
      }
    };
  })() :

// Non standard browser env (web workers, react-native) lack needed support.
  (function nonStandardBrowserEnv() {
    return {
      write: function write() {},
      read: function read() { return null; },
      remove: function remove() {}
    };
  })();

/**
 * Determines whether the specified URL is absolute
 *
 * @param {string} url The URL to test
 *
 * @returns {boolean} True if the specified URL is absolute, otherwise false
 */
function isAbsoluteURL(url) {
  // A URL is considered absolute if it begins with "<scheme>://" or "//" (protocol-relative URL).
  // RFC 3986 defines scheme name as a sequence of characters beginning with a letter and followed
  // by any combination of letters, digits, plus, period, or hyphen.
  return /^([a-z][a-z\d+\-.]*:)?\/\//i.test(url);
}

/**
 * Creates a new URL by combining the specified URLs
 *
 * @param {string} baseURL The base URL
 * @param {string} relativeURL The relative URL
 *
 * @returns {string} The combined URL
 */
function combineURLs(baseURL, relativeURL) {
  return relativeURL
    ? baseURL.replace(/\/+$/, '') + '/' + relativeURL.replace(/^\/+/, '')
    : baseURL;
}

/**
 * Creates a new URL by combining the baseURL with the requestedURL,
 * only when the requestedURL is not already an absolute URL.
 * If the requestURL is absolute, this function returns the requestedURL untouched.
 *
 * @param {string} baseURL The base URL
 * @param {string} requestedURL Absolute or relative URL to combine
 *
 * @returns {string} The combined full path
 */
function buildFullPath(baseURL, requestedURL) {
  if (baseURL && !isAbsoluteURL(requestedURL)) {
    return combineURLs(baseURL, requestedURL);
  }
  return requestedURL;
}

var isURLSameOrigin = platform.isStandardBrowserEnv ?

// Standard browser envs have full support of the APIs needed to test
// whether the request URL is of the same origin as current location.
  (function standardBrowserEnv() {
    const msie = /(msie|trident)/i.test(navigator.userAgent);
    const urlParsingNode = document.createElement('a');
    let originURL;

    /**
    * Parse a URL to discover it's components
    *
    * @param {String} url The URL to be parsed
    * @returns {Object}
    */
    function resolveURL(url) {
      let href = url;

      if (msie) {
        // IE needs attribute set twice to normalize properties
        urlParsingNode.setAttribute('href', href);
        href = urlParsingNode.href;
      }

      urlParsingNode.setAttribute('href', href);

      // urlParsingNode provides the UrlUtils interface - http://url.spec.whatwg.org/#urlutils
      return {
        href: urlParsingNode.href,
        protocol: urlParsingNode.protocol ? urlParsingNode.protocol.replace(/:$/, '') : '',
        host: urlParsingNode.host,
        search: urlParsingNode.search ? urlParsingNode.search.replace(/^\?/, '') : '',
        hash: urlParsingNode.hash ? urlParsingNode.hash.replace(/^#/, '') : '',
        hostname: urlParsingNode.hostname,
        port: urlParsingNode.port,
        pathname: (urlParsingNode.pathname.charAt(0) === '/') ?
          urlParsingNode.pathname :
          '/' + urlParsingNode.pathname
      };
    }

    originURL = resolveURL(window.location.href);

    /**
    * Determine if a URL shares the same origin as the current location
    *
    * @param {String} requestURL The URL to test
    * @returns {boolean} True if URL shares the same origin, otherwise false
    */
    return function isURLSameOrigin(requestURL) {
      const parsed = (utils.isString(requestURL)) ? resolveURL(requestURL) : requestURL;
      return (parsed.protocol === originURL.protocol &&
          parsed.host === originURL.host);
    };
  })() :

  // Non standard browser envs (web workers, react-native) lack needed support.
  (function nonStandardBrowserEnv() {
    return function isURLSameOrigin() {
      return true;
    };
  })();

function parseProtocol(url) {
  const match = /^([-+\w]{1,25})(:?\/\/|:)/.exec(url);
  return match && match[1] || '';
}

/**
 * Calculate data maxRate
 * @param {Number} [samplesCount= 10]
 * @param {Number} [min= 1000]
 * @returns {Function}
 */
function speedometer(samplesCount, min) {
  samplesCount = samplesCount || 10;
  const bytes = new Array(samplesCount);
  const timestamps = new Array(samplesCount);
  let head = 0;
  let tail = 0;
  let firstSampleTS;

  min = min !== undefined ? min : 1000;

  return function push(chunkLength) {
    const now = Date.now();

    const startedAt = timestamps[tail];

    if (!firstSampleTS) {
      firstSampleTS = now;
    }

    bytes[head] = chunkLength;
    timestamps[head] = now;

    let i = tail;
    let bytesCount = 0;

    while (i !== head) {
      bytesCount += bytes[i++];
      i = i % samplesCount;
    }

    head = (head + 1) % samplesCount;

    if (head === tail) {
      tail = (tail + 1) % samplesCount;
    }

    if (now - firstSampleTS < min) {
      return;
    }

    const passed = startedAt && now - startedAt;

    return passed ? Math.round(bytesCount * 1000 / passed) : undefined;
  };
}

function progressEventReducer(listener, isDownloadStream) {
  let bytesNotified = 0;
  const _speedometer = speedometer(50, 250);

  return e => {
    const loaded = e.loaded;
    const total = e.lengthComputable ? e.total : undefined;
    const progressBytes = loaded - bytesNotified;
    const rate = _speedometer(progressBytes);
    const inRange = loaded <= total;

    bytesNotified = loaded;

    const data = {
      loaded,
      total,
      progress: total ? (loaded / total) : undefined,
      bytes: progressBytes,
      rate: rate ? rate : undefined,
      estimated: rate && total && inRange ? (total - loaded) / rate : undefined,
      event: e
    };

    data[isDownloadStream ? 'download' : 'upload'] = true;

    listener(data);
  };
}

const isXHRAdapterSupported = typeof XMLHttpRequest !== 'undefined';

var xhrAdapter = isXHRAdapterSupported && function (config) {
  return new Promise(function dispatchXhrRequest(resolve, reject) {
    let requestData = config.data;
    const requestHeaders = AxiosHeaders$1.from(config.headers).normalize();
    const responseType = config.responseType;
    let onCanceled;
    function done() {
      if (config.cancelToken) {
        config.cancelToken.unsubscribe(onCanceled);
      }

      if (config.signal) {
        config.signal.removeEventListener('abort', onCanceled);
      }
    }

    let contentType;

    if (utils.isFormData(requestData)) {
      if (platform.isStandardBrowserEnv || platform.isStandardBrowserWebWorkerEnv) {
        requestHeaders.setContentType(false); // Let the browser set it
      } else if(!requestHeaders.getContentType(/^\s*multipart\/form-data/)){
        requestHeaders.setContentType('multipart/form-data'); // mobile/desktop app frameworks
      } else if(utils.isString(contentType = requestHeaders.getContentType())){
        // fix semicolon duplication issue for ReactNative FormData implementation
        requestHeaders.setContentType(contentType.replace(/^\s*(multipart\/form-data);+/, '$1'));
      }
    }

    let request = new XMLHttpRequest();

    // HTTP basic authentication
    if (config.auth) {
      const username = config.auth.username || '';
      const password = config.auth.password ? unescape(encodeURIComponent(config.auth.password)) : '';
      requestHeaders.set('Authorization', 'Basic ' + btoa(username + ':' + password));
    }

    const fullPath = buildFullPath(config.baseURL, config.url);

    request.open(config.method.toUpperCase(), buildURL(fullPath, config.params, config.paramsSerializer), true);

    // Set the request timeout in MS
    request.timeout = config.timeout;

    function onloadend() {
      if (!request) {
        return;
      }
      // Prepare the response
      const responseHeaders = AxiosHeaders$1.from(
        'getAllResponseHeaders' in request && request.getAllResponseHeaders()
      );
      const responseData = !responseType || responseType === 'text' || responseType === 'json' ?
        request.responseText : request.response;
      const response = {
        data: responseData,
        status: request.status,
        statusText: request.statusText,
        headers: responseHeaders,
        config,
        request
      };

      settle(function _resolve(value) {
        resolve(value);
        done();
      }, function _reject(err) {
        reject(err);
        done();
      }, response);

      // Clean up request
      request = null;
    }

    if ('onloadend' in request) {
      // Use onloadend if available
      request.onloadend = onloadend;
    } else {
      // Listen for ready state to emulate onloadend
      request.onreadystatechange = function handleLoad() {
        if (!request || request.readyState !== 4) {
          return;
        }

        // The request errored out and we didn't get a response, this will be
        // handled by onerror instead
        // With one exception: request that using file: protocol, most browsers
        // will return status as 0 even though it's a successful request
        if (request.status === 0 && !(request.responseURL && request.responseURL.indexOf('file:') === 0)) {
          return;
        }
        // readystate handler is calling before onerror or ontimeout handlers,
        // so we should call onloadend on the next 'tick'
        setTimeout(onloadend);
      };
    }

    // Handle browser request cancellation (as opposed to a manual cancellation)
    request.onabort = function handleAbort() {
      if (!request) {
        return;
      }

      reject(new AxiosError('Request aborted', AxiosError.ECONNABORTED, config, request));

      // Clean up request
      request = null;
    };

    // Handle low level network errors
    request.onerror = function handleError() {
      // Real errors are hidden from us by the browser
      // onerror should only fire if it's a network error
      reject(new AxiosError('Network Error', AxiosError.ERR_NETWORK, config, request));

      // Clean up request
      request = null;
    };

    // Handle timeout
    request.ontimeout = function handleTimeout() {
      let timeoutErrorMessage = config.timeout ? 'timeout of ' + config.timeout + 'ms exceeded' : 'timeout exceeded';
      const transitional = config.transitional || transitionalDefaults;
      if (config.timeoutErrorMessage) {
        timeoutErrorMessage = config.timeoutErrorMessage;
      }
      reject(new AxiosError(
        timeoutErrorMessage,
        transitional.clarifyTimeoutError ? AxiosError.ETIMEDOUT : AxiosError.ECONNABORTED,
        config,
        request));

      // Clean up request
      request = null;
    };

    // Add xsrf header
    // This is only done if running in a standard browser environment.
    // Specifically not if we're in a web worker, or react-native.
    if (platform.isStandardBrowserEnv) {
      // Add xsrf header
      const xsrfValue = (config.withCredentials || isURLSameOrigin(fullPath))
        && config.xsrfCookieName && cookies.read(config.xsrfCookieName);

      if (xsrfValue) {
        requestHeaders.set(config.xsrfHeaderName, xsrfValue);
      }
    }

    // Remove Content-Type if data is undefined
    requestData === undefined && requestHeaders.setContentType(null);

    // Add headers to the request
    if ('setRequestHeader' in request) {
      utils.forEach(requestHeaders.toJSON(), function setRequestHeader(val, key) {
        request.setRequestHeader(key, val);
      });
    }

    // Add withCredentials to request if needed
    if (!utils.isUndefined(config.withCredentials)) {
      request.withCredentials = !!config.withCredentials;
    }

    // Add responseType to request if needed
    if (responseType && responseType !== 'json') {
      request.responseType = config.responseType;
    }

    // Handle progress if needed
    if (typeof config.onDownloadProgress === 'function') {
      request.addEventListener('progress', progressEventReducer(config.onDownloadProgress, true));
    }

    // Not all browsers support upload events
    if (typeof config.onUploadProgress === 'function' && request.upload) {
      request.upload.addEventListener('progress', progressEventReducer(config.onUploadProgress));
    }

    if (config.cancelToken || config.signal) {
      // Handle cancellation
      // eslint-disable-next-line func-names
      onCanceled = cancel => {
        if (!request) {
          return;
        }
        reject(!cancel || cancel.type ? new CanceledError(null, config, request) : cancel);
        request.abort();
        request = null;
      };

      config.cancelToken && config.cancelToken.subscribe(onCanceled);
      if (config.signal) {
        config.signal.aborted ? onCanceled() : config.signal.addEventListener('abort', onCanceled);
      }
    }

    const protocol = parseProtocol(fullPath);

    if (protocol && platform.protocols.indexOf(protocol) === -1) {
      reject(new AxiosError('Unsupported protocol ' + protocol + ':', AxiosError.ERR_BAD_REQUEST, config));
      return;
    }


    // Send the request
    request.send(requestData || null);
  });
};

const knownAdapters = {
  http: httpAdapter,
  xhr: xhrAdapter
};

utils.forEach(knownAdapters, (fn, value) => {
  if (fn) {
    try {
      Object.defineProperty(fn, 'name', {value});
    } catch (e) {
      // eslint-disable-next-line no-empty
    }
    Object.defineProperty(fn, 'adapterName', {value});
  }
});

const renderReason = (reason) => `- ${reason}`;

const isResolvedHandle = (adapter) => utils.isFunction(adapter) || adapter === null || adapter === false;

var adapters = {
  getAdapter: (adapters) => {
    adapters = utils.isArray(adapters) ? adapters : [adapters];

    const {length} = adapters;
    let nameOrAdapter;
    let adapter;

    const rejectedReasons = {};

    for (let i = 0; i < length; i++) {
      nameOrAdapter = adapters[i];
      let id;

      adapter = nameOrAdapter;

      if (!isResolvedHandle(nameOrAdapter)) {
        adapter = knownAdapters[(id = String(nameOrAdapter)).toLowerCase()];

        if (adapter === undefined) {
          throw new AxiosError(`Unknown adapter '${id}'`);
        }
      }

      if (adapter) {
        break;
      }

      rejectedReasons[id || '#' + i] = adapter;
    }

    if (!adapter) {

      const reasons = Object.entries(rejectedReasons)
        .map(([id, state]) => `adapter ${id} ` +
          (state === false ? 'is not supported by the environment' : 'is not available in the build')
        );

      let s = length ?
        (reasons.length > 1 ? 'since :\n' + reasons.map(renderReason).join('\n') : ' ' + renderReason(reasons[0])) :
        'as no adapter specified';

      throw new AxiosError(
        `There is no suitable adapter to dispatch the request ` + s,
        'ERR_NOT_SUPPORT'
      );
    }

    return adapter;
  },
  adapters: knownAdapters
};

/**
 * Throws a `CanceledError` if cancellation has been requested.
 *
 * @param {Object} config The config that is to be used for the request
 *
 * @returns {void}
 */
function throwIfCancellationRequested(config) {
  if (config.cancelToken) {
    config.cancelToken.throwIfRequested();
  }

  if (config.signal && config.signal.aborted) {
    throw new CanceledError(null, config);
  }
}

/**
 * Dispatch a request to the server using the configured adapter.
 *
 * @param {object} config The config that is to be used for the request
 *
 * @returns {Promise} The Promise to be fulfilled
 */
function dispatchRequest(config) {
  throwIfCancellationRequested(config);

  config.headers = AxiosHeaders$1.from(config.headers);

  // Transform request data
  config.data = transformData.call(
    config,
    config.transformRequest
  );

  if (['post', 'put', 'patch'].indexOf(config.method) !== -1) {
    config.headers.setContentType('application/x-www-form-urlencoded', false);
  }

  const adapter = adapters.getAdapter(config.adapter || defaults$1.adapter);

  return adapter(config).then(function onAdapterResolution(response) {
    throwIfCancellationRequested(config);

    // Transform response data
    response.data = transformData.call(
      config,
      config.transformResponse,
      response
    );

    response.headers = AxiosHeaders$1.from(response.headers);

    return response;
  }, function onAdapterRejection(reason) {
    if (!isCancel(reason)) {
      throwIfCancellationRequested(config);

      // Transform response data
      if (reason && reason.response) {
        reason.response.data = transformData.call(
          config,
          config.transformResponse,
          reason.response
        );
        reason.response.headers = AxiosHeaders$1.from(reason.response.headers);
      }
    }

    return Promise.reject(reason);
  });
}

const headersToObject = (thing) => thing instanceof AxiosHeaders$1 ? thing.toJSON() : thing;

/**
 * Config-specific merge-function which creates a new config-object
 * by merging two configuration objects together.
 *
 * @param {Object} config1
 * @param {Object} config2
 *
 * @returns {Object} New object resulting from merging config2 to config1
 */
function mergeConfig(config1, config2) {
  // eslint-disable-next-line no-param-reassign
  config2 = config2 || {};
  const config = {};

  function getMergedValue(target, source, caseless) {
    if (utils.isPlainObject(target) && utils.isPlainObject(source)) {
      return utils.merge.call({caseless}, target, source);
    } else if (utils.isPlainObject(source)) {
      return utils.merge({}, source);
    } else if (utils.isArray(source)) {
      return source.slice();
    }
    return source;
  }

  // eslint-disable-next-line consistent-return
  function mergeDeepProperties(a, b, caseless) {
    if (!utils.isUndefined(b)) {
      return getMergedValue(a, b, caseless);
    } else if (!utils.isUndefined(a)) {
      return getMergedValue(undefined, a, caseless);
    }
  }

  // eslint-disable-next-line consistent-return
  function valueFromConfig2(a, b) {
    if (!utils.isUndefined(b)) {
      return getMergedValue(undefined, b);
    }
  }

  // eslint-disable-next-line consistent-return
  function defaultToConfig2(a, b) {
    if (!utils.isUndefined(b)) {
      return getMergedValue(undefined, b);
    } else if (!utils.isUndefined(a)) {
      return getMergedValue(undefined, a);
    }
  }

  // eslint-disable-next-line consistent-return
  function mergeDirectKeys(a, b, prop) {
    if (prop in config2) {
      return getMergedValue(a, b);
    } else if (prop in config1) {
      return getMergedValue(undefined, a);
    }
  }

  const mergeMap = {
    url: valueFromConfig2,
    method: valueFromConfig2,
    data: valueFromConfig2,
    baseURL: defaultToConfig2,
    transformRequest: defaultToConfig2,
    transformResponse: defaultToConfig2,
    paramsSerializer: defaultToConfig2,
    timeout: defaultToConfig2,
    timeoutMessage: defaultToConfig2,
    withCredentials: defaultToConfig2,
    adapter: defaultToConfig2,
    responseType: defaultToConfig2,
    xsrfCookieName: defaultToConfig2,
    xsrfHeaderName: defaultToConfig2,
    onUploadProgress: defaultToConfig2,
    onDownloadProgress: defaultToConfig2,
    decompress: defaultToConfig2,
    maxContentLength: defaultToConfig2,
    maxBodyLength: defaultToConfig2,
    beforeRedirect: defaultToConfig2,
    transport: defaultToConfig2,
    httpAgent: defaultToConfig2,
    httpsAgent: defaultToConfig2,
    cancelToken: defaultToConfig2,
    socketPath: defaultToConfig2,
    responseEncoding: defaultToConfig2,
    validateStatus: mergeDirectKeys,
    headers: (a, b) => mergeDeepProperties(headersToObject(a), headersToObject(b), true)
  };

  utils.forEach(Object.keys(Object.assign({}, config1, config2)), function computeConfigValue(prop) {
    const merge = mergeMap[prop] || mergeDeepProperties;
    const configValue = merge(config1[prop], config2[prop], prop);
    (utils.isUndefined(configValue) && merge !== mergeDirectKeys) || (config[prop] = configValue);
  });

  return config;
}

const VERSION = "1.5.1";

const validators$1 = {};

// eslint-disable-next-line func-names
['object', 'boolean', 'number', 'function', 'string', 'symbol'].forEach((type, i) => {
  validators$1[type] = function validator(thing) {
    return typeof thing === type || 'a' + (i < 1 ? 'n ' : ' ') + type;
  };
});

const deprecatedWarnings = {};

/**
 * Transitional option validator
 *
 * @param {function|boolean?} validator - set to false if the transitional option has been removed
 * @param {string?} version - deprecated version / removed since version
 * @param {string?} message - some message with additional info
 *
 * @returns {function}
 */
validators$1.transitional = function transitional(validator, version, message) {
  function formatMessage(opt, desc) {
    return '[Axios v' + VERSION + '] Transitional option \'' + opt + '\'' + desc + (message ? '. ' + message : '');
  }

  // eslint-disable-next-line func-names
  return (value, opt, opts) => {
    if (validator === false) {
      throw new AxiosError(
        formatMessage(opt, ' has been removed' + (version ? ' in ' + version : '')),
        AxiosError.ERR_DEPRECATED
      );
    }

    if (version && !deprecatedWarnings[opt]) {
      deprecatedWarnings[opt] = true;
      // eslint-disable-next-line no-console
      console.warn(
        formatMessage(
          opt,
          ' has been deprecated since v' + version + ' and will be removed in the near future'
        )
      );
    }

    return validator ? validator(value, opt, opts) : true;
  };
};

/**
 * Assert object's properties type
 *
 * @param {object} options
 * @param {object} schema
 * @param {boolean?} allowUnknown
 *
 * @returns {object}
 */

function assertOptions(options, schema, allowUnknown) {
  if (typeof options !== 'object') {
    throw new AxiosError('options must be an object', AxiosError.ERR_BAD_OPTION_VALUE);
  }
  const keys = Object.keys(options);
  let i = keys.length;
  while (i-- > 0) {
    const opt = keys[i];
    const validator = schema[opt];
    if (validator) {
      const value = options[opt];
      const result = value === undefined || validator(value, opt, options);
      if (result !== true) {
        throw new AxiosError('option ' + opt + ' must be ' + result, AxiosError.ERR_BAD_OPTION_VALUE);
      }
      continue;
    }
    if (allowUnknown !== true) {
      throw new AxiosError('Unknown option ' + opt, AxiosError.ERR_BAD_OPTION);
    }
  }
}

var validator = {
  assertOptions,
  validators: validators$1
};

const validators = validator.validators;

/**
 * Create a new instance of Axios
 *
 * @param {Object} instanceConfig The default config for the instance
 *
 * @return {Axios} A new instance of Axios
 */
class Axios {
  constructor(instanceConfig) {
    this.defaults = instanceConfig;
    this.interceptors = {
      request: new InterceptorManager$1(),
      response: new InterceptorManager$1()
    };
  }

  /**
   * Dispatch a request
   *
   * @param {String|Object} configOrUrl The config specific for this request (merged with this.defaults)
   * @param {?Object} config
   *
   * @returns {Promise} The Promise to be fulfilled
   */
  request(configOrUrl, config) {
    /*eslint no-param-reassign:0*/
    // Allow for axios('example/url'[, config]) a la fetch API
    if (typeof configOrUrl === 'string') {
      config = config || {};
      config.url = configOrUrl;
    } else {
      config = configOrUrl || {};
    }

    config = mergeConfig(this.defaults, config);

    const {transitional, paramsSerializer, headers} = config;

    if (transitional !== undefined) {
      validator.assertOptions(transitional, {
        silentJSONParsing: validators.transitional(validators.boolean),
        forcedJSONParsing: validators.transitional(validators.boolean),
        clarifyTimeoutError: validators.transitional(validators.boolean)
      }, false);
    }

    if (paramsSerializer != null) {
      if (utils.isFunction(paramsSerializer)) {
        config.paramsSerializer = {
          serialize: paramsSerializer
        };
      } else {
        validator.assertOptions(paramsSerializer, {
          encode: validators.function,
          serialize: validators.function
        }, true);
      }
    }

    // Set config.method
    config.method = (config.method || this.defaults.method || 'get').toLowerCase();

    // Flatten headers
    let contextHeaders = headers && utils.merge(
      headers.common,
      headers[config.method]
    );

    headers && utils.forEach(
      ['delete', 'get', 'head', 'post', 'put', 'patch', 'common'],
      (method) => {
        delete headers[method];
      }
    );

    config.headers = AxiosHeaders$1.concat(contextHeaders, headers);

    // filter out skipped interceptors
    const requestInterceptorChain = [];
    let synchronousRequestInterceptors = true;
    this.interceptors.request.forEach(function unshiftRequestInterceptors(interceptor) {
      if (typeof interceptor.runWhen === 'function' && interceptor.runWhen(config) === false) {
        return;
      }

      synchronousRequestInterceptors = synchronousRequestInterceptors && interceptor.synchronous;

      requestInterceptorChain.unshift(interceptor.fulfilled, interceptor.rejected);
    });

    const responseInterceptorChain = [];
    this.interceptors.response.forEach(function pushResponseInterceptors(interceptor) {
      responseInterceptorChain.push(interceptor.fulfilled, interceptor.rejected);
    });

    let promise;
    let i = 0;
    let len;

    if (!synchronousRequestInterceptors) {
      const chain = [dispatchRequest.bind(this), undefined];
      chain.unshift.apply(chain, requestInterceptorChain);
      chain.push.apply(chain, responseInterceptorChain);
      len = chain.length;

      promise = Promise.resolve(config);

      while (i < len) {
        promise = promise.then(chain[i++], chain[i++]);
      }

      return promise;
    }

    len = requestInterceptorChain.length;

    let newConfig = config;

    i = 0;

    while (i < len) {
      const onFulfilled = requestInterceptorChain[i++];
      const onRejected = requestInterceptorChain[i++];
      try {
        newConfig = onFulfilled(newConfig);
      } catch (error) {
        onRejected.call(this, error);
        break;
      }
    }

    try {
      promise = dispatchRequest.call(this, newConfig);
    } catch (error) {
      return Promise.reject(error);
    }

    i = 0;
    len = responseInterceptorChain.length;

    while (i < len) {
      promise = promise.then(responseInterceptorChain[i++], responseInterceptorChain[i++]);
    }

    return promise;
  }

  getUri(config) {
    config = mergeConfig(this.defaults, config);
    const fullPath = buildFullPath(config.baseURL, config.url);
    return buildURL(fullPath, config.params, config.paramsSerializer);
  }
}

// Provide aliases for supported request methods
utils.forEach(['delete', 'get', 'head', 'options'], function forEachMethodNoData(method) {
  /*eslint func-names:0*/
  Axios.prototype[method] = function(url, config) {
    return this.request(mergeConfig(config || {}, {
      method,
      url,
      data: (config || {}).data
    }));
  };
});

utils.forEach(['post', 'put', 'patch'], function forEachMethodWithData(method) {
  /*eslint func-names:0*/

  function generateHTTPMethod(isForm) {
    return function httpMethod(url, data, config) {
      return this.request(mergeConfig(config || {}, {
        method,
        headers: isForm ? {
          'Content-Type': 'multipart/form-data'
        } : {},
        url,
        data
      }));
    };
  }

  Axios.prototype[method] = generateHTTPMethod();

  Axios.prototype[method + 'Form'] = generateHTTPMethod(true);
});

var Axios$1 = Axios;

/**
 * A `CancelToken` is an object that can be used to request cancellation of an operation.
 *
 * @param {Function} executor The executor function.
 *
 * @returns {CancelToken}
 */
class CancelToken {
  constructor(executor) {
    if (typeof executor !== 'function') {
      throw new TypeError('executor must be a function.');
    }

    let resolvePromise;

    this.promise = new Promise(function promiseExecutor(resolve) {
      resolvePromise = resolve;
    });

    const token = this;

    // eslint-disable-next-line func-names
    this.promise.then(cancel => {
      if (!token._listeners) return;

      let i = token._listeners.length;

      while (i-- > 0) {
        token._listeners[i](cancel);
      }
      token._listeners = null;
    });

    // eslint-disable-next-line func-names
    this.promise.then = onfulfilled => {
      let _resolve;
      // eslint-disable-next-line func-names
      const promise = new Promise(resolve => {
        token.subscribe(resolve);
        _resolve = resolve;
      }).then(onfulfilled);

      promise.cancel = function reject() {
        token.unsubscribe(_resolve);
      };

      return promise;
    };

    executor(function cancel(message, config, request) {
      if (token.reason) {
        // Cancellation has already been requested
        return;
      }

      token.reason = new CanceledError(message, config, request);
      resolvePromise(token.reason);
    });
  }

  /**
   * Throws a `CanceledError` if cancellation has been requested.
   */
  throwIfRequested() {
    if (this.reason) {
      throw this.reason;
    }
  }

  /**
   * Subscribe to the cancel signal
   */

  subscribe(listener) {
    if (this.reason) {
      listener(this.reason);
      return;
    }

    if (this._listeners) {
      this._listeners.push(listener);
    } else {
      this._listeners = [listener];
    }
  }

  /**
   * Unsubscribe from the cancel signal
   */

  unsubscribe(listener) {
    if (!this._listeners) {
      return;
    }
    const index = this._listeners.indexOf(listener);
    if (index !== -1) {
      this._listeners.splice(index, 1);
    }
  }

  /**
   * Returns an object that contains a new `CancelToken` and a function that, when called,
   * cancels the `CancelToken`.
   */
  static source() {
    let cancel;
    const token = new CancelToken(function executor(c) {
      cancel = c;
    });
    return {
      token,
      cancel
    };
  }
}

var CancelToken$1 = CancelToken;

/**
 * Syntactic sugar for invoking a function and expanding an array for arguments.
 *
 * Common use case would be to use `Function.prototype.apply`.
 *
 *  ```js
 *  function f(x, y, z) {}
 *  var args = [1, 2, 3];
 *  f.apply(null, args);
 *  ```
 *
 * With `spread` this example can be re-written.
 *
 *  ```js
 *  spread(function(x, y, z) {})([1, 2, 3]);
 *  ```
 *
 * @param {Function} callback
 *
 * @returns {Function}
 */
function spread(callback) {
  return function wrap(arr) {
    return callback.apply(null, arr);
  };
}

/**
 * Determines whether the payload is an error thrown by Axios
 *
 * @param {*} payload The value to test
 *
 * @returns {boolean} True if the payload is an error thrown by Axios, otherwise false
 */
function isAxiosError(payload) {
  return utils.isObject(payload) && (payload.isAxiosError === true);
}

const HttpStatusCode = {
  Continue: 100,
  SwitchingProtocols: 101,
  Processing: 102,
  EarlyHints: 103,
  Ok: 200,
  Created: 201,
  Accepted: 202,
  NonAuthoritativeInformation: 203,
  NoContent: 204,
  ResetContent: 205,
  PartialContent: 206,
  MultiStatus: 207,
  AlreadyReported: 208,
  ImUsed: 226,
  MultipleChoices: 300,
  MovedPermanently: 301,
  Found: 302,
  SeeOther: 303,
  NotModified: 304,
  UseProxy: 305,
  Unused: 306,
  TemporaryRedirect: 307,
  PermanentRedirect: 308,
  BadRequest: 400,
  Unauthorized: 401,
  PaymentRequired: 402,
  Forbidden: 403,
  NotFound: 404,
  MethodNotAllowed: 405,
  NotAcceptable: 406,
  ProxyAuthenticationRequired: 407,
  RequestTimeout: 408,
  Conflict: 409,
  Gone: 410,
  LengthRequired: 411,
  PreconditionFailed: 412,
  PayloadTooLarge: 413,
  UriTooLong: 414,
  UnsupportedMediaType: 415,
  RangeNotSatisfiable: 416,
  ExpectationFailed: 417,
  ImATeapot: 418,
  MisdirectedRequest: 421,
  UnprocessableEntity: 422,
  Locked: 423,
  FailedDependency: 424,
  TooEarly: 425,
  UpgradeRequired: 426,
  PreconditionRequired: 428,
  TooManyRequests: 429,
  RequestHeaderFieldsTooLarge: 431,
  UnavailableForLegalReasons: 451,
  InternalServerError: 500,
  NotImplemented: 501,
  BadGateway: 502,
  ServiceUnavailable: 503,
  GatewayTimeout: 504,
  HttpVersionNotSupported: 505,
  VariantAlsoNegotiates: 506,
  InsufficientStorage: 507,
  LoopDetected: 508,
  NotExtended: 510,
  NetworkAuthenticationRequired: 511,
};

Object.entries(HttpStatusCode).forEach(([key, value]) => {
  HttpStatusCode[value] = key;
});

var HttpStatusCode$1 = HttpStatusCode;

/**
 * Create an instance of Axios
 *
 * @param {Object} defaultConfig The default config for the instance
 *
 * @returns {Axios} A new instance of Axios
 */
function createInstance(defaultConfig) {
  const context = new Axios$1(defaultConfig);
  const instance = bind(Axios$1.prototype.request, context);

  // Copy axios.prototype to instance
  utils.extend(instance, Axios$1.prototype, context, {allOwnKeys: true});

  // Copy context to instance
  utils.extend(instance, context, null, {allOwnKeys: true});

  // Factory for creating new instances
  instance.create = function create(instanceConfig) {
    return createInstance(mergeConfig(defaultConfig, instanceConfig));
  };

  return instance;
}

// Create the default instance to be exported
const axios = createInstance(defaults$1);

// Expose Axios class to allow class inheritance
axios.Axios = Axios$1;

// Expose Cancel & CancelToken
axios.CanceledError = CanceledError;
axios.CancelToken = CancelToken$1;
axios.isCancel = isCancel;
axios.VERSION = VERSION;
axios.toFormData = toFormData;

// Expose AxiosError class
axios.AxiosError = AxiosError;

// alias for CanceledError for backward compatibility
axios.Cancel = axios.CanceledError;

// Expose all/spread
axios.all = function all(promises) {
  return Promise.all(promises);
};

axios.spread = spread;

// Expose isAxiosError
axios.isAxiosError = isAxiosError;

// Expose mergeConfig
axios.mergeConfig = mergeConfig;

axios.AxiosHeaders = AxiosHeaders$1;

axios.formToJSON = thing => formDataToJSON(utils.isHTMLForm(thing) ? new FormData(thing) : thing);

axios.getAdapter = adapters.getAdapter;

axios.HttpStatusCode = HttpStatusCode$1;

axios.default = axios;

// this module should only have a default export
var axios$1 = axios;

function getDefaultExportFromCjs (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

let TinyEmitter$1 = class TinyEmitter {
  constructor() {
    Object.defineProperty(this, '__listeners', {
      value: {},
      enumerable: false,
      writable: false
    });
  }

  emit(eventName, ...args) {
    if(!this.__listeners[eventName])
      return this;

    for(const handler of this.__listeners[eventName]) {
      handler(...args);
    }

    return this;
  }

  once(eventName, handler) {
    const once = (...args) => {
      this.off(eventName, once);
      handler(...args);
    };

    return this.on(eventName, once);
  }

  on(eventName, handler) {
    if(!this.__listeners[eventName])
      this.__listeners[eventName] = [];

    this.__listeners[eventName].push(handler);

    return this;
  }

  off(eventName, handler) {
    if(handler)
      this.__listeners[eventName] = this.__listeners[eventName].filter(h => h !== handler);
    else
      this.__listeners[eventName] = [];

    return this;
  }
};

var tinyEmitter = TinyEmitter$1;

const TinyEmitter = tinyEmitter;

const MESSAGE_RESULT = 0;
const MESSAGE_EVENT = 1;
const RESULT_SUCCESS = 1;

let Worker$1 = class Worker extends TinyEmitter {
  /**
   *
   * @param worker {Worker}
   */
  constructor(worker) {
    super();

    this._messageId = 1;
    this._messages = new Map();

    this._worker = worker;
    this._worker.onmessage = this._onMessage.bind(this);
    this._id = Math.ceil(Math.random() * 10000000);
  }

  terminate() {
    this._worker.terminate();
  }

  /**
   * return true if there is no unresolved jobs
   * @returns {boolean}
   */
  isFree() {
    return this._messages.size === 0;
  }

  jobsLength() {
    return this._messages.size;
  }

  /**
   * @param operationName string
   * @param data any
   * @param transferable array
   * @param onEvent function
   * @returns {Promise}
   */
  exec(operationName, data = null, transferable = [], onEvent) {
    return new Promise((res, rej) => {
      const messageId = this._messageId++;
      this._messages.set(messageId, [res, rej, onEvent]);
      this._worker.postMessage([messageId, data, operationName], transferable || []);
    });
  }

  /**
   *
   * @param data any
   * @param transferable array
   * @param onEvent function
   * @returns {Promise}
   */
  postMessage(data = null, transferable = [], onEvent) {
    return new Promise((res, rej) => {
      const messageId = this._messageId++;
      this._messages.set(messageId, [res, rej, onEvent]);
      this._worker.postMessage([messageId, data], transferable || []);
    });
  }

  emit(eventName, ...args) {
    this._worker.postMessage({eventName, args});
  }

  _onMessage(e) {
    //if we got usual event, just emit it locally
    if(!Array.isArray(e.data) && e.data.eventName) {
      return super.emit(e.data.eventName, ...e.data.args);
    }

    const [type, ...args] = e.data;

    if(type === MESSAGE_EVENT)
      this._onEvent(...args);
    else if(type === MESSAGE_RESULT)
      this._onResult(...args);
    else
      throw new Error(`Wrong message type '${type}'`);
  }

  _onResult(messageId, success, payload) {
    const [res, rej] = this._messages.get(messageId);
    this._messages.delete(messageId);

    return success === RESULT_SUCCESS ? res(payload) : rej(payload);
  }

  _onEvent(messageId, eventName, data) {
    const [,,onEvent] = this._messages.get(messageId);

    if(onEvent) {
      onEvent(eventName, data);
    }
  }

};

var src = Worker$1;

var WebworkerPromise = /*@__PURE__*/getDefaultExportFromCjs(src);

// Internal function to create a web worker promise
async function createWebWorkerPromise(existingWorker, pipelineWorkerUrl) {
    let workerPromise;
    if (existingWorker != null) {
        // See if we have a worker promise attached the worker, if so reuse it. This ensures
        // that we can safely reuse the worker without issues.
        const itkWebWorker = existingWorker;
        if (itkWebWorker.workerPromise !== undefined) {
            workerPromise = itkWebWorker.workerPromise;
        }
        else {
            workerPromise = new WebworkerPromise(existingWorker);
        }
        return await Promise.resolve({ webworkerPromise: workerPromise, worker: existingWorker });
    }
    const workerUrl = typeof pipelineWorkerUrl === 'undefined' ? config.pipelineWorkerUrl : pipelineWorkerUrl;
    let worker = null;
    // @ts-expect-error: error TS2339: Property 'webWorkersUrl' does not exist on type '{ pipelineWorkerUrl: string; imageIOUrl: string; meshIOUrl: string; pipelinesUrl: string; }
    const webWorkersUrl = config.webWorkersUrl;
    if (typeof webWorkersUrl !== 'undefined') {
        console.warn('itkConfig webWorkersUrl is deprecated. Please use pipelineWorkerUrl with the full path to the pipeline worker.');
        const min = 'min-';
        // debug
        // const min = ''
        const webWorkerString = webWorkersUrl;
        if (webWorkerString.startsWith('http')) {
            const response = await axios$1.get(`${webWorkerString}/${min}bundles/pipeline.worker.js`, { responseType: 'blob' });
            worker = new Worker(URL.createObjectURL(response.data));
        }
        else {
            worker = new Worker(`${webWorkerString}/${min}bundles/pipeline.worker.js`);
        }
    }
    else if (workerUrl === null) {
        // Use the version built with the bundler
        //
        // Bundlers, e.g. WebPack, Vite, Rollup, see these paths at build time
        worker = new Worker(new URL('../web-workers/pipeline.worker.js', import.meta.url), { type: 'module' });
    }
    else {
        if (workerUrl.startsWith('http')) {
            const response = await axios$1.get(workerUrl, { responseType: 'blob' });
            worker = new Worker(URL.createObjectURL(response.data));
        }
        else {
            worker = new Worker(workerUrl);
        }
    }
    const webworkerPromise = new WebworkerPromise(worker);
    // Attach the worker promise to the worker, so if the worker is reused we can
    // also reuse the the worker promise.
    const itkWebWorker = worker;
    itkWebWorker.workerPromise = webworkerPromise;
    return { webworkerPromise, worker: itkWebWorker };
}

let pipelinesBaseUrl$1;
function getPipelinesBaseUrl$1() {
    return pipelinesBaseUrl$1;
}

function getFileExtension(filePath) {
    let extension = filePath.slice((filePath.lastIndexOf('.') - 1 >>> 0) + 2);
    if (extension.toLowerCase() === 'gz') {
        const index = filePath.slice(0, -3).lastIndexOf('.');
        extension = filePath.slice((index - 1 >>> 0) + 2);
    }
    else if (extension.toLowerCase() === 'cbor') {
        const index = filePath.slice(0, -5).lastIndexOf('.');
        extension = filePath.slice((index - 1 >>> 0) + 2);
    }
    else if (extension.toLowerCase() === 'zst') {
        // .iwi.cbor.zstd
        const index = filePath.slice(0, -10).lastIndexOf('.');
        extension = filePath.slice((index - 1 >>> 0) + 2);
    }
    else if (extension.toLowerCase() === 'zip') {
        const index = filePath.slice(0, -4).lastIndexOf('.');
        extension = filePath.slice((index - 1 >>> 0) + 2);
    }
    return extension;
}

function imageTransferables(image) {
    return [
        image.data,
        image.direction
    ];
}

function meshTransferables(mesh) {
    return [
        mesh.points,
        mesh.pointData,
        mesh.cells,
        mesh.cellData
    ];
}

async function loadEmscriptenModuleMainThread(moduleRelativePathOrURL, baseUrl) {
    let modulePrefix = 'unknown';
    if (typeof moduleRelativePathOrURL !== 'string') {
        modulePrefix = moduleRelativePathOrURL.href;
    }
    else if (moduleRelativePathOrURL.startsWith('http')) {
        modulePrefix = moduleRelativePathOrURL;
    }
    else {
        modulePrefix = `${baseUrl}/${moduleRelativePathOrURL}`;
    }
    if (modulePrefix.endsWith('.js')) {
        modulePrefix = modulePrefix.substring(0, modulePrefix.length - 3);
    }
    if (modulePrefix.endsWith('.wasm')) {
        modulePrefix = modulePrefix.substring(0, modulePrefix.length - 5);
    }
    const wasmBinaryPath = `${modulePrefix}.wasm`;
    const response = await axios$1.get(wasmBinaryPath, { responseType: 'arraybuffer' });
    const wasmBinary = response.data;
    const fullModulePath = `${modulePrefix}.js`;
    const result = await import(/* webpackIgnore: true */ /* @vite-ignore */ fullModulePath);
    const instantiated = result.default({ wasmBinary });
    return instantiated;
}

const simd=async()=>WebAssembly.validate(new Uint8Array([0,97,115,109,1,0,0,0,1,5,1,96,0,1,123,3,2,1,0,10,10,1,8,0,65,0,253,15,253,98,11]));

const haveSharedArrayBuffer = typeof globalThis.SharedArrayBuffer === 'function';
const encoder = new TextEncoder();
const decoder = new TextDecoder('utf-8');
function readFileSharedArray(emscriptenModule, path) {
    const opts = { flags: 'r', encoding: 'binary' };
    const stream = emscriptenModule.fs_open(path, opts.flags);
    const stat = emscriptenModule.fs_stat(path);
    const length = stat.size;
    let arrayBufferData = null;
    if (haveSharedArrayBuffer) {
        arrayBufferData = new SharedArrayBuffer(length); // eslint-disable-line
    }
    else {
        arrayBufferData = new ArrayBuffer(length);
    }
    const array = new Uint8Array(arrayBufferData);
    emscriptenModule.fs_read(stream, array, 0, length, 0);
    emscriptenModule.fs_close(stream);
    return array;
}
function memoryUint8SharedArray(emscriptenModule, byteOffset, length) {
    let arrayBufferData = null;
    if (haveSharedArrayBuffer) {
        arrayBufferData = new SharedArrayBuffer(length); // eslint-disable-line
    }
    else {
        arrayBufferData = new ArrayBuffer(length);
    }
    const array = new Uint8Array(arrayBufferData);
    const dataArrayView = new Uint8Array(emscriptenModule.HEAPU8.buffer, byteOffset, length);
    array.set(dataArrayView);
    return array;
}
function setPipelineModuleInputArray(emscriptenModule, dataArray, inputIndex, subIndex) {
    let dataPtr = 0;
    if (dataArray !== null) {
        dataPtr = emscriptenModule.ccall('itk_wasm_input_array_alloc', 'number', ['number', 'number', 'number', 'number'], [0, inputIndex, subIndex, dataArray.buffer.byteLength]);
        emscriptenModule.HEAPU8.set(new Uint8Array(dataArray.buffer), dataPtr);
    }
    return dataPtr;
}
function setPipelineModuleInputJSON(emscriptenModule, dataObject, inputIndex) {
    const dataJSON = JSON.stringify(dataObject);
    const jsonPtr = emscriptenModule.ccall('itk_wasm_input_json_alloc', 'number', ['number', 'number', 'number'], [0, inputIndex, dataJSON.length]);
    emscriptenModule.writeAsciiToMemory(dataJSON, jsonPtr, false);
}
function getPipelineModuleOutputArray(emscriptenModule, outputIndex, subIndex, componentType) {
    const dataPtr = emscriptenModule.ccall('itk_wasm_output_array_address', 'number', ['number', 'number', 'number'], [0, outputIndex, subIndex]);
    const dataSize = emscriptenModule.ccall('itk_wasm_output_array_size', 'number', ['number', 'number', 'number'], [0, outputIndex, subIndex]);
    const dataUint8 = memoryUint8SharedArray(emscriptenModule, dataPtr, dataSize);
    const data = bufferToTypedArray(componentType, dataUint8.buffer);
    return data;
}
function getPipelineModuleOutputJSON(emscriptenModule, outputIndex) {
    const jsonPtr = emscriptenModule.ccall('itk_wasm_output_json_address', 'number', ['number', 'number'], [0, outputIndex]);
    const dataJSON = emscriptenModule.AsciiToString(jsonPtr);
    const dataObject = JSON.parse(dataJSON);
    return dataObject;
}
function runPipelineEmscripten(pipelineModule, args, outputs, inputs) {
    if (!(inputs == null) && inputs.length > 0) {
        inputs.forEach(function (input, index) {
            var _a;
            switch (input.type) {
                case InterfaceTypes$1.TextStream:
                    {
                        const dataArray = encoder.encode(input.data.data);
                        const arrayPtr = setPipelineModuleInputArray(pipelineModule, dataArray, index, 0);
                        const dataJSON = { size: dataArray.buffer.byteLength, data: `data:application/vnd.itk.address,0:${arrayPtr}` };
                        setPipelineModuleInputJSON(pipelineModule, dataJSON, index);
                        break;
                    }
                case InterfaceTypes$1.JsonCompatible:
                    {
                        const dataArray = encoder.encode(JSON.stringify(input.data));
                        const arrayPtr = setPipelineModuleInputArray(pipelineModule, dataArray, index, 0);
                        const dataJSON = { size: dataArray.buffer.byteLength, data: `data:application/vnd.itk.address,0:${arrayPtr}` };
                        setPipelineModuleInputJSON(pipelineModule, dataJSON, index);
                        break;
                    }
                case InterfaceTypes$1.BinaryStream:
                    {
                        const dataArray = input.data.data;
                        const arrayPtr = setPipelineModuleInputArray(pipelineModule, dataArray, index, 0);
                        const dataJSON = { size: dataArray.buffer.byteLength, data: `data:application/vnd.itk.address,0:${arrayPtr}` };
                        setPipelineModuleInputJSON(pipelineModule, dataJSON, index);
                        break;
                    }
                case InterfaceTypes$1.TextFile:
                    {
                        pipelineModule.fs_writeFile(input.data.path, input.data.data);
                        break;
                    }
                case InterfaceTypes$1.BinaryFile:
                    {
                        pipelineModule.fs_writeFile(input.data.path, input.data.data);
                        break;
                    }
                case InterfaceTypes$1.Image:
                    {
                        const image = input.data;
                        const dataPtr = setPipelineModuleInputArray(pipelineModule, image.data, index, 0);
                        const directionPtr = setPipelineModuleInputArray(pipelineModule, image.direction, index, 1);
                        const metadata = typeof ((_a = image.metadata) === null || _a === void 0 ? void 0 : _a.entries) !== 'undefined' ? JSON.stringify(Array.from(image.metadata.entries())) : '[]';
                        const imageJSON = {
                            imageType: image.imageType,
                            name: image.name,
                            origin: image.origin,
                            spacing: image.spacing,
                            direction: `data:application/vnd.itk.address,0:${directionPtr}`,
                            size: image.size,
                            data: `data:application/vnd.itk.address,0:${dataPtr}`,
                            metadata
                        };
                        setPipelineModuleInputJSON(pipelineModule, imageJSON, index);
                        break;
                    }
                case InterfaceTypes$1.Mesh:
                    {
                        const mesh = input.data;
                        const pointsPtr = setPipelineModuleInputArray(pipelineModule, mesh.points, index, 0);
                        const cellsPtr = setPipelineModuleInputArray(pipelineModule, mesh.cells, index, 1);
                        const pointDataPtr = setPipelineModuleInputArray(pipelineModule, mesh.pointData, index, 2);
                        const cellDataPtr = setPipelineModuleInputArray(pipelineModule, mesh.cellData, index, 3);
                        const meshJSON = {
                            meshType: mesh.meshType,
                            name: mesh.name,
                            numberOfPoints: mesh.numberOfPoints,
                            points: `data:application/vnd.itk.address,0:${pointsPtr}`,
                            numberOfCells: mesh.numberOfCells,
                            cells: `data:application/vnd.itk.address,0:${cellsPtr}`,
                            cellBufferSize: mesh.cellBufferSize,
                            numberOfPointPixels: mesh.numberOfPointPixels,
                            pointData: `data:application/vnd.itk.address,0:${pointDataPtr}`,
                            numberOfCellPixels: mesh.numberOfCellPixels,
                            cellData: `data:application/vnd.itk.address,0:${cellDataPtr}`
                        };
                        setPipelineModuleInputJSON(pipelineModule, meshJSON, index);
                        break;
                    }
                case InterfaceTypes$1.PolyData:
                    {
                        const polyData = input.data;
                        const pointsPtr = setPipelineModuleInputArray(pipelineModule, polyData.points, index, 0);
                        const verticesPtr = setPipelineModuleInputArray(pipelineModule, polyData.vertices, index, 1);
                        const linesPtr = setPipelineModuleInputArray(pipelineModule, polyData.lines, index, 2);
                        const polygonsPtr = setPipelineModuleInputArray(pipelineModule, polyData.polygons, index, 3);
                        const triangleStripsPtr = setPipelineModuleInputArray(pipelineModule, polyData.triangleStrips, index, 4);
                        const pointDataPtr = setPipelineModuleInputArray(pipelineModule, polyData.pointData, index, 5);
                        const cellDataPtr = setPipelineModuleInputArray(pipelineModule, polyData.pointData, index, 6);
                        const polyDataJSON = {
                            polyDataType: polyData.polyDataType,
                            name: polyData.name,
                            numberOfPoints: polyData.numberOfPoints,
                            points: `data:application/vnd.itk.address,0:${pointsPtr}`,
                            verticesBufferSize: polyData.verticesBufferSize,
                            vertices: `data:application/vnd.itk.address,0:${verticesPtr}`,
                            linesBufferSize: polyData.linesBufferSize,
                            lines: `data:application/vnd.itk.address,0:${linesPtr}`,
                            polygonsBufferSize: polyData.polygonsBufferSize,
                            polygons: `data:application/vnd.itk.address,0:${polygonsPtr}`,
                            triangleStripsBufferSize: polyData.triangleStripsBufferSize,
                            triangleStrips: `data:application/vnd.itk.address,0:${triangleStripsPtr}`,
                            numberOfPointPixels: polyData.numberOfPointPixels,
                            pointData: `data:application/vnd.itk.address,0:${pointDataPtr}`,
                            numberOfCellPixels: polyData.numberOfCellPixels,
                            cellData: `data:application/vnd.itk.address,0:${cellDataPtr}`
                        };
                        setPipelineModuleInputJSON(pipelineModule, polyDataJSON, index);
                        break;
                    }
                case IOTypes$1.Text:
                    {
                        pipelineModule.fs_writeFile(input.path, input.data);
                        break;
                    }
                case IOTypes$1.Binary:
                    {
                        pipelineModule.fs_writeFile(input.path, input.data);
                        break;
                    }
                case IOTypes$1.Image:
                    {
                        const image = input.data;
                        const imageJSON = {
                            imageType: image.imageType,
                            name: image.name,
                            origin: image.origin,
                            spacing: image.spacing,
                            direction: 'data:application/vnd.itk.path,data/direction.raw',
                            size: image.size,
                            data: 'data:application/vnd.itk.path,data/data.raw'
                        };
                        pipelineModule.fs_mkdirs(`${input.path}/data`);
                        pipelineModule.fs_writeFile(`${input.path}/index.json`, JSON.stringify(imageJSON));
                        if (image.data === null) {
                            throw Error('image.data is null');
                        }
                        pipelineModule.fs_writeFile(`${input.path}/data/data.raw`, new Uint8Array(image.data.buffer));
                        pipelineModule.fs_writeFile(`${input.path}/data/direction.raw`, new Uint8Array(image.direction.buffer));
                        break;
                    }
                case IOTypes$1.Mesh:
                    {
                        const mesh = input.data;
                        const meshJSON = {
                            meshType: mesh.meshType,
                            name: mesh.name,
                            numberOfPoints: mesh.numberOfPoints,
                            points: 'data:application/vnd.itk.path,data/points.raw',
                            numberOfPointPixels: mesh.numberOfPointPixels,
                            pointData: 'data:application/vnd.itk.path,data/pointData.raw',
                            numberOfCells: mesh.numberOfCells,
                            cells: 'data:application/vnd.itk.path,data/cells.raw',
                            numberOfCellPixels: mesh.numberOfCellPixels,
                            cellData: 'data:application/vnd.itk.path,data/cellData.raw',
                            cellBufferSize: mesh.cellBufferSize
                        };
                        pipelineModule.fs_mkdirs(`${input.path}/data`);
                        pipelineModule.fs_writeFile(`${input.path}/index.json`, JSON.stringify(meshJSON));
                        if (meshJSON.numberOfPoints > 0) {
                            if (mesh.points === null) {
                                throw Error('mesh.points is null');
                            }
                            pipelineModule.fs_writeFile(`${input.path}/data/points.raw`, new Uint8Array(mesh.points.buffer));
                        }
                        if (meshJSON.numberOfPointPixels > 0) {
                            if (mesh.pointData === null) {
                                throw Error('mesh.pointData is null');
                            }
                            pipelineModule.fs_writeFile(`${input.path}/data/pointData.raw`, new Uint8Array(mesh.pointData.buffer));
                        }
                        if (meshJSON.numberOfCells > 0) {
                            if (mesh.cells === null) {
                                throw Error('mesh.cells is null');
                            }
                            pipelineModule.fs_writeFile(`${input.path}/data/cells.raw`, new Uint8Array(mesh.cells.buffer));
                        }
                        if (meshJSON.numberOfCellPixels > 0) {
                            if (mesh.cellData === null) {
                                throw Error('mesh.cellData is null');
                            }
                            pipelineModule.fs_writeFile(`${input.path}/data/cellData.raw`, new Uint8Array(mesh.cellData.buffer));
                        }
                        break;
                    }
                default:
                    throw Error('Unsupported input InterfaceType');
            }
        });
    }
    pipelineModule.resetModuleStdout();
    pipelineModule.resetModuleStderr();
    const stackPtr = pipelineModule.stackSave();
    let returnValue = 0;
    try {
        returnValue = pipelineModule.callMain(args.slice());
    }
    catch (exception) {
        // Note: Module must be built with CMAKE_BUILD_TYPE set to Debug.
        // e.g.: itk-wasm build my/project -- -DCMAKE_BUILD_TYPE:STRING=Debug
        if (typeof exception === 'number') {
            console.log('Exception while running pipeline:');
            console.log('stdout:', pipelineModule.getModuleStdout());
            console.error('stderr:', pipelineModule.getModuleStderr());
            if (typeof pipelineModule.getExceptionMessage !== 'undefined') {
                console.error('exception:', pipelineModule.getExceptionMessage(exception));
            }
            else {
                console.error('Build module in Debug mode for exception message information.');
            }
        }
        throw exception;
    }
    finally {
        pipelineModule.stackRestore(stackPtr);
    }
    const stdout = pipelineModule.getModuleStdout();
    const stderr = pipelineModule.getModuleStderr();
    const populatedOutputs = [];
    if (!(outputs == null) && outputs.length > 0 && returnValue === 0) {
        outputs.forEach(function (output, index) {
            let outputData = null;
            switch (output.type) {
                case InterfaceTypes$1.TextStream:
                    {
                        const dataPtr = pipelineModule.ccall('itk_wasm_output_array_address', 'number', ['number', 'number', 'number'], [0, index, 0]);
                        const dataSize = pipelineModule.ccall('itk_wasm_output_array_size', 'number', ['number', 'number', 'number'], [0, index, 0]);
                        const dataArrayView = new Uint8Array(pipelineModule.HEAPU8.buffer, dataPtr, dataSize);
                        outputData = { data: decoder.decode(dataArrayView) };
                        break;
                    }
                case InterfaceTypes$1.JsonCompatible:
                    {
                        const dataPtr = pipelineModule.ccall('itk_wasm_output_array_address', 'number', ['number', 'number', 'number'], [0, index, 0]);
                        const dataSize = pipelineModule.ccall('itk_wasm_output_array_size', 'number', ['number', 'number', 'number'], [0, index, 0]);
                        const dataArrayView = new Uint8Array(pipelineModule.HEAPU8.buffer, dataPtr, dataSize);
                        outputData = JSON.parse(decoder.decode(dataArrayView));
                        break;
                    }
                case InterfaceTypes$1.BinaryStream:
                    {
                        const dataPtr = pipelineModule.ccall('itk_wasm_output_array_address', 'number', ['number', 'number', 'number'], [0, index, 0]);
                        const dataSize = pipelineModule.ccall('itk_wasm_output_array_size', 'number', ['number', 'number', 'number'], [0, index, 0]);
                        outputData = { data: memoryUint8SharedArray(pipelineModule, dataPtr, dataSize) };
                        break;
                    }
                case InterfaceTypes$1.TextFile:
                    {
                        outputData = { path: output.data.path, data: pipelineModule.fs_readFile(output.data.path, { encoding: 'utf8' }) };
                        break;
                    }
                case InterfaceTypes$1.BinaryFile:
                    {
                        outputData = { path: output.data.path, data: readFileSharedArray(pipelineModule, output.data.path) };
                        break;
                    }
                case InterfaceTypes$1.Image:
                    {
                        const image = getPipelineModuleOutputJSON(pipelineModule, index);
                        image.data = getPipelineModuleOutputArray(pipelineModule, index, 0, image.imageType.componentType);
                        image.direction = getPipelineModuleOutputArray(pipelineModule, index, 1, FloatTypes$1.Float64);
                        image.metadata = new Map(image.metadata);
                        outputData = image;
                        break;
                    }
                case InterfaceTypes$1.Mesh:
                    {
                        const mesh = getPipelineModuleOutputJSON(pipelineModule, index);
                        if (mesh.numberOfPoints > 0) {
                            mesh.points = getPipelineModuleOutputArray(pipelineModule, index, 0, mesh.meshType.pointComponentType);
                        }
                        else {
                            mesh.points = bufferToTypedArray(mesh.meshType.pointComponentType, new ArrayBuffer(0));
                        }
                        if (mesh.numberOfCells > 0) {
                            mesh.cells = getPipelineModuleOutputArray(pipelineModule, index, 1, mesh.meshType.cellComponentType);
                        }
                        else {
                            mesh.cells = bufferToTypedArray(mesh.meshType.cellComponentType, new ArrayBuffer(0));
                        }
                        if (mesh.numberOfPointPixels > 0) {
                            mesh.pointData = getPipelineModuleOutputArray(pipelineModule, index, 2, mesh.meshType.pointPixelComponentType);
                        }
                        else {
                            mesh.pointData = bufferToTypedArray(mesh.meshType.pointPixelComponentType, new ArrayBuffer(0));
                        }
                        if (mesh.numberOfCellPixels > 0) {
                            mesh.cellData = getPipelineModuleOutputArray(pipelineModule, index, 3, mesh.meshType.cellPixelComponentType);
                        }
                        else {
                            mesh.cellData = bufferToTypedArray(mesh.meshType.cellPixelComponentType, new ArrayBuffer(0));
                        }
                        outputData = mesh;
                        break;
                    }
                case InterfaceTypes$1.PolyData:
                    {
                        const polyData = getPipelineModuleOutputJSON(pipelineModule, index);
                        if (polyData.numberOfPoints > 0) {
                            polyData.points = getPipelineModuleOutputArray(pipelineModule, index, 0, FloatTypes$1.Float32);
                        }
                        else {
                            polyData.points = new Float32Array();
                        }
                        if (polyData.verticesBufferSize > 0) {
                            polyData.vertices = getPipelineModuleOutputArray(pipelineModule, index, 1, IntTypes$1.UInt32);
                        }
                        else {
                            polyData.vertices = new Uint32Array();
                        }
                        if (polyData.linesBufferSize > 0) {
                            polyData.lines = getPipelineModuleOutputArray(pipelineModule, index, 2, IntTypes$1.UInt32);
                        }
                        else {
                            polyData.lines = new Uint32Array();
                        }
                        if (polyData.polygonsBufferSize > 0) {
                            polyData.polygons = getPipelineModuleOutputArray(pipelineModule, index, 3, IntTypes$1.UInt32);
                        }
                        else {
                            polyData.polygons = new Uint32Array();
                        }
                        if (polyData.triangleStripsBufferSize > 0) {
                            polyData.triangleStrips = getPipelineModuleOutputArray(pipelineModule, index, 4, IntTypes$1.UInt32);
                        }
                        else {
                            polyData.triangleStrips = new Uint32Array();
                        }
                        if (polyData.numberOfPointPixels > 0) {
                            polyData.pointData = getPipelineModuleOutputArray(pipelineModule, index, 5, polyData.polyDataType.pointPixelComponentType);
                        }
                        else {
                            polyData.pointData = bufferToTypedArray(polyData.polyDataType.pointPixelComponentType, new ArrayBuffer(0));
                        }
                        if (polyData.numberOfCellPixels > 0) {
                            polyData.cellData = getPipelineModuleOutputArray(pipelineModule, index, 6, polyData.polyDataType.cellPixelComponentType);
                        }
                        else {
                            polyData.cellData = bufferToTypedArray(polyData.polyDataType.cellPixelComponentType, new ArrayBuffer(0));
                        }
                        outputData = polyData;
                        break;
                    }
                case IOTypes$1.Text:
                    {
                        if (typeof output.path === 'undefined') {
                            throw new Error('output.path not defined');
                        }
                        outputData = pipelineModule.fs_readFile(output.path, { encoding: 'utf8' });
                        break;
                    }
                case IOTypes$1.Binary:
                    {
                        if (typeof output.path === 'undefined') {
                            throw new Error('output.path not defined');
                        }
                        outputData = readFileSharedArray(pipelineModule, output.path);
                        break;
                    }
                case IOTypes$1.Image:
                    {
                        if (typeof output.path === 'undefined') {
                            throw new Error('output.path not defined');
                        }
                        const imageJSON = pipelineModule.fs_readFile(`${output.path}/index.json`, { encoding: 'utf8' });
                        const image = JSON.parse(imageJSON);
                        const dataUint8 = readFileSharedArray(pipelineModule, `${output.path}/data/data.raw`);
                        image.data = bufferToTypedArray(image.imageType.componentType, dataUint8.buffer);
                        const directionUint8 = readFileSharedArray(pipelineModule, `${output.path}/data/direction.raw`);
                        image.direction = bufferToTypedArray(FloatTypes$1.Float64, directionUint8.buffer);
                        outputData = image;
                        break;
                    }
                case IOTypes$1.Mesh:
                    {
                        if (typeof output.path === 'undefined') {
                            throw new Error('output.path not defined');
                        }
                        const meshJSON = pipelineModule.fs_readFile(`${output.path}/index.json`, { encoding: 'utf8' });
                        const mesh = JSON.parse(meshJSON);
                        if (mesh.numberOfPoints > 0) {
                            const dataUint8Points = readFileSharedArray(pipelineModule, `${output.path}/data/points.raw`);
                            mesh.points = bufferToTypedArray(mesh.meshType.pointComponentType, dataUint8Points.buffer);
                        }
                        else {
                            mesh.points = bufferToTypedArray(mesh.meshType.pointComponentType, new ArrayBuffer(0));
                        }
                        if (mesh.numberOfPointPixels > 0) {
                            const dataUint8PointData = readFileSharedArray(pipelineModule, `${output.path}/data/pointData.raw`);
                            mesh.pointData = bufferToTypedArray(mesh.meshType.pointPixelComponentType, dataUint8PointData.buffer);
                        }
                        else {
                            mesh.pointData = bufferToTypedArray(mesh.meshType.pointPixelComponentType, new ArrayBuffer(0));
                        }
                        if (mesh.numberOfCells > 0) {
                            const dataUint8Cells = readFileSharedArray(pipelineModule, `${output.path}/data/cells.raw`);
                            mesh.cells = bufferToTypedArray(mesh.meshType.cellComponentType, dataUint8Cells.buffer);
                        }
                        else {
                            mesh.cells = bufferToTypedArray(mesh.meshType.cellComponentType, new ArrayBuffer(0));
                        }
                        if (mesh.numberOfCellPixels > 0) {
                            const dataUint8CellData = readFileSharedArray(pipelineModule, `${output.path}/data/cellData.raw`);
                            mesh.cellData = bufferToTypedArray(mesh.meshType.cellPixelComponentType, dataUint8CellData.buffer);
                        }
                        else {
                            mesh.cellData = bufferToTypedArray(mesh.meshType.cellPixelComponentType, new ArrayBuffer(0));
                        }
                        outputData = mesh;
                        break;
                    }
                default:
                    throw Error('Unsupported output InterfaceType');
            }
            const populatedOutput = {
                type: output.type,
                data: outputData
            };
            populatedOutputs.push(populatedOutput);
        });
    }
    return { returnValue, stdout, stderr, outputs: populatedOutputs };
}

// To cache loaded pipeline modules
const pipelineToModule = new Map();
async function loadPipelineModule(pipelinePath) {
    let moduleRelativePathOrURL = pipelinePath;
    let pipeline = pipelinePath;
    if (typeof pipelinePath !== 'string') {
        moduleRelativePathOrURL = new URL(pipelinePath.href);
        pipeline = moduleRelativePathOrURL.href;
    }
    if (pipelineToModule.has(pipeline)) {
        return pipelineToModule.get(pipeline);
    }
    else {
        const pipelineModule = (await loadEmscriptenModuleMainThread(pipelinePath, config.pipelinesUrl));
        pipelineToModule.set(pipeline, pipelineModule);
        return pipelineModule;
    }
}
async function runPipeline(webWorker, pipelinePath, args, outputs, inputs, options) {
    var _a, _b;
    if (!await simd()) {
        const simdErrorMessage = 'WebAssembly SIMD support is required -- please update your browser.';
        alert(simdErrorMessage);
        throw new Error(simdErrorMessage);
    }
    if (webWorker === false) {
        const pipelineModule = await loadPipelineModule(pipelinePath.toString());
        const result = runPipelineEmscripten(pipelineModule, args, outputs, inputs);
        return result;
    }
    let worker = webWorker;
    const pipelineWorkerUrl = (_a = options === null || options === void 0 ? void 0 : options.pipelineWorkerUrl) !== null && _a !== void 0 ? _a : null;
    const pipelineWorkerUrlString = typeof pipelineWorkerUrl !== 'string' && typeof (pipelineWorkerUrl === null || pipelineWorkerUrl === void 0 ? void 0 : pipelineWorkerUrl.href) !== 'undefined' ? pipelineWorkerUrl.href : pipelineWorkerUrl;
    const { webworkerPromise, worker: usedWorker } = await createWebWorkerPromise(worker, pipelineWorkerUrlString);
    worker = usedWorker;
    const transferables = [];
    if (!(inputs == null) && inputs.length > 0) {
        inputs.forEach(function (input) {
            if (input.type === InterfaceTypes$1.BinaryStream) {
                // Binary data
                const dataArray = input.data.data;
                transferables.push(dataArray);
            }
            else if (input.type === InterfaceTypes$1.BinaryFile) {
                // Binary data
                const dataArray = input.data.data;
                transferables.push(dataArray);
            }
            else if (input.type === InterfaceTypes$1.Image) {
                // Image data
                const image = input.data;
                if (image.data === null) {
                    throw Error('image data cannot be null');
                }
                transferables.push(...imageTransferables(image));
            }
            else if (input.type === IOTypes$1.Binary) {
                // Binary data
                transferables.push(input.data);
            }
            else if (input.type === IOTypes$1.Image) {
                // Image data
                const image = input.data;
                if (image.data === null) {
                    throw Error('image data cannot be null');
                }
                transferables.push(...imageTransferables(image));
            }
            else if (input.type === IOTypes$1.Mesh) {
                // Mesh data
                const mesh = input.data;
                transferables.push(...meshTransferables(mesh));
            }
        });
    }
    const pipelineBaseUrl = (_b = options === null || options === void 0 ? void 0 : options.pipelineBaseUrl) !== null && _b !== void 0 ? _b : 'pipelinesUrl';
    const pipelineBaseUrlString = typeof pipelineBaseUrl !== 'string' && typeof (pipelineBaseUrl === null || pipelineBaseUrl === void 0 ? void 0 : pipelineBaseUrl.href) !== 'undefined' ? pipelineBaseUrl.href : pipelineBaseUrl;
    const result = await webworkerPromise.postMessage({
        operation: 'runPipeline',
        config: config,
        pipelinePath: pipelinePath.toString(),
        pipelineBaseUrl: pipelineBaseUrlString,
        args,
        outputs,
        inputs
    }, getTransferables(transferables));
    return {
        returnValue: result.returnValue,
        stdout: result.stdout,
        stderr: result.stderr,
        outputs: result.outputs,
        webWorker: worker
    };
}

var name = "@itk-wasm/image-io";
var version = "0.2.1";
var description = "Input and output for scientific and medical image file formats.";
var type = "module";
var module = "./dist/bundles/image-io.js";
var types = "./dist/src/index.d.ts";
var exports = {
	".": {
		types: "./dist/src/index.d.js",
		browser: "./dist/bundles/image-io.js",
		node: "./dist/bundles/image-io-node.js",
		"default": "./dist/bundles/image-io.js"
	}
};
var scripts = {
	start: "npm run copyShoelaceAssets && vite -c build/vite.config.js",
	test: "npm run test:node && npm run test:browser",
	"test:node": "ava",
	"test:browser": "npm run test:browser:chrome && npm run test:browser:firefox",
	"test:browser:firefox": "start-server-and-test rollup:start http-get://localhost:5004 cypress:runFirefox",
	"test:browser:chrome": "start-server-and-test rollup:start http-get://localhost:5004 cypress:runChrome",
	"test:browser:debug": "start-server-and-test rollup:start http-get://localhost:5004 cypress:open",
	"cypress:open": "npx cypress open",
	"cypress:runChrome": "npx cypress run --browser chrome",
	"cypress:runFirefox": "npx cypress run --browser firefox",
	build: "npm run build:tsc && npm run build:node && npm run build:browser && npm run build:demo",
	"build:node": "rollup -c ./build/rollup.node.config.js",
	"build:browser": "rollup -c ./build/rollup.browser.config.js",
	"build:tsc": "tsc --pretty",
	copyShoelaceAssets: "shx mkdir -p test/browser/demo-app/public && shx cp -r node_modules/@shoelace-style/shoelace/dist/assets test/browser/demo-app/public/",
	"build:demo": "npm run copyShoelaceAssets && vite -c build/vite.config.js build",
	"rollup:start": "npm run copyShoelaceAssets && concurrently npm:rollup:dev npm:rollup:preview",
	"rollup:dev": "vite build --config build/vite-rollup-watch.config.ts",
	"rollup:preview": "vite preview --config build/vite-rollup-watch.config.ts"
};
var keywords = [
	"itk",
	"wasm",
	"webassembly",
	"wasi"
];
var author = "";
var license = "Apache-2.0";
var dependencies = {
	"itk-wasm": "^1.0.0-b.149"
};
var devDependencies = {
	"@rollup/plugin-commonjs": "^24.0.0",
	"@rollup/plugin-json": "^6.0.0",
	"@rollup/plugin-node-resolve": "^15.1.0",
	"@rollup/plugin-terser": "^0.4.0",
	"@rollup/plugin-typescript": "^11.1.1",
	"@shoelace-style/shoelace": "^2.5.2",
	"@types/node": "^20.2.5",
	ava: "^5.3.1",
	concurrently: "^8.2.1",
	cypress: "^13.3.0",
	debug: "^4.3.4",
	rollup: "^3.9.0",
	"rollup-plugin-copy": "^3.4.0",
	"rollup-plugin-ignore": "^1.0.10",
	"rollup-plugin-polyfill-node": "^0.12.0",
	shx: "^0.3.4",
	"supports-color": "^9.3.1",
	tslib: "^2.5.2",
	typescript: "^5.0.4",
	vite: "^4.4.11",
	"vite-plugin-static-copy": "^0.17.0"
};
var repository = {
	type: "git",
	url: "https://github.com/InsightSoftwareConsortium/itk-wasm"
};
var ava = {
	files: [
		"test/node/**/*",
		"!test/node/common.js"
	]
};
var packageJson = {
	name: name,
	version: version,
	description: description,
	type: type,
	module: module,
	types: types,
	exports: exports,
	scripts: scripts,
	keywords: keywords,
	author: author,
	license: license,
	dependencies: dependencies,
	devDependencies: devDependencies,
	repository: repository,
	ava: ava
};

let pipelinesBaseUrl;
let defaultPipelinesBaseUrl = `https://cdn.jsdelivr.net/npm/@itk-wasm/image-io@${packageJson.version}/dist/pipelines`;
function setPipelinesBaseUrl(baseUrl) {
    pipelinesBaseUrl = baseUrl;
}
function getPipelinesBaseUrl() {
    if (typeof pipelinesBaseUrl !== 'undefined') {
        return pipelinesBaseUrl;
    }
    const itkWasmPipelinesBaseUrl = getPipelinesBaseUrl$1();
    if (typeof itkWasmPipelinesBaseUrl !== 'undefined') {
        return itkWasmPipelinesBaseUrl;
    }
    return defaultPipelinesBaseUrl;
}

const mimeToImageIo = new Map([
    ['image/jpeg', 'jpeg'],
    ['image/png', 'png'],
    ['image/tiff', 'tiff'],
    ['image/x-ms-bmp', 'bmp'],
    ['image/x-bmp', 'bmp'],
    ['image/bmp', 'bmp'],
    ['application/dicom', 'gdcm']
]);

const extensionToImageIo = new Map([
    ['bmp', 'bmp'],
    ['dcm', 'gdcm'],
    ['gipl', 'gipl'],
    ['gipl.gz', 'gipl'],
    ['hdf5', 'hdf5'],
    ['jpg', 'jpeg'],
    ['jpeg', 'jpeg'],
    ['iwi', 'wasm'],
    ['iwi.cbor', 'wasm'],
    ['iwi.cbor.zst', 'wasmZstd'],
    ['lsm', 'lsm'],
    ['mnc', 'mnc'],
    ['mnc.gz', 'mnc'],
    ['mnc2', 'mnc'],
    ['mgh', 'mgh'],
    ['mgz', 'mgh'],
    ['mgh.gz', 'mgh'],
    ['mha', 'meta'],
    ['mhd', 'meta'],
    ['mrc', 'mrc'],
    ['nia', 'nifti'],
    ['nii', 'nifti'],
    ['nii.gz', 'nifti'],
    ['hdr', 'nifti'],
    ['nrrd', 'nrrd'],
    ['nhdr', 'nrrd'],
    ['png', 'png'],
    ['pic', 'bioRad'],
    ['tif', 'tiff'],
    ['tiff', 'tiff'],
    ['vtk', 'vtk'],
    ['isq', 'scanco'],
    ['fdf', 'fdf'],
]);

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {PngReadImageOptions} options - options object
 *
 * @returns {Promise<PngReadImageResult>} - result object
 */
async function pngReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'png-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {PngWriteImageOptions} options - options object
 *
 * @returns {Promise<PngWriteImageResult>} - result object
 */
async function pngWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'png-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {MetaReadImageOptions} options - options object
 *
 * @returns {Promise<MetaReadImageResult>} - result object
 */
async function metaReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'meta-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {MetaWriteImageOptions} options - options object
 *
 * @returns {Promise<MetaWriteImageResult>} - result object
 */
async function metaWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'meta-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {TiffReadImageOptions} options - options object
 *
 * @returns {Promise<TiffReadImageResult>} - result object
 */
async function tiffReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'tiff-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {TiffWriteImageOptions} options - options object
 *
 * @returns {Promise<TiffWriteImageResult>} - result object
 */
async function tiffWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'tiff-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {NiftiReadImageOptions} options - options object
 *
 * @returns {Promise<NiftiReadImageResult>} - result object
 */
async function niftiReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'nifti-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {NiftiWriteImageOptions} options - options object
 *
 * @returns {Promise<NiftiWriteImageResult>} - result object
 */
async function niftiWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'nifti-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {JpegReadImageOptions} options - options object
 *
 * @returns {Promise<JpegReadImageResult>} - result object
 */
async function jpegReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'jpeg-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {JpegWriteImageOptions} options - options object
 *
 * @returns {Promise<JpegWriteImageResult>} - result object
 */
async function jpegWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'jpeg-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {NrrdReadImageOptions} options - options object
 *
 * @returns {Promise<NrrdReadImageResult>} - result object
 */
async function nrrdReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'nrrd-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {NrrdWriteImageOptions} options - options object
 *
 * @returns {Promise<NrrdWriteImageResult>} - result object
 */
async function nrrdWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'nrrd-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {VtkReadImageOptions} options - options object
 *
 * @returns {Promise<VtkReadImageResult>} - result object
 */
async function vtkReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'vtk-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {VtkWriteImageOptions} options - options object
 *
 * @returns {Promise<VtkWriteImageResult>} - result object
 */
async function vtkWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'vtk-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {BmpReadImageOptions} options - options object
 *
 * @returns {Promise<BmpReadImageResult>} - result object
 */
async function bmpReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'bmp-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {BmpWriteImageOptions} options - options object
 *
 * @returns {Promise<BmpWriteImageResult>} - result object
 */
async function bmpWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'bmp-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {Hdf5ReadImageOptions} options - options object
 *
 * @returns {Promise<Hdf5ReadImageResult>} - result object
 */
async function hdf5ReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'hdf5-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {Hdf5WriteImageOptions} options - options object
 *
 * @returns {Promise<Hdf5WriteImageResult>} - result object
 */
async function hdf5WriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'hdf5-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {MincReadImageOptions} options - options object
 *
 * @returns {Promise<MincReadImageResult>} - result object
 */
async function mincReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'minc-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {MincWriteImageOptions} options - options object
 *
 * @returns {Promise<MincWriteImageResult>} - result object
 */
async function mincWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'minc-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {MrcReadImageOptions} options - options object
 *
 * @returns {Promise<MrcReadImageResult>} - result object
 */
async function mrcReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'mrc-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {MrcWriteImageOptions} options - options object
 *
 * @returns {Promise<MrcWriteImageResult>} - result object
 */
async function mrcWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'mrc-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {LsmReadImageOptions} options - options object
 *
 * @returns {Promise<LsmReadImageResult>} - result object
 */
async function lsmReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'lsm-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {LsmWriteImageOptions} options - options object
 *
 * @returns {Promise<LsmWriteImageResult>} - result object
 */
async function lsmWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'lsm-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {MghReadImageOptions} options - options object
 *
 * @returns {Promise<MghReadImageResult>} - result object
 */
async function mghReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'mgh-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {MghWriteImageOptions} options - options object
 *
 * @returns {Promise<MghWriteImageResult>} - result object
 */
async function mghWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'mgh-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {BioRadReadImageOptions} options - options object
 *
 * @returns {Promise<BioRadReadImageResult>} - result object
 */
async function bioRadReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'bio-rad-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {BioRadWriteImageOptions} options - options object
 *
 * @returns {Promise<BioRadWriteImageResult>} - result object
 */
async function bioRadWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'bio-rad-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {GiplReadImageOptions} options - options object
 *
 * @returns {Promise<GiplReadImageResult>} - result object
 */
async function giplReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'gipl-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {GiplWriteImageOptions} options - options object
 *
 * @returns {Promise<GiplWriteImageResult>} - result object
 */
async function giplWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'gipl-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {GeAdwReadImageOptions} options - options object
 *
 * @returns {Promise<GeAdwReadImageResult>} - result object
 */
async function geAdwReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'ge-adw-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {GeAdwWriteImageOptions} options - options object
 *
 * @returns {Promise<GeAdwWriteImageResult>} - result object
 */
async function geAdwWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'ge-adw-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {Ge4ReadImageOptions} options - options object
 *
 * @returns {Promise<Ge4ReadImageResult>} - result object
 */
async function ge4ReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'ge4-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {Ge4WriteImageOptions} options - options object
 *
 * @returns {Promise<Ge4WriteImageResult>} - result object
 */
async function ge4WriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'ge4-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {Ge5ReadImageOptions} options - options object
 *
 * @returns {Promise<Ge5ReadImageResult>} - result object
 */
async function ge5ReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'ge5-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {Ge5WriteImageOptions} options - options object
 *
 * @returns {Promise<Ge5WriteImageResult>} - result object
 */
async function ge5WriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'ge5-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {GdcmReadImageOptions} options - options object
 *
 * @returns {Promise<GdcmReadImageResult>} - result object
 */
async function gdcmReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'gdcm-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {GdcmWriteImageOptions} options - options object
 *
 * @returns {Promise<GdcmWriteImageResult>} - result object
 */
async function gdcmWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'gdcm-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {ScancoReadImageOptions} options - options object
 *
 * @returns {Promise<ScancoReadImageResult>} - result object
 */
async function scancoReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'scanco-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {ScancoWriteImageOptions} options - options object
 *
 * @returns {Promise<ScancoWriteImageResult>} - result object
 */
async function scancoWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'scanco-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {FdfReadImageOptions} options - options object
 *
 * @returns {Promise<FdfReadImageResult>} - result object
 */
async function fdfReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'fdf-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {WasmReadImageOptions} options - options object
 *
 * @returns {Promise<WasmReadImageResult>} - result object
 */
async function wasmReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'wasm-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {WasmWriteImageOptions} options - options object
 *
 * @returns {Promise<WasmWriteImageResult>} - result object
 */
async function wasmWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'wasm-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {WasmZstdReadImageOptions} options - options object
 *
 * @returns {Promise<WasmZstdReadImageResult>} - result object
 */
async function wasmZstdReadImage(webWorker, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.Image },
    ];
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof File) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    const inputs = [
        { type: InterfaceTypes$1.BinaryFile, data: serializedImageFile },
    ];
    const args = [];
    // Inputs
    const serializedImageName = serializedImageFile.path;
    args.push(serializedImageName);
    // Outputs
    const couldReadName = '0';
    args.push(couldReadName);
    const imageName = '1';
    args.push(imageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    const pipelinePath = 'wasm-zstd-read-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldRead: outputs[0]?.data,
        image: outputs[1]?.data,
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {WasmZstdWriteImageOptions} options - options object
 *
 * @returns {Promise<WasmZstdWriteImageResult>} - result object
 */
async function wasmZstdWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'wasm-zstd-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

const imageIoIndex = new Map([
    ['png', [pngReadImage, pngWriteImage]],
    ['meta', [metaReadImage, metaWriteImage]],
    ['tiff', [tiffReadImage, tiffWriteImage]],
    ['nifti', [niftiReadImage, niftiWriteImage]],
    ['jpeg', [jpegReadImage, jpegWriteImage]],
    ['nrrd', [nrrdReadImage, nrrdWriteImage]],
    ['vtk', [vtkReadImage, vtkWriteImage]],
    ['bmp', [bmpReadImage, bmpWriteImage]],
    ['hdf5', [hdf5ReadImage, hdf5WriteImage]],
    ['minc', [mincReadImage, mincWriteImage]],
    ['mrc', [mrcReadImage, mrcWriteImage]],
    ['lsm', [lsmReadImage, lsmWriteImage]],
    ['mgh', [mghReadImage, mghWriteImage]],
    ['bioRad', [bioRadReadImage, bioRadWriteImage]],
    ['gipl', [giplReadImage, giplWriteImage]],
    ['geAdw', [geAdwReadImage, geAdwWriteImage]],
    ['ge4', [ge4ReadImage, ge4WriteImage]],
    ['ge5', [ge5ReadImage, ge5WriteImage]],
    ['gdcm', [gdcmReadImage, gdcmWriteImage]],
    ['scanco', [scancoReadImage, scancoWriteImage]],
    ['fdf', [fdfReadImage, null]],
    ['wasm', [wasmReadImage, wasmWriteImage]],
    ['wasmZstd', [wasmZstdReadImage, wasmZstdWriteImage]]
]);

/**
 * Read an image file format and convert it to the itk-wasm file format
 *
 * @param {webWorker} null | webWorker - Web worker to run the pipeline or null to run it in a new worker
 * @param {File | BinaryFile} serializedImage - Input image serialized in the file format
 * @param {ReadImageOptions} options - options to cast the resulting image type or to only read image metadata
 *
 * @returns {Promise<ReadImageResult>} - result object with the image and the web worker used
 */
async function readImage(webWorker, serializedImage, options = {}) {
    const mimeType = serializedImage.type ?? '';
    const fileName = serializedImage.name ?? serializedImage.path ?? 'fileName';
    const extension = getFileExtension(fileName).toLowerCase();
    let usedWebWorker = webWorker;
    let serializedImageFile = serializedImage;
    if (serializedImage instanceof Blob) {
        const serializedImageBuffer = await serializedImage.arrayBuffer();
        serializedImageFile = { path: serializedImage.name, data: new Uint8Array(serializedImageBuffer) };
    }
    let io = null;
    if (mimeType && mimeToImageIo.has(mimeType)) {
        io = mimeToImageIo.get(mimeType);
    }
    else if (extensionToImageIo.has(extension)) {
        io = extensionToImageIo.get(extension);
    }
    else {
        for (const readerWriter of imageIoIndex.values()) {
            if (readerWriter[0] !== null) {
                let { webWorker: testWebWorker, couldRead, image } = await readerWriter[0](usedWebWorker, { path: serializedImageFile.path, data: serializedImageFile.data.slice() }, { informationOnly: options.informationOnly });
                usedWebWorker = testWebWorker;
                if (couldRead) {
                    if (typeof options !== 'undefined') {
                        image = castImage(image, options);
                    }
                    return { webWorker: usedWebWorker, image };
                }
            }
        }
    }
    if (!io) {
        throw Error('Could not find IO for: ' + fileName);
    }
    const readerWriter = imageIoIndex.get(io);
    const reader = readerWriter[0];
    let { webWorker: testWebWorker, couldRead, image } = await reader(usedWebWorker, serializedImageFile, { informationOnly: options.informationOnly });
    usedWebWorker = testWebWorker;
    if (!couldRead) {
        throw Error('Could not read: ' + fileName);
    }
    if (typeof options !== 'undefined') {
        image = castImage(image, options);
    }
    return { webWorker: usedWebWorker, image };
}

const numberOfWorkers = typeof globalThis.navigator?.hardwareConcurrency === 'number' ? globalThis.navigator.hardwareConcurrency : 6;
const workerPool = new WorkerPool$1(numberOfWorkers, readImage);
async function readImageFileSeries(fileList, options) {
    let zSpacing = 1.0;
    let zOrigin = 0.0;
    let sortedSeries = false;
    if (typeof options === 'object') {
        if (typeof options.zSpacing !== 'undefined') {
            zSpacing = options.zSpacing;
        }
        if (typeof options.zOrigin !== 'undefined') {
            zOrigin = options.zOrigin;
        }
        if (typeof options.sortedSeries !== 'undefined') {
            sortedSeries = options.sortedSeries;
        }
    }
    const fetchFileDescriptions = Array.from(fileList, async function (file) {
        return await file.arrayBuffer().then(function (arrayBuffer) {
            const fileDescription = {
                name: file.name,
                type: file.type,
                data: arrayBuffer
            };
            return fileDescription;
        });
    });
    const fileDescriptions = await Promise.all(fetchFileDescriptions);
    if (!sortedSeries) {
        fileDescriptions.sort((a, b) => {
            if (a.name < b.name) {
                return -1;
            }
            if (a.name > b.name) {
                return 1;
            }
            return 0;
        });
    }
    const taskArgsArray = [];
    for (let index = 0; index < fileDescriptions.length; index++) {
        taskArgsArray.push([fileDescriptions[index].data, fileDescriptions[index].name]);
    }
    const results = await workerPool.runTasks(taskArgsArray).promise;
    const images = results.map((result) => {
        const image = result.image;
        image.imageType.dimension = 3;
        image.size.push(1);
        image.spacing.push(zSpacing);
        image.origin.push(zOrigin);
        image.direction = new Float64Array(9);
        image.direction.fill(0.0);
        image.direction[0] = 1.0;
        image.direction[4] = 1.0;
        image.direction[8] = 1.0;
        return image;
    });
    let stacked = stackImages(images);
    if (typeof options === 'object' && (typeof options.componentType !== 'undefined' || typeof options.pixelType !== 'undefined')) {
        stacked = castImage(stacked, options);
    }
    return { image: stacked, webWorkerPool: workerPool };
}

/**
 * Write an itk-wasm Image converted to an serialized image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {WriteImageOptions} options - options object
 *
 * @returns {Promise<WriteImageResult>} - result object
 */
async function writeImage(webWorker, image, serializedImage, options = {}) {
    let inputImage = image;
    if (typeof options.componentType !== 'undefined' || typeof options.pixelType !== 'undefined') {
        inputImage = castImage(image, options);
    }
    const mimeType = options.mimeType;
    const extension = getFileExtension(serializedImage).toLowerCase();
    let usedWebWorker = webWorker;
    let io = null;
    if (typeof mimeType !== 'undefined' && mimeToImageIo.has(mimeType)) {
        io = mimeToImageIo.get(mimeType);
    }
    else if (extensionToImageIo.has(extension)) {
        io = extensionToImageIo.get(extension);
    }
    else {
        for (const readerWriter of imageIoIndex.values()) {
            if (readerWriter[1] !== null) {
                let { webWorker: testWebWorker, couldWrite, serializedImage: serializedImageBuffer } = await readerWriter[1](usedWebWorker, copyImage(inputImage), serializedImage, options);
                usedWebWorker = testWebWorker;
                if (couldWrite) {
                    return { webWorker: usedWebWorker, serializedImage: serializedImageBuffer };
                }
            }
        }
    }
    if (!io) {
        throw Error('Could not find IO for: ' + serializedImage);
    }
    const readerWriter = imageIoIndex.get(io);
    const writer = readerWriter[1];
    let { webWorker: testWebWorker, couldWrite, serializedImage: serializedImageBuffer } = await writer(usedWebWorker, inputImage, serializedImage, options);
    usedWebWorker = testWebWorker;
    if (!couldWrite) {
        throw Error('Could not write: ' + serializedImage);
    }
    const result = {
        webWorker: usedWebWorker,
        serializedImage: serializedImageBuffer
    };
    return result;
}

// Generated file. To retain edits, remove this comment.
/**
 * Write an itk-wasm file format converted to an image file format
 *
 * @param {Image} image - Input image
 * @param {string} serializedImage - Output image serialized in the file format.
 * @param {FdfWriteImageOptions} options - options object
 *
 * @returns {Promise<FdfWriteImageResult>} - result object
 */
async function fdfWriteImage(webWorker, image, serializedImage, options = {}) {
    const desiredOutputs = [
        { type: InterfaceTypes$1.JsonCompatible },
        { type: InterfaceTypes$1.BinaryFile, data: { path: serializedImage, data: new Uint8Array() } },
    ];
    const inputs = [
        { type: InterfaceTypes$1.Image, data: image },
    ];
    const args = [];
    // Inputs
    const imageName = '0';
    args.push(imageName);
    // Outputs
    const couldWriteName = '0';
    args.push(couldWriteName);
    const serializedImageName = serializedImage;
    args.push(serializedImageName);
    // Options
    args.push('--memory-io');
    if (typeof options.informationOnly !== "undefined") {
        options.informationOnly && args.push('--information-only');
    }
    if (typeof options.useCompression !== "undefined") {
        options.useCompression && args.push('--use-compression');
    }
    const pipelinePath = 'fdf-write-image';
    const { webWorker: usedWebWorker, returnValue, stderr, outputs } = await runPipeline(webWorker, pipelinePath, args, desiredOutputs, inputs, { pipelineBaseUrl: getPipelinesBaseUrl() });
    if (returnValue !== 0 && stderr !== "") {
        throw new Error(stderr);
    }
    const result = {
        webWorker: usedWebWorker,
        couldWrite: outputs[0]?.data,
        serializedImage: outputs[1]?.data,
    };
    return result;
}

export { bioRadReadImage, bioRadWriteImage, bmpReadImage, bmpWriteImage, fdfReadImage, fdfWriteImage, gdcmReadImage, gdcmWriteImage, ge4ReadImage, ge4WriteImage, ge5ReadImage, ge5WriteImage, geAdwReadImage, geAdwWriteImage, getPipelinesBaseUrl, giplReadImage, giplWriteImage, hdf5ReadImage, hdf5WriteImage, jpegReadImage, jpegWriteImage, lsmReadImage, lsmWriteImage, metaReadImage, metaWriteImage, mghReadImage, mghWriteImage, mincReadImage, mincWriteImage, mrcReadImage, mrcWriteImage, niftiReadImage, niftiWriteImage, nrrdReadImage, nrrdWriteImage, pngReadImage, pngWriteImage, readImage, readImageFileSeries, scancoReadImage, scancoWriteImage, setPipelinesBaseUrl, tiffReadImage, tiffWriteImage, vtkReadImage, vtkWriteImage, wasmReadImage, wasmWriteImage, wasmZstdReadImage, wasmZstdWriteImage, writeImage };
//# sourceMappingURL=image-io.js.map
